<?php
///////////////////////////////////////////////////////////////////////////

require_once 'lib/default_dune_plugin_fw.php';
require_once 'lib/default_dune_plugin.php';
require_once 'lib/utils.php';
require_once 'lib/control_factory.php';
require_once 'lib/action_factory.php';
require_once 'lib/list_util.php';

require_once 'utils.php';

require 'cursor.php';
require 'folder.php';
require 'folder_cache.php';
require 'playback_url_cache.php';
require 'video_cache.php';


///////////////////////////////////////////////////////////////////////////

$hd_interface_language = 'english';

///////////////////////////////////////////////////////////////////////////

function update_interface_language()
{
    global $hd_interface_language;

    $v =
        trim(
            shell_exec(
                "grep 'interface_language' /config/settings.properties" . ' | sed "s/^.*= *//"'));

    $lang_saved = $hd_interface_language;
    $hd_interface_language = (isset($v) && strlen($v) > 0) ? $v : 'english';

    return ($lang_saved !== $hd_interface_language) ? 1 : 0;
}

///////////////////////////////////////////////////////////////////////////

function get_interface_language()
{
    global $hd_interface_language;
    return $hd_interface_language;
}

function is_interface_language_russian_or_similar()
{
    return get_interface_language() === 'russian' ||
        get_interface_language() === 'ukrainian';
}

///////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////

class FSUAPlugin extends DefaultDunePlugin implements UserInputHandler
{
    const ID = 'FSUA';
    ///////////////////////////////////////////////////////////////////////
    private $playback_info;
    private $series_list;
    private $series_idx;
    
    private $fav_build_in_progess;

    private static $open_folder_action = array
    (
        GuiAction::handler_string_id => PLUGIN_OPEN_FOLDER_ACTION_ID,
    );

    ///////////////////////////////////////////////////////////////////////

    private static $vod_play_action = array
    (
        GuiAction::handler_string_id => PLUGIN_VOD_PLAY_ACTION_ID,
        GuiAction::caption           => 'Play'
    );

    ///////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////

    private static $catalog_vip = array
    (
        ViewItemParams::icon_path => 'gui_skin://small_icons/folder.aai',
        ViewItemParams::item_layout => HALIGN_LEFT,
        ViewItemParams::icon_valign => VALIGN_CENTER,
        ViewItemParams::icon_dx => 20,
        ViewItemParams::icon_dy => -5,
        ViewItemParams::item_caption_font_size => FONT_SIZE_NORMAL,
        ViewItemParams::item_caption_width => 1550
    );

    public static function get_skin_path($plugin_cookies)
    {
        $skin = isset($plugin_cookies->skin) ? $plugin_cookies->skin : 'default';
        
        $skins = array
        (
            'default' =>  'plugin_file://skins/default/',
            'juniperus' => 'plugin_file://skins/juniperus/'
        );
        
        return $skins[$skin];
    }

    public static function get_main_vip($plugin_cookies)
    {
        $skin = isset($plugin_cookies->skin) ? $plugin_cookies->skin : 'default';
        
        $skins = array
        (
            'default' =>  array
                            (
                                ViewItemParams::icon_scale_factor =>0.75,
                                ViewItemParams::icon_sel_scale_factor =>1,
                                ViewItemParams::icon_path => 'gui_skin://small_icons/folder.aai',
                                ViewItemParams::item_layout => VALIGN_CENTER,
                                ViewItemParams::icon_valign => VALIGN_CENTER,
                                ViewItemParams::item_caption_font_size => FONT_SIZE_NORMAL
                            ),
            'juniperus' =>    array
                            (
                                ViewItemParams::icon_scale_factor =>1.0,        //jun
                                ViewItemParams::icon_sel_scale_factor =>1.2,        //jun
                                ViewItemParams::icon_path => 'gui_skin://small_icons/folder.aai',
                                ViewItemParams::item_layout => VALIGN_CENTER,
                                ViewItemParams::icon_valign => VALIGN_CENTER,
                                ViewItemParams::item_caption_dy => -50,                     //jun
                                ViewItemParams::item_caption_sel_dy => -15,                //jun
                                ViewItemParams::icon_dy => -14,                            //jun
                                ViewItemParams::icon_sel_dy => -14,                         //jun
                                ViewItemParams::item_caption_font_size => FONT_SIZE_NORMAL
                            )
        );
        
        return $skins[$skin];
    }

    public static function get_main_menu_folder_view($plugin_cookies)
    {
        $skin = isset($plugin_cookies->skin) ? $plugin_cookies->skin : 'default';
        
        $skins = array
        (
            'juniperus' =>  array
                            (
                                ViewParams::num_cols => 4,        //jun
                                ViewParams::num_rows => 2,        //jun
                                ViewParams::icon_selection_box_dx => -118,        //jun
                                ViewParams::icon_selection_box_dy => -154,        //jun
                                ViewParams::icon_selection_box_width => 240,        //jun
                                ViewParams::icon_selection_box_height => 240,        //jun
                                ViewParams::paint_sandwich => false,                    //jun
                                ViewParams::sandwich_base => 'gui_skin://special_icons/sandwich_base.aai',
                                ViewParams::sandwich_mask => 'cut_icon://{name=sandwich_mask}',
                                ViewParams::sandwich_cover => 'cut_icon://{name=sandwich_cover}',
                                ViewParams::sandwich_width => 200,            //jun
                                ViewParams::sandwich_height => 200,         //jun
                                ViewParams::sandwich_icon_upscale_enabled => false,        //jun
                                ViewParams::paint_details => false        //jun
                            ),
            'default' =>    array
                            (
                                ViewParams::num_cols => 3,
                                ViewParams::num_rows => 2,
                                ViewParams::icon_selection_box_dx => -100,
                                ViewParams::icon_selection_box_dy => -100,
                                ViewParams::icon_selection_box_width => 324,
                                ViewParams::icon_selection_box_height => 200,
                                ViewParams::paint_sandwich => true,
                                ViewParams::sandwich_base => 'gui_skin://special_icons/sandwich_base.aai',
                                ViewParams::sandwich_mask => 'cut_icon://{name=sandwich_mask}',
                                ViewParams::sandwich_cover => 'cut_icon://{name=sandwich_cover}',
                                ViewParams::sandwich_width => 324,
                                ViewParams::sandwich_height => 180,
                                ViewParams::sandwich_icon_upscale_enabled => false,
                                ViewParams::paint_details => true
                            ),
        );
        
        return $skins[$skin];
    }

    public function __construct()
    {
        $this->fav_build_in_progess = false;
    }

    private static function get_preview_url($v)
    {
        if (!HD::has_attribute($v, 'season_image'))
            return '';

        return $v->season_image;
    }

    private function get_quality_array($cfg)
    {
        // 240, 360, 480, 720 и 1080
        $vquality = array();
        if ($cfg === true)
            $vquality[0] = 'Веселые/Поп-музыка';
        else
            $vquality[0] = 'Веселые/Поп-музыка';
        $vquality[1] = 'Не выбрано';
        $vquality[2] = 'Страшные/Тяжёлая';
        $vquality[3] = 'Динамичные/Танцевальная';
        if ($cfg === true)
            $vquality[4] = 'Для души/Для души';
        else
            $vquality[4] = 'Для души/Для души';

        return $vquality;        
    }
	private function get_poisk_array($cfg)
    {
        // production
        $vpoisk = array();
        if ($cfg === true)
        $vpoisk[0] = 'Видео';
        else
        $vpoisk[0] = 'Видео';
        $vpoisk[1] = 'Не выбрано';
        if ($cfg === true)
            $vpoisk[2] = 'Аудио';
        else
            $vpoisk[2] = 'Аудио';

        return $vpoisk;        
    }
	
	private function get_play_timer_array($cfg)
    {
        // production
        $play_timer_ops = array();
        if ($cfg === true)
        $play_timer_ops[0] = 'Не отмечать позицию и [ПРОСМ]';
        else
        $play_timer_ops[0] = 'Не отмечать позицию и [ПРОСМ]';
        $play_timer_ops[1] = 'Не запоминать позицию';
		$play_timer_ops[1000] = '1 сек.';
		$play_timer_ops[5000] = '5 сек.';
		$play_timer_ops[10000] = '10 сек.';
		$play_timer_ops[15000] = '15 сек.';
		$play_timer_ops[20000] = '20 сек.';
		$play_timer_ops[30000] = '30 сек.';
        if ($cfg === true)
            $play_timer_ops[60000] = '60 сек.';
        else
           $play_timer_ops[60000] = '60 сек.';

        return $play_timer_ops;        
    }
	
	private function get_sorting_array($cfg)
    {
        // sort
        $vsorting = array();
        if ($cfg === true)
        $vsorting[0] = 'по рейтингу';
        else
        $vsorting[0] = 'по рейтингу';
        $vsorting[1] = 'по дате обновления';
        if ($cfg === true)
            $vsorting[2] = 'по году выпуска';
        else
            $vsorting[2] = 'по году выпуска';

        return $vsorting;        
    }
	private function get_production_array($cfg)
    {
        // production
        $vproduction = array();
        if ($cfg === true)
        $vproduction[0] = 'Показывать Аниме и Эротика';
        else				
        $vproduction[0] = 'Показывать Аниме и Эротика';
        $vproduction[1] = 'Не выбрано';
        if ($cfg === true)
            $vproduction[2] = 'Не показывать Аниме и Эротика';
        else
            $vproduction[2] = 'Не показывать Аниме и Эротика';

        return $vproduction;        
    }
	private function get_newview_array($cfg)
    {
        // production
        $nview = array();
        if ($cfg === true)
        $nview[0] = 'Нет';
        else				
        $nview[0] = 'Нет';
        if ($cfg === true)
            $nview[1] = 'Да';
        else
            $nview[1] = 'Да';

        return $nview;        
    }
	private function get_backgroundview_array($cfg)
    {
        $bgview = array();
        if ($cfg === true)
        $bgview[0] = 'Да';
        else				
        $bgview[0] = 'Да';
        if ($cfg === true)
            $bgview[1] = 'Нет';
        else
            $bgview[1] = 'Нет';
        return $bgview;        
    }
	
	private function get_showcoment_array($cfg)
    {
        // production
        $shw_coment = array();
        if ($cfg === true)
        $shw_coment[0] = 'Нет';
        else				
        $shw_coment[0] = 'Нет';
        if ($cfg === true)
            $shw_coment[1] = 'Да';
        else
            $shw_coment[1] = 'Да';

        return $shw_coment;        
    }
	
	private function get_language_array($cfg)
    {
        // language
        $vlanguage = array();
        if ($cfg === true)
        $vlanguage[0] = 'Русский';
        else
        $vlanguage[0] = 'Русский';
        $vlanguage[1] = 'Не выбрано';
        if ($cfg === true)
            $vlanguage[2] = 'Украинский';
        else
            $vlanguage[2] = 'Украинский';

        return $vlanguage;        
    }
	//resolution
	 private function get_resolution_array($cfg)
    {
        // 240, 360, 480, 720 и 1080
        $vresolution = array();
        if ($cfg === true)
            $vresolution[0] = 'HD';
        else
            $vresolution[0] = 'HD';
        $vresolution[1] = 'Не выбрано';
        $vresolution[2] = 'Хорошее';
        if ($cfg === true)
            $vresolution[3] = 'Среднее';
        else
            $vresolution[3] = 'Среднее';

        return $vresolution;        
    }
	 	
    ///////////////////////////////////////////////////////////////////////
    public function get_play_action($media_url, &$plugin_cookies)
    {
        $add_play_action = UserInputHandlerRegistry::create_action(
                        $this, 'play_item');
        $add_play_action['caption'] = (
                is_interface_language_russian_or_similar() ? 'Воспроизвести' : "Play"
                );

        return $add_play_action;

    }

    ///////////////////////////////////////////////////////////////////////
    public function get_item_action_map($media_url, &$plugin_cookies)
    {
        if (file_exists('/tmp/brb.cookies'))
		{
		$add_fav_action = UserInputHandlerRegistry::create_action(
                        $this, 'add_favorite');
        $add_fav_action['caption'] = (
                is_interface_language_russian_or_similar() ? 'Добавить в Избранное' : "Add Favorite"
                );

        $switch_view = UserInputHandlerRegistry::create_action(
                        $this, 'switch_view');
        $switch_view['caption'] = (
                is_interface_language_russian_or_similar() ? "Вид" : 'View mode'
                );
        $view_info = UserInputHandlerRegistry::create_action(
                        $this, 'ex_view_info');
        $view_info['caption'] = (
                is_interface_language_russian_or_similar() ? "Инфо" : 'Info'
                );
		$search_view = UserInputHandlerRegistry::create_action(
                        $this, 'do_search_menu');
        $search_view['caption'] = (is_interface_language_russian_or_similar() ? "Поиск" : 'Search');
		
		$go_page = UserInputHandlerRegistry::create_action(
                        $this, 'do_go_page');
        $go_page['caption'] = (is_interface_language_russian_or_similar() ? "Перейти на страницу" : 'Go page');
		$brbfav_action = UserInputHandlerRegistry::create_action(
								$this, 'brbfav_action');
					$brbfav_action['caption'] = (
								is_interface_language_russian_or_similar() ? 'Добавить в Избранное brb.to / Удалить из Избранное brb.to' : "brbfav_action"
							);
							
					$brbftr_action = UserInputHandlerRegistry::create_action(
											$this, 'brbftr_action');
					$brbftr_action['caption'] = (
								is_interface_language_russian_or_similar() ? 'Добавить в На будущее brb.to / Удалить из На будущее brb.to' : "brbftr_action"
										);

					$menu_items[] = array(
						GuiMenuItemDef::caption => $brbfav_action['caption'],
						GuiMenuItemDef::action => $brbfav_action
					);
					$menu_items[] = array(
						GuiMenuItemDef::caption => $brbftr_action['caption'],
						GuiMenuItemDef::action => $brbftr_action
					);
					
					$popup_addfav_action = ActionFactory::show_popup_menu($menu_items);
					
        $skip_details = isset($plugin_cookies->skip_details) ? $plugin_cookies->skip_details : 'no';

        if ($skip_details === 'yes')
        {
            return array
                (
				GUI_EVENT_KEY_POPUP_MENU => $popup_addfav_action,
                GUI_EVENT_KEY_ENTER => UserInputHandlerRegistry::create_action($this, 'view_series'),
                GUI_EVENT_KEY_PLAY => $this->get_play_action($media_url, $plugin_cookies),
                GUI_EVENT_KEY_A_RED => $switch_view,
                GUI_EVENT_KEY_INFO => $view_info,
				GUI_EVENT_KEY_B_GREEN => $search_view,
                GUI_EVENT_KEY_C_YELLOW => $add_fav_action,
				GUI_EVENT_KEY_D_BLUE => $go_page
            );
            
        }
        else
        {
				return array
					(
					GUI_EVENT_KEY_POPUP_MENU => $popup_addfav_action,
					GUI_EVENT_KEY_ENTER => $view_info,
					GUI_EVENT_KEY_PLAY => $this->get_play_action($media_url, $plugin_cookies),
					GUI_EVENT_KEY_A_RED => $switch_view,
					GUI_EVENT_KEY_B_GREEN => $search_view,
					GUI_EVENT_KEY_C_YELLOW => $add_fav_action,
					GUI_EVENT_KEY_D_BLUE => $go_page
				);
        }
		}
		else
		{
		$add_fav_action = UserInputHandlerRegistry::create_action(
                        $this, 'add_favorite');
        $add_fav_action['caption'] = (
                is_interface_language_russian_or_similar() ? 'Добавить в Избранное' : "Add Favorite"
                );

        $switch_view = UserInputHandlerRegistry::create_action(
                        $this, 'switch_view');
        $switch_view['caption'] = (
                is_interface_language_russian_or_similar() ? "Вид" : 'View mode'
                );
        $view_info = UserInputHandlerRegistry::create_action(
                        $this, 'ex_view_info');
        $view_info['caption'] = (
                is_interface_language_russian_or_similar() ? "Инфо" : 'Info'
                );
		$search_view = UserInputHandlerRegistry::create_action(
                        $this, 'do_search_menu');
        $search_view['caption'] = (is_interface_language_russian_or_similar() ? "Поиск" : 'Search');
		
		$go_page = UserInputHandlerRegistry::create_action(
                        $this, 'do_go_page');
        $go_page['caption'] = (is_interface_language_russian_or_similar() ? "Перейти на страницу" : 'Go page');
		
        $skip_details = isset($plugin_cookies->skip_details) ? $plugin_cookies->skip_details : 'no';

        if ($skip_details === 'yes')
        {
            return array
                (
                GUI_EVENT_KEY_ENTER => UserInputHandlerRegistry::create_action($this, 'view_series'),
                GUI_EVENT_KEY_PLAY => $this->get_play_action($media_url, $plugin_cookies),
                GUI_EVENT_KEY_A_RED => $switch_view,
                GUI_EVENT_KEY_INFO => $view_info,
				GUI_EVENT_KEY_B_GREEN => $search_view,
                GUI_EVENT_KEY_C_YELLOW => $add_fav_action,
				GUI_EVENT_KEY_D_BLUE => $go_page
            );
            
        }
        else
        {
				return array
					(
					GUI_EVENT_KEY_ENTER => $view_info,
					GUI_EVENT_KEY_PLAY => $this->get_play_action($media_url, $plugin_cookies),
					GUI_EVENT_KEY_A_RED => $switch_view,
					GUI_EVENT_KEY_B_GREEN => $search_view,
					GUI_EVENT_KEY_C_YELLOW => $add_fav_action,
					GUI_EVENT_KEY_D_BLUE => $go_page
				);
        }		
		}
		
    }
	
    ///////////////////////////////////////////////////////////////////////
    public function get_search_action_map($media_url, &$plugin_cookies)
    {
        $perform_search_action = UserInputHandlerRegistry::create_action(
                        $this, 'perform_search');
        $perform_search_action['caption'] = (
                is_interface_language_russian_or_similar() ? 'Выполнить поиск' : "Perform Search"
                );
        $add_search_action = UserInputHandlerRegistry::create_action(
                        $this, 'add_search');
        $add_search_action['caption'] = (
                is_interface_language_russian_or_similar() ? 'Добавить поиск' : "Add Search"
                );
        $remove_search_action = UserInputHandlerRegistry::create_action(
                        $this, 'remove_search');
        $remove_search_action['caption'] = (
                is_interface_language_russian_or_similar() ? "Удалить" : 'Remove'
                );
        $move_search_top_action = UserInputHandlerRegistry::create_action(
                        $this, 'move_search_top');
        $move_search_top_action['caption'] = (
                is_interface_language_russian_or_similar() ? "Переместить вверх" : 'Move to top'
                );
		$go_ex = UserInputHandlerRegistry::create_action(
                        $this, 'ex_start');
		$go_ex['caption'] = (
                is_interface_language_russian_or_similar() ? 'Перейли в плагин EX.UA' : "go EX"
                );
        $menu_items[] = array(
            GuiMenuItemDef::caption => $move_search_top_action['caption'],
           GuiMenuItemDef::action => $move_search_top_action
        );
        $menu_items[] = array(
            GuiMenuItemDef::caption => $remove_search_action['caption'],
            GuiMenuItemDef::action => $remove_search_action
        );
        $popup_search_action = ActionFactory::show_popup_menu($menu_items);
        $do_search_action = UserInputHandlerRegistry::create_action($this, 'do_search');
        return array
            (
            GUI_EVENT_KEY_ENTER => $do_search_action,
			GUI_EVENT_KEY_B_GREEN => $go_ex,
            GUI_EVENT_KEY_POPUP_MENU => $popup_search_action
        );
    }

    ///////////////////////////////////////////////////////////////////////

    public function get_fav_action_map($media_url, &$plugin_cookies)
    {
        
		$remove_fav_action = UserInputHandlerRegistry::create_action(
                        $this, 'remove_fav');
        $remove_fav_action['caption'] = (
                is_interface_language_russian_or_similar() ? "Удалить" : 'Remove'
                );
        $move_fav_top_action = UserInputHandlerRegistry::create_action(
                        $this, 'move_fav_top');
        $move_fav_top_action['caption'] = (
                is_interface_language_russian_or_similar() ? "Переместить вверх" : 'Move to top'
                );
        $switch_view = UserInputHandlerRegistry::create_action(
                        $this, 'switch_view');
        $switch_view['caption'] = (
                is_interface_language_russian_or_similar() ? "Вид" : 'View mode'
                );
        $view_info = UserInputHandlerRegistry::create_action(
                        $this, 'ex_view_info');
        $view_info['caption'] = (
                is_interface_language_russian_or_similar() ? "Инфо" : 'Info'
                );
       $menu_items[] = array(
            GuiMenuItemDef::caption => $move_fav_top_action['caption'],
            GuiMenuItemDef::action => $move_fav_top_action
        );
        $menu_items[] = array(
            GuiMenuItemDef::caption => $remove_fav_action['caption'],
            GuiMenuItemDef::action => $remove_fav_action
        );
        $popup_search_action = ActionFactory::show_popup_menu($menu_items);

        $skip_details = isset($plugin_cookies->skip_details) ? $plugin_cookies->skip_details : 'no';

        if ($skip_details === 'yes')
        {
            return array
                (
                GUI_EVENT_KEY_ENTER => UserInputHandlerRegistry::create_action($this, 'view_series'),
                GUI_EVENT_KEY_PLAY => $this->get_play_action($media_url, $plugin_cookies),
                GUI_EVENT_KEY_A_RED => $switch_view,
                GUI_EVENT_KEY_INFO => $view_info,
                GUI_EVENT_KEY_POPUP_MENU => $popup_search_action
            );
        }
        else
        {
            // if (preg_match("/\/video\/clips\//i",$media_url))
			// {
				// return array
					// (
				// GUI_EVENT_KEY_ENTER => UserInputHandlerRegistry::create_action($this, 'view_series'),
                // GUI_EVENT_KEY_PLAY => $this->get_play_action($media_url, $plugin_cookies),
                // GUI_EVENT_KEY_A_RED => $switch_view,
                // GUI_EVENT_KEY_INFO => $view_info,
                // GUI_EVENT_KEY_POPUP_MENU => $popup_search_action
				// );
			// }
			// else
			// {
				return array
					(
					GUI_EVENT_KEY_ENTER => $view_info,
					GUI_EVENT_KEY_PLAY => $this->get_play_action($media_url, $plugin_cookies),
					GUI_EVENT_KEY_A_RED => $switch_view,
					GUI_EVENT_KEY_POPUP_MENU => $popup_search_action
				);
			//}
        }
    }

    ///////////////////////////////////////////////////////////////////////
    public function get_video_action_map($media_url, &$plugin_cookies) 
    {
	$parent_dlg = UserInputHandlerRegistry::create_action(
                        $this, 'parent_dialog');
	$go_ex = UserInputHandlerRegistry::create_action(
                        $this, 'ex_start');
    $go_ex['caption'] = (
                is_interface_language_russian_or_similar() ? 'Перейли в плагин EX.UA' : "go EX"
                );

	return array
            (
			GUI_EVENT_KEY_B_GREEN => $go_ex,
            GUI_EVENT_KEY_ENTER => $parent_dlg        
			);
    }

    ///////////////////////////////////////////////////////////////////////
    public function get_main_menu_action_map($media_url, &$plugin_cookies)
    {
		$ver = file_get_contents(dirname(__FILE__).'/VERSION');
		$ver = str_replace("﻿version=", "", $ver);
		$ver = str_replace('date=', "[", $ver);
		$ver = str_replace(" 00:00", "]", $ver);
		$ver = str_replace("\n", " ", $ver);
		
		$aria_inf = UserInputHandlerRegistry::create_action(
						$this, 'aria_inf');
		$aria_inf['caption'] = (is_interface_language_russian_or_similar() ? 'Информация о закачке (aria)' : "aria");	
		
		$setup_view = UserInputHandlerRegistry::create_action(
                        $this, 'setup');
        $setup_view['caption'] = (is_interface_language_russian_or_similar() ? "Настройки" : 'Settings');
		
		$info_release = UserInputHandlerRegistry::create_action(
                        $this, 'info_release');
        $info_release['caption'] = (is_interface_language_russian_or_similar() ? "Изменения в v$ver" : 'Info Release');
		
		$view_sorting = UserInputHandlerRegistry::create_action(
                        $this, 'setup_sorting');
        $view_sorting['caption'] = (is_interface_language_russian_or_similar() ? "Настройки сортировок каталогов" : 'Settings sorting');
		
		$login_action = UserInputHandlerRegistry::create_action(
                        $this, 'login_action');
        $login_action['caption'] = (
                is_interface_language_russian_or_similar() ? "Логин brb.to" : 'Login brb.to'
                );
				
		$menu_items[] = array(
            GuiMenuItemDef::caption => $login_action['caption'],
            GuiMenuItemDef::action => $login_action
        );
		
		$popup_action = ActionFactory::show_popup_menu($menu_items);
        return array
            (
			GUI_EVENT_KEY_POPUP_MENU => $popup_action,
            GUI_EVENT_KEY_ENTER => self::$open_folder_action,
			GUI_EVENT_KEY_B_GREEN => $setup_view,
            GUI_EVENT_KEY_C_YELLOW => $info_release,
			GUI_EVENT_KEY_D_BLUE => $view_sorting,
			GUI_EVENT_KEY_INFO => $aria_inf,
        );
    }

    ///////////////////////////////////////////////////////////////////////
    public function handle_user_input(&$user_input, &$plugin_cookies)
    {
        //hd_print('--> Entry handler: handle_user_input:');
        foreach ($user_input as $key => $value)
            //hd_print("  $key => $value");
        update_interface_language();
		$add_fav = UserInputHandlerRegistry::create_action($this, 'add_favorite');
        $remove_fav = UserInputHandlerRegistry::create_action($this, 'remove_fav');
        $do_search_action = UserInputHandlerRegistry::create_action($this, 'do_search');
        $do_new_search_action = UserInputHandlerRegistry::create_action($this, 'do_new_search');
        $perform_new_search_action = UserInputHandlerRegistry::create_action($this, 'perform_new_search');
        $do_add_search_action = UserInputHandlerRegistry::create_action($this, 'do_add_search');
		$do_search_menu = UserInputHandlerRegistry::create_action($this, 'do_search_menu');
		$do_sort_menu = UserInputHandlerRegistry::create_action($this, 'series_sorting');
		$do_similar_menu = UserInputHandlerRegistry::create_action($this, 'series_similar');
		$do_other_works_menu = UserInputHandlerRegistry::create_action($this, 'other_works');
		$do_other_fav_menu = UserInputHandlerRegistry::create_action($this, 'brbfav_action');
		$do_other_ftr_menu = UserInputHandlerRegistry::create_action($this, 'brbftr_action');
		$do_other_minus_menu = UserInputHandlerRegistry::create_action($this, 'minus_menu_action');
		$do_other_plus_menu = UserInputHandlerRegistry::create_action($this, 'plus_menu_action');
		$do_other_cancel_menu = UserInputHandlerRegistry::create_action($this, 'plus_cancel_action');
        $do_mark_viewed = UserInputHandlerRegistry::create_action($this, 'mark_viewed');
        $add_fav_complite = UserInputHandlerRegistry::create_action($this, 'add_fav_complite');
        $del_fav_complite = UserInputHandlerRegistry::create_action($this, 'del_fav_complite');
		$brb_fav_complite = UserInputHandlerRegistry::create_action($this, 'brb_fav_complite');
		$brb_ftr_complite = UserInputHandlerRegistry::create_action($this, 'brb_ftr_complite');
		$brb_plus_complite = UserInputHandlerRegistry::create_action($this, 'brb_plus_complite');
		$brb_cancel_complite = UserInputHandlerRegistry::create_action($this, 'brb_cancel_complite');
		$brb_minus_complite = UserInputHandlerRegistry::create_action($this, 'brb_minus_complite');
        $just_play = UserInputHandlerRegistry::create_action($this, 'just_play');
		$pin_apply = UserInputHandlerRegistry::create_action($this, 'pin_apply');
		$pin_control = UserInputHandlerRegistry::create_action($this, 'pin_control');


        if (isset($user_input->control_id))
        {
            $control_id = $user_input->control_id;
            switch ($control_id)
            {
			case 'postroll_watched':
				$id = $user_input->plugin_vod_id;
				$init_idx = $user_input->plugin_vod_series_ndx;
                if ($this->series_idx != -1 && !isset($user_input->playback_stop_pressed))
                    $init_idx = $this->series_idx;
				$play_timer = isset($plugin_cookies->play_timer) ? $plugin_cookies->play_timer : 5000;
				
				if ($play_timer == 0)
				{
					$viewed_key = $id . '.viewed';
					if (!isset($plugin_cookies->$viewed_key))
						$plugin_cookies->$viewed_key = '';

					$series_url = $this->series_list[$init_idx];

					$v = $plugin_cookies->$viewed_key;
					
					
				}
				
				elseif ($play_timer == 1)
				{
					$viewed_key = $id . '.viewed';
					if (!isset($plugin_cookies->$viewed_key))
						$plugin_cookies->$viewed_key = '';

					$series_url = $this->series_list[$init_idx];

					$v = $plugin_cookies->$viewed_key;
					
					if (!preg_match("/\.(?:mp3|wav|og(?:g|a)|flac|midi?|rm|m4a|aac|wma|mka|ape)$/i",$v))
						{
						$plugin_cookies->$viewed_key = ListUtil::add_item($v, $series_url);
						}
				}
				
				else 
				{
					if (isset($this->playback_info))
					{
						$movie_id = $id;
						$series_url = $this->series_list[$init_idx];
						$main_key = $movie_id . '.' . $series_url;
						
						if (!preg_match("/\.(?:mp3|wav|og(?:g|a)|flac|midi?|rm|m4a|aac|wma|mka|ape)$/i",$main_key))
							{
							$plugin_cookies->$main_key = $this->playback_info;
							unset($this->playback_info);
							}
					}
				}
				
                $this->series_idx = -1;
				break;

			case 'content_timer': // Watch has started.
				$id = $user_input->plugin_vod_id;
				$init_idx = $user_input->plugin_vod_series_ndx;
                $this->series_idx = $init_idx;
				
				$movie_id = $id;
				$series_url = $this->series_list[$init_idx];
				$main_key = $movie_id . '.' . $series_url;
				if(!isset($this->playback_info))
				{
					$ret = shell_exec('cat /tmp/run/ext_command.state | grep playback_duration');
					preg_match('/playback_duration = (\S*)/', $ret, $matches);
					$this->playback_info = 0 . '|' . $matches[1];
				}
				else
				{
					$t = explode('|', $this->playback_info);
					if (intval($t[1]) < 0)
					{
						$ret = shell_exec('cat /tmp/run/ext_command.state | grep playback_duration');
						preg_match('/playback_duration = (\S*)/', $ret, $matches);
						$this->playback_info = 0 . '|' . $matches[1];
					}
				}
				
				$ret = shell_exec('cat /tmp/run/ext_command.state | grep playback_speed');
				preg_match('/playback_speed = (\S*)/', $ret, $matches);
				
				if (intval($matches[1]) > 0)
				{
					$ret = shell_exec('cat /tmp/run/ext_command.state | grep playback_position');
					preg_match('/playback_position = (\S*)/', $ret, $matches);
					if (intval($matches[1]) > 0 )
					{
						$t = explode('|', $this->playback_info);
						$this->playback_info = $matches[1] . '|' . $t[1];
					}
				}
				
				$actions[GUI_EVENT_PLAYBACK_STOP] =
					UserInputHandlerRegistry::create_action($this, 'postroll_watched');
											   
				$actions[GUI_EVENT_TIMER] =
					UserInputHandlerRegistry::create_action($this, 'content_timer');

				$play_timer = isset($plugin_cookies->play_timer) ? $plugin_cookies->play_timer : 5000;
				
				$data = array
						(
							ChangeBehaviourActionData::actions => $actions,
							ChangeBehaviourActionData::timer => array(GuiTimerDef::delay_ms => $play_timer),
						);

				$params = array(
					'handler_id' => $this->get_handler_id(),
					'control_id' => 'timer');
				
				return array
				(
					GuiAction::handler_string_id => CHANGE_BEHAVIOUR_ACTION_ID,
					GuiAction::caption => null,
					GuiAction::data => $data,
					GuiAction::params => $params,
				);
				
				case 'additional':
                    HD::decode_user_data($user_input->parent_media_url, $media_str, $user_data);
					$selection = $user_data->video_id;
					
                    $from_fav = false;
                    $fav_items = isset($plugin_cookies->fav_items) ? $plugin_cookies->fav_items : '';
                    if (ListUtil::is_in_list($fav_items, $user_data->video_id))
                        $from_fav = true;

                    $defs = array();
					 if (!preg_match("/\/audio\//i",$selection))
					{
						ControlFactory::add_custom_close_dialog_and_apply_buffon($defs,'_sort', 'Фильтры по качеству видео', 950, $do_sort_menu);
					}
                    ControlFactory::add_custom_close_dialog_and_apply_buffon
                    (
                        $defs,
                        '_favorites',
                         ($from_fav ? 'Удалить из Избранного' : 'Добавить в Избранное'),
                          950,
                          ($from_fav ? $remove_fav : $add_fav)
                    );
                    ControlFactory::add_custom_close_dialog_and_apply_buffon($defs,'_search', 'Поиск', 950, $do_search_menu);
					ControlFactory::add_custom_close_dialog_and_apply_buffon($defs,'_similar', 'Похожие материалы', 950, $do_similar_menu);
					 if (!preg_match("/\/audio\//i",$selection))
					{
						ControlFactory::add_custom_close_dialog_and_apply_buffon($defs,'_other_works', 'Другие работы режиссёра/актеров', 950, $do_other_works_menu);
					}
					if (file_exists('/tmp/brb.cookies'))
					{
					$items = $this->parse_season_info($selection, $plugin_cookies);
					HD::decode_user_data($items, $media_str, $udata);
					$myrating = $udata->myrating;
					$brbfav_in = $udata->brbfav_in;
					$brbftr_in = $udata->brbftr_in;
					if (preg_match("/В избранном/i",$brbfav_in))
					{
					ControlFactory::add_custom_close_dialog_and_apply_buffon($defs,'_other_favs', 'Удалить из Избранное brb.to', 950, $do_other_fav_menu);
					}
					else
					{
					ControlFactory::add_custom_close_dialog_and_apply_buffon($defs,'_other_favs', 'Добавить в Избранное brb.to', 950, $do_other_fav_menu);
					}
					if (preg_match("/На будущее/i",$brbftr_in))
					{
					ControlFactory::add_custom_close_dialog_and_apply_buffon($defs,'_other_ftrs', 'Удалить из На будущее brb.to', 950, $do_other_ftr_menu);
					}
					else
					{
					ControlFactory::add_custom_close_dialog_and_apply_buffon($defs,'_other_ftrs', 'Добавить в На будущее brb.to', 950, $do_other_ftr_menu);
					}
					if (preg_match("/1/i",$myrating))
					{
					ControlFactory::add_custom_close_dialog_and_apply_buffon($defs,'_other_cancel', 'Отмена Оценки', 950, $do_other_cancel_menu);
					}
					else
					{
					ControlFactory::add_custom_close_dialog_and_apply_buffon($defs,'_other_plus', 'Нравится +1', 950, $do_other_plus_menu);
					ControlFactory::add_custom_close_dialog_and_apply_buffon($defs,'_other_minus', 'Не нравится -1', 950, $do_other_minus_menu);
					}
					}
                    ControlFactory::add_close_dialog_button($defs, 'Назад', 300);
                    return ActionFactory::show_dialog('Действия', $defs);
					
				case 'search':
                    $new_value = $user_input->{$control_id};
                    // last entered search
                    $plugin_cookies->search = $new_value;
                    $defs = $this->do_get_search_defs($plugin_cookies);
                    return  ActionFactory::reset_controls
                            (
                                $defs,
                                null,
                                1
                            );
//                    return null;
                case 'perform_new_search':
                    $new_value = $plugin_cookies->search;
                    // perform search
                    $search_text = urlencode($new_value);
                    //hd_print("Search: performing search of $search_text");
                    $search_url = HD::encode_user_data(array('ser_feed' => $search_text));
                    return array
                    (
                        GuiAction::handler_string_id => PLUGIN_OPEN_FOLDER_ACTION_ID,
                        GuiAction::data =>
                        array
                        (
                            'media_url' => $search_url
                        )
                    );
                case 'do_new_search':
                    $new_value = $plugin_cookies->search;
                    // search history
                    $search_items = isset($plugin_cookies->search_items) ? $plugin_cookies->search_items : '';
                    $plugin_cookies->search_items = ListUtil::add_item($search_items,$new_value);
                    return ActionFactory::invalidate_folders(array($user_input->parent_media_url), $perform_new_search_action);
				
				case 'do_new_go_page':
                     if (!isset($user_input->selected_media_url))
                        return null;
                    $new_value = $user_input->{$control_id};
                    HD::decode_user_data($user_input->parent_media_url, $media_str, $user_data);
					if (!isset($user_data->subcategory_ref))
					{
					return ActionFactory::show_title_dialog('В поиске перехода на страницу нет!');
					}
					
					$url = $user_data->subcategory_ref;
					if (preg_match("/&GoPage=/i",$url))
					{
					$tmp = explode('&GoPage=', $url);
					$url = $tmp[0];
					}
					$page = $url . "&GoPage=" . $new_value;
					//hd_print("page--->>> $page");
                        return array(
                            GuiAction::handler_string_id => PLUGIN_OPEN_FOLDER_ACTION_ID,
                            GuiAction::data =>
                            array(
                                'media_url' => HD::encode_user_data(
                                                                array(
                                                                    'subcategory_ref' => $page,
                                                                )
                                                           )
                            )
                        );
				case 'do_new_install':
					$add_params = array('name' => 'Ex.ua');
                    $url = 'plugin_installer://http://dl.dropboxusercontent.com/s/2obt8jgj1oa7k0u/dune_plugin_exua_v9.28.tar.gz:::name=Ex.ua&space_needed=2000000&need_confirm_replace=0';
					return ActionFactory::launch_media_url($url,UserInputHandlerRegistry::create_action($this, 'refresh', $add_params));	
                case 'do_search':
                    if (!isset($user_input->selected_media_url))
                        return null;
                    HD::decode_user_data($user_input->selected_media_url, $media_str, $user_data);
                    if ($user_data->ser_feed != '')
                        break;

                    $defs = $this->do_get_search_defs($plugin_cookies);

                    return ActionFactory::show_dialog(
                            (is_interface_language_russian_or_similar() ? 'Поиск' : 'Search'),
                            $defs);
                    
                    break;
				case 'do_search_menu':
                    
					HD::decode_user_data($user_input->selected_media_url, $media_str, $user_data);
					
                    $c = VideoCache::get($user_data->video_id);
                    HD::decode_user_data($c, $media_str, $user_data);
                    $title = '';
                    if (HD::has_attribute($user_data, 'season_title'))
                        $title = $user_data->season_title;
                    $plugin_cookies->search = $title;
                    $defs = $this->do_get_search_defs($plugin_cookies);
                    return ActionFactory::show_dialog(
                            (is_interface_language_russian_or_similar() ? 'Поиск' : 'Search'),
                            $defs);
                    
                    break;
				case 'do_go_page':
                    
					HD::decode_user_data($user_input->selected_media_url, $media_str, $user_data);
					
                    $c = VideoCache::get($user_data->video_id);
                    HD::decode_user_data($c, $media_str, $user_data);
                    $title = '';
                    if (HD::has_attribute($user_data, 'season_title'))
                        $title = $user_data->season_title;
                    $plugin_cookies->search = $title;
                    $defs = $this->do_get_go_page_defs($plugin_cookies);
                    return ActionFactory::show_dialog(
                            (is_interface_language_russian_or_similar() ? 'Перейти на страницу' : 'Go page'),
                            $defs);
                    
                    break;
				case 'brbfav_action':
                    
					HD::decode_user_data($user_input->selected_media_url, $media_str, $user_data);
					$video_id = $user_data->video_id;
					$referer = 'http://brb.to' . $video_id;
					
					$id = "";
					$tmp = explode('/i', $video_id);
					if (count($tmp) > 1)
					$id = 'i' . strstr($tmp[1], '-', true);
					
					$url = "http://brb.to/addto/favorites/$id?json";
					$doc = HD::http_get_document_cookie($url, $referer);
					$i_id = $user_input->parent_media_url;
					if (preg_match("/page=favorites/i",$i_id))
					{
					$folder = FolderCache::get(('||{"subcategory_ref":"/myfavourites.aspx?page=favorites","valid_pin":true}'));
					$folder->set_expired();
					return ActionFactory::update_regular_folder(
                                    $this->get_regular_folder_items(('||{"subcategory_ref":"/myfavourites.aspx?page=favorites","valid_pin":true}'), 0, $plugin_cookies), true);
					}
					else
					{
					$folder = FolderCache::get(('||{"subcategory_ref":"/myfavourites.aspx?page=favorites","valid_pin":true}'));
					$folder->set_expired();
					return ActionFactory::invalidate_folders(array($user_input->parent_media_url), $brb_fav_complite);
					}
                    break;
				case 'brbftr_action':
                    
					HD::decode_user_data($user_input->selected_media_url, $media_str, $user_data);
					$video_id = $user_data->video_id;
					$referer = 'http://brb.to' . $video_id;
					
					$id = "";
					$tmp = explode('/i', $video_id);
					if (count($tmp) > 1)
					$id = 'i' . strstr($tmp[1], '-', true);
					
					$url = "http://brb.to/addto/forlater/$id?json";
					$doc = HD::http_get_document_cookie($url, $referer);
					$i_id = $user_input->parent_media_url;
					if (preg_match("/page=forlater/i",$i_id))
					{
					$folder = FolderCache::get(('||{"subcategory_ref":"/myfavourites.aspx?page=forlater","valid_pin":true}'));
					$folder->set_expired();
					return ActionFactory::update_regular_folder(
                                    $this->get_regular_folder_items(('||{"subcategory_ref":"/myfavourites.aspx?page=forlater","valid_pin":true}'), 0, $plugin_cookies), true);
					}
					else
					{
					$folder = FolderCache::get(('||{"subcategory_ref":"/myfavourites.aspx?page=forlater","valid_pin":true}'));
					$folder->set_expired();
					return ActionFactory::invalidate_folders(array($user_input->parent_media_url), $brb_ftr_complite);
					}
                    break;
				case 'plus_menu_action':
                    
					HD::decode_user_data($user_input->selected_media_url, $media_str, $user_data);
					$video_id = $user_data->video_id;
					$referer = 'http://brb.to' . $video_id;
					
					$id = "";
					$tmp = explode('/i', $video_id);
					if (count($tmp) > 1)
					$id = 'i' . strstr($tmp[1], '-', true);
					
					$url = "http://brb.to/vote/item/up/$id?json=1";
					$doc = HD::http_get_document_cookie($url, $referer);
					
					$folder = FolderCache::get(('||{"subcategory_ref":"/myfavourites.aspx?page=irecommended","valid_pin":true}'));
					$folder->set_expired();
					return ActionFactory::invalidate_folders(array($user_input->parent_media_url), $brb_plus_complite);

                    break;
				case 'plus_cancel_action':
                    
					HD::decode_user_data($user_input->selected_media_url, $media_str, $user_data);
					$video_id = $user_data->video_id;
					$referer = 'http://brb.to' . $video_id;
					
					$id = "";
					$tmp = explode('/i', $video_id);
					if (count($tmp) > 1)
					$id = 'i' . strstr($tmp[1], '-', true);
					
					$url = "http://brb.to/vote/item/undo/$id?json=1";
					$doc = HD::http_get_document_cookie($url, $referer);
					
					$folder = FolderCache::get(('||{"subcategory_ref":"/myfavourites.aspx?page=irecommended","valid_pin":true}'));
					$folder->set_expired();
					return ActionFactory::invalidate_folders(array($user_input->parent_media_url), $brb_cancel_complite);

                    break;
				case 'minus_menu_action':
                    
					HD::decode_user_data($user_input->selected_media_url, $media_str, $user_data);
					$video_id = $user_data->video_id;
					$referer = 'http://brb.to' . $video_id;
					
					$id = "";
					$tmp = explode('/i', $video_id);
					if (count($tmp) > 1)
					$id = 'i' . strstr($tmp[1], '-', true);
					
					$url = "http://brb.to/vote/item/down/$id?json=1";
					$doc = HD::http_get_document_cookie($url, $referer);
					
					$folder = FolderCache::get(('||{"subcategory_ref":"/myfavourites.aspx?page=irecommended","valid_pin":true}'));
					$folder->set_expired();
					return ActionFactory::invalidate_folders(array($user_input->parent_media_url), $brb_minus_complite);

                    break;
                case 'skin':
                    $new_value = $user_input->{$control_id};
                    $plugin_cookies->skin = $new_value;
                    shell_exec("cp -f /persistfs/plugins/FS.TO/skins/$new_value/icons/logo.png /persistfs/plugins/FS.TO/skins/logo.png");
                    break;
                case 'restart':
					shell_exec('killall shell');
                    break;
				case 'info_release':
                   
				$post_action = null;
					
						$doc = HD::http_get_document('http://copy.com/8aMn0OK4hrZlxQk4/Dune/update/FS.TO/info.txt');
						$tmp = explode('>>>', $doc);
						$text = $tmp[0];
						$defs = array();
						$texts = explode("\n", $text);
						$texts = array_values($texts);
						foreach($texts as $text)
						{
						$text = str_remove_spec($text);
						ControlFactory::add_label($defs, "", $text);
						}
						ControlFactory::add_custom_close_dialog_and_apply_buffon($defs, 'setup', 'Ok', 150,  $post_action);

						return ActionFactory::show_dialog('Информация об изменениях.', $defs);
                    break;
				case 'aria_stop':
					shell_exec('/ltu/bin/aria-stop');
					return ActionFactory::show_title_dialog("Stop - Aria2c");
                    break;
				case 'pin_apply':
                    if ($user_input->pin2 == '' )
						return null;
					
					$msg = '';
					$action = null;

					$pin = isset($plugin_cookies->pin) ? $plugin_cookies->pin : '0000';

					if ($user_input->pin1 == $pin)
					{
					$plugin_cookies->pin = $user_input->{'pin2'};
					$msg = 'Код изменен!';
					}
					else
					{
					$msg = 'Код не изменен!';
					}
				
					return  ActionFactory::show_title_dialog
							(
								$msg,
								$action
							);
					break;
				case 'pin_control':
                    foreach ($user_input as $key => $value)
					if ($user_input->pin_control == '')
						 return null;
                    HD::decode_user_data($user_input->selected_media_url, $media_str, $user_data);
					$msg = '';
					$action = null;

					$pin = isset($plugin_cookies->pin) ? $plugin_cookies->pin : '0000';
					$view_type = isset($plugin_cookies->view_type) ? $plugin_cookies->view_type : 2;
					if ($user_input->pin_control == $pin)
					{
									$media_url = HD::encode_user_data
                                                (
                                                    array(
                                                        'subcategory_ref' => $user_data->subcategory_ref,
                                                        'valid_pin' => true)
                                                );
                                    return ActionFactory::close_dialog_and_run(ActionFactory::open_folder($media_url));
					}
					else
					{
					
					$msg = 'Код не верный!';
					return ActionFactory::close_dialog_and_run(ActionFactory::show_title_dialog
							(
								$msg,
								$action
							));
					}
				
					break;
				    case 'parent_dialog':
                    HD::decode_user_data($user_input->selected_media_url, $media_str, $user_data);
					$pin = isset($plugin_cookies->pin) ? $plugin_cookies->pin : '0000';
                    $pin_valid=$user_data->subcategory_ref;
					$pin_valid = str_replace("/%D0%AD%D1%80%D0%BE%D1%82%D0%B8%D0%BA%D0%B0/", "xxx", $pin_valid);
										
					if ($pin == "1111")
					{
					$pin_valid = "1";
					}
					if (!preg_match("/xxx/i",$pin_valid))
                    {
                        $media_url = HD::encode_user_data
                                    (
                                        array(
                                            'subcategory_ref' => $user_data->subcategory_ref,
                                            'valid_pin' => true)
                                    );
                        return array
                        (
                        GuiAction::handler_string_id => PLUGIN_OPEN_FOLDER_ACTION_ID,
                        GuiAction::data => array(
                            'media_url' => $media_url
                                )
                         );
                    }

                    $defs = $this->do_get_parent_control_defs();
                    return  ActionFactory::show_dialog
                            (
                                "Родительский контроль",
                                $defs,
                                true
                            );
				case 'pin_dialog':
                    $defs = $this->do_get_pin_control_defs($plugin_cookies);
					
					return  ActionFactory::show_dialog
							(
								"Родительский контроль",
								$defs,
								true
							);
                    break;

				case 'setup':
                    return ActionFactory::open_folder('setup');
					break;
				case 'series_similar':
                    
					if (!isset($user_input->selected_media_url))
                            return null;
                    HD::decode_user_data($user_input->selected_media_url, $media_str, $user_data);
                    $video_id = $user_data->video_id;
					$items = $this->parse_season_info($video_id, $plugin_cookies);
					HD::decode_user_data($items, $media_str, $udata);
					$similar = $udata->similar;
					if ($similar == "")
					{
					return ActionFactory::show_title_dialog('Похожих материалов нет!');
					}
					else
					{
					$similar_id = HD::encode_user_data(array(
					'vid_similar' => $similar));
					
							return array(
								GuiAction::handler_string_id => PLUGIN_OPEN_FOLDER_ACTION_ID,
								GuiAction::data =>
								array(
									'media_url' => $similar_id
								)
							);
                    }
					break;
				case 'other_works':
                    
					HD::decode_user_data($user_input->parent_media_url, $media_str, $user_data);
					$video_id = $user_data->video_id;
					
					$defs = $this->do_get_other_works_control_defs($video_id, $plugin_cookies);
					
					return  ActionFactory::show_dialog
							(
								"Другие работы режиссёра/актеров",
								$defs,
								true
							);
					break;
				case 'setup_sorting':
                    $defs = $this->do_get_setup_sorting_control_defs($plugin_cookies);
                
					return  ActionFactory::show_dialog
							(
								"Параметры сортировок каталогов",
								$defs,
								true
							);
					break;
				case 'login_action':
                    $defs = $this->do_get_setup_login_control_defs($plugin_cookies);
                
					return  ActionFactory::show_dialog
							(
								"Вход на сайт",
								$defs,
								true
							);
					break;
				case 'series_sorting':
                    HD::decode_user_data($user_input->parent_media_url, $media_str, $user_data);
					$video_id = $user_data->video_id;
					
					$defs = $this->do_get_series_sorting_control_defs($video_id, $plugin_cookies);
					
					return  ActionFactory::show_dialog
							(
								"Фильтры качества видео",
								$defs,
								true
							);
					
					break;
				case 'do_login_action':
                    $url = 'plugin_launcher://FS.TO/';
					return ActionFactory::launch_media_url($url);
					break;
                case 'show_main_icon':
                    $new_value = $user_input->{$control_id};
                    $plugin_cookies->main_icon = $new_value;
		            if ($new_value == 'yes')
		            {
			            $plugin_cookies->show_channels_v4 = 'no';
			            $plugin_cookies->show_search_v4 = 'no';
			            $plugin_cookies->show_favorites = 'no';
		            }
		            else
		            {
			            $plugin_cookies->show_channels_v4 = 'yes';
			            $plugin_cookies->show_search_v4 = 'yes';
			            $plugin_cookies->show_favorites = 'yes';
		            }
                    return ActionFactory::reset_controls(
                            $this->do_get_control_defs($plugin_cookies)
                    );
                    break;
                case 'video_quality':
                    $new_value = $user_input->{$control_id};
                    $plugin_cookies->video_quality = $new_value;
                    return ActionFactory::reset_controls(
                            $this->do_get_setup_sorting_control_defs($plugin_cookies)
                    );
					break;
				case 'video_poisk':
                    $new_value = $user_input->{$control_id};
                    $plugin_cookies->video_poisk = $new_value;
                    return ActionFactory::reset_controls(
                            $this->do_get_setup_sorting_control_defs($plugin_cookies)
                    );
				case 'play_timer':
                    $new_value = $user_input->{$control_id};
                    $plugin_cookies->play_timer = $new_value;
                    return ActionFactory::reset_controls(
                            $this->do_get_control_defs($plugin_cookies)
                    );
					break;	
				case 'video_sorting':
                    $new_value = $user_input->{$control_id};
                    $plugin_cookies->video_sorting = $new_value;
                    return ActionFactory::reset_controls(
                            $this->do_get_setup_sorting_control_defs($plugin_cookies)
                    );
					break;
				case 'video_production':
                    $new_value = $user_input->{$control_id};
                    $plugin_cookies->video_production = $new_value;
                    return ActionFactory::reset_controls(
                            $this->do_get_setup_sorting_control_defs($plugin_cookies)
                    );
					break;
				case 'new_view':
                    $new_value = $user_input->{$control_id};
                    $plugin_cookies->new_view = $new_value;
                    return ActionFactory::reset_controls(
                            $this->do_get_control_defs($plugin_cookies)
                    );
				case 'backgr_view':
                    $new_value = $user_input->{$control_id};
                    $plugin_cookies->backgr_view = $new_value;
                    return ActionFactory::reset_controls(
                            $this->do_get_control_defs($plugin_cookies)
                    );
				case 'show_coment':
                    $new_value = $user_input->{$control_id};
                    $plugin_cookies->show_coment = $new_value;
                    return ActionFactory::reset_controls(
                            $this->do_get_control_defs($plugin_cookies)
                    );
					break;
				case 'series_quality':
					HD::decode_user_data($user_input->parent_media_url, $media_str, $user_data);
					$video_id = $user_data->video_id;
                    $new_value = $user_input->{$control_id};
                    
					$s= $new_value;
					if (strlen($s) == 0)
					continue;
						$i = false;
						$ser_qual = "series_quality&". $video_id;
						$plugin_cookies->$ser_qual = ListUtil::add_item($i, $s);
						
                    return ActionFactory::reset_controls(
                            $this->do_get_series_sorting_control_defs($video_id, $plugin_cookies)
                    );
					break;
				case 'series_translation':
					HD::decode_user_data($user_input->parent_media_url, $media_str, $user_data);
					$video_id = $user_data->video_id;
                    $new_value = $user_input->{$control_id};
                    
					$s= $new_value;
					if (strlen($s) == 0)
					continue;
						$i = false;
						$ser_trans = "series_translation&". $video_id;
						$plugin_cookies->$ser_trans = ListUtil::add_item($i, $s);
						return ActionFactory::reset_controls(
								$this->do_get_series_sorting_control_defs($video_id, $plugin_cookies)
                    );
					break;
				case 'series_season':
					HD::decode_user_data($user_input->parent_media_url, $media_str, $user_data);
					$video_id = $user_data->video_id;
                    $new_value = $user_input->{$control_id};
                    
					$s= $new_value;
					if (strlen($s) == 0)
					continue;
						$i = false;
						$ser_season = "series_season&". $video_id;
						$plugin_cookies->$ser_season = ListUtil::add_item($i, $s);
						return ActionFactory::reset_controls(
								$this->do_get_series_sorting_control_defs($video_id, $plugin_cookies)
                    );
					break;
				case 'video_language':
                    $new_value = $user_input->{$control_id};
                    $plugin_cookies->video_language = $new_value;
                    return ActionFactory::reset_controls(
                            $this->do_get_setup_sorting_control_defs($plugin_cookies)
                    );
					break;
				case 'video_resolution':
                    $new_value = $user_input->{$control_id};
                    $plugin_cookies->video_resolution = $new_value;
                    return ActionFactory::reset_controls(
                            $this->do_get_setup_sorting_control_defs($plugin_cookies)
                    );	
                    break;
				case 'user_folder':
                    $new_value = $user_input->{$control_id};
                    $plugin_cookies->user_folder = $new_value;
                    return ActionFactory::reset_controls(
                            $this->do_get_control_defs($plugin_cookies)
                    );
                    break;
				case 'user_login':
                    $new_value = $user_input->{$control_id};
                    $new_value = engleef($new_value); 
                    $plugin_cookies->user_login = $new_value;
                    return ActionFactory::reset_controls(
                            $this->do_get_setup_login_control_defs($plugin_cookies)
                    );
                    break;
				case 'user_pass':
                    $new_value = $user_input->{$control_id};
                    $new_value = engleef($new_value); 
                    $plugin_cookies->user_pass = $new_value;
                    return ActionFactory::reset_controls(
                            $this->do_get_setup_login_control_defs($plugin_cookies)
                    );
                case 'use_osk':
                    $new_value = $user_input->{$control_id};
                    $plugin_cookies->use_osk = $new_value;
                    return ActionFactory::reset_controls(
                            $this->do_get_control_defs($plugin_cookies)
                    );
                    break;
                case 'skip_details':
                    $new_value = $user_input->{$control_id};
                    $plugin_cookies->skip_details = $new_value;
                    return ActionFactory::reset_controls(
                            $this->do_get_control_defs($plugin_cookies)
                    );
                    break;

                case 'buf_time':
                    $new_value = $user_input->{$control_id};
                    $plugin_cookies->buf_time = $new_value;
                    return  ActionFactory::reset_controls
                            (
                                $this->do_get_control_defs($plugin_cookies)
                            );
                    break;
                case 'ex_view_info':
                    if (!isset($user_input->selected_media_url))
                        return null;
                    HD::decode_user_data($user_input->selected_media_url, $media_str, $user_data);
                    
                    if (!HD::has_attribute($user_data, 'video_id'))
                        return null;
                        
                    return array
                        (
                             GuiAction::handler_string_id => PLUGIN_OPEN_FOLDER_ACTION_ID,
                             GuiAction::data => array(
                                    'media_url' => HD::encode_user_data(
                                                        array(
                                                            'info' => '1',
                                                            'video_id' => $user_data->video_id,
                                                            'season_ref' => $user_data->video_id,
                                                            //'season_title' => $user_data->season_title,
                                                        )
                                                    )
                             )
                         );
                    break;
                    
                case 'add_fav_complite':
                    return ActionFactory::show_title_dialog(
                        (
                            is_interface_language_russian_or_similar() ? 'Видео добавлено в избранное' : "Video has been added to favorites"
                        )
                    );
                    break;
                 case 'del_fav_complite':
                    return ActionFactory::show_title_dialog(
                        (
                            is_interface_language_russian_or_similar() ? 'Видео было удалено из избранного' : "Video has been deleted from favorites"
                        )
                    );
                    break;
				case 'brb_fav_complite':
                    return ActionFactory::show_title_dialog(
                        (
                            is_interface_language_russian_or_similar() ? 'Добавлено(Удалено) в Избранное brb.to' : "fav"
                        )
                    );
                    break;	
				case 'brb_plus_complite':
                    return ActionFactory::show_title_dialog(
                        (
                            is_interface_language_russian_or_similar() ? 'Ваша оценка +1' : "+1"
                        )
                    );
                    break;	
				case 'brb_cancel_complite':
                    return ActionFactory::show_title_dialog(
                        (
                            is_interface_language_russian_or_similar() ? 'Ваша оценка отменена!' : "0 с"
                        )
                    );
                    break;	
				case 'brb_minus_complite':
                    return ActionFactory::show_title_dialog(
                        (
                            is_interface_language_russian_or_similar() ? 'Ваша оценка -1' : "-1"
                        )
                    );
                    break;	
				case 'brb_ftr_complite':
                    return ActionFactory::show_title_dialog(
                        (
                            is_interface_language_russian_or_similar() ? 'Добавлено(Удалено) в На будущее brb.to' : "ftr"
                        )
                    );
                    break;	
                case 'wget_download':
                    if (!isset($user_input->selected_media_url))
                        return null;
					$folder_s = isset($plugin_cookies->user_folder) ? $plugin_cookies->user_folder : 1;
					$folder_s = '/D/'.$folder_s.'/';
					$folder_s = str_replace('/D//', '/D/', $folder_s);
                    $user_data = $user_input->selected_media_url;
					$tmp = explode('episode_ref":"', $user_data);
					$aria_link = str_remove_spec(strstr($tmp[1], '"', true));
					$tmp = explode('episode_title":"', $user_data);
					$aria_name = str_remove_spec(strstr($tmp[1], '"', true));
					shell_exec("wget -c -t 0 $aria_link -P $folder_s > /tmp/run/aria_fs.log &");
					return ActionFactory::show_title_dialog("$aria_name - добавлен в закачки");
					
                case 'aria_inf':
				
					$lines = file('/tmp/run/aria_fs.log'); 
					$trans_log = $lines[count($lines)-1]; 
					$trans_log = str_remove_spec($trans_log);
					$trans_log = str_replace('#1 SIZE:', 'Загружено/Размер:', $trans_log);
					$trans_log = str_replace(' CN:2', '', $trans_log);
					$trans_log = str_replace('SPD', 'Скорость загрузки', $trans_log);
					$trans_log = str_replace('ETA', 'Осталось', $trans_log);
					$trans_log = str_replace('(OK):download completed.', 'Закачки завершены.', $trans_log);
					return ActionFactory::show_title_dialog($trans_log);
                    break;
				
				case 'aria_download':
					if (!isset($user_input->selected_media_url))
                        return null;
					$folder_s = isset($plugin_cookies->user_folder) ? $plugin_cookies->user_folder : "";
					if (preg_match("/((25[0-5]|2[0-4]\d|[01]?\d\d?)\.){3}(25[0-5]|2[0-4]\d|[01]?\d\d?)/",$folder_s))
					{
						$tmp = explode('/', $folder_s);
						$ip_lan = $tmp[0].'/'.$tmp[1];
						$tmp = explode($ip_lan, $folder_s);
						$folder_lan = $tmp[1];
						$smb_name = isset($plugin_cookies->smb_user) ? $plugin_cookies->smb_user : 'guest';
						$smb_pass = isset($plugin_cookies->smb_pass) ? $plugin_cookies->smb_pass : 'guest';
						if (!file_exists('/tmp/mnt/smb/50')) {
						shell_exec('mkdir /tmp/mnt/smb/50');
						}
						shell_exec('mount -t cifs -o rw,username='.$smb_name.',password='.$smb_pass.' //'.$ip_lan.' /tmp/mnt/smb/50');
						$folder_s ='/tmp/mnt/smb/50/'.$folder_lan.'/' ;
						$folder_s = str_replace('/tmp/mnt/smb/50//','/tmp/mnt/smb/50/',$folder_s);
					}
					else
					{
						$folder_s = '/D/'. $folder_s;
						$folder_s = str_replace('//', '/', $folder_s);
                    }
                    $user_data = $user_input->selected_media_url;
					$tmp = explode('episode_ref":"', $user_data);
					$aria_link = str_remove_spec(strstr($tmp[1], '"', true));
					$tmp = explode('episode_title":"', $user_data);
					$aria_name = str_remove_spec(strstr($tmp[1], '"', true));
					if (file_exists('/ltu/bin/aria2c')) {
					exec("/ltu/bin/aria2c -x2 --file-allocation=none $aria_link --dir=$folder_s > /tmp/run/aria_fs.log &");
					return ActionFactory::show_title_dialog("$aria_name - добавлен в закачки");
					} else {
					return ActionFactory::show_title_dialog('Для скачивания файлов необходимо установить пакет LTU (Linux Term Utils) для Dune');
					}
                    break;
				case 'aria_download_all':
					if (!isset($user_input->selected_media_url))
                        return null;
					
					if (file_exists('/ltu/bin/aria2c')) 
					{
					$folder_s = isset($plugin_cookies->user_folder) ? $plugin_cookies->user_folder : "";
					HD::decode_user_data($user_input->selected_media_url, $media_str, $user_data);
					$title = $user_data->season_title;
					$video_id = $user_data->season_ref;
					$items = $this->parse_season($video_id, $plugin_cookies);
					if (preg_match("/((25[0-5]|2[0-4]\d|[01]?\d\d?)\.){3}(25[0-5]|2[0-4]\d|[01]?\d\d?)/",$folder_s))
					{
						$tmp = explode('/', $folder_s);
						$ip_lan = $tmp[0].'/'.$tmp[1];
						$tmp = explode($ip_lan, $folder_s);
						$folder_lan = $tmp[1];
						$smb_name = isset($plugin_cookies->smb_user) ? $plugin_cookies->smb_user : 'guest';
						$smb_pass = isset($plugin_cookies->smb_pass) ? $plugin_cookies->smb_pass : 'guest';
						if (!file_exists('/tmp/mnt/smb/50')) {
						shell_exec('mkdir /tmp/mnt/smb/50');
						}
						shell_exec('mount -t cifs -o rw,username='.$smb_name.',password='.$smb_pass.' //'.$ip_lan.' /tmp/mnt/smb/50');
						$folder_s ='/tmp/mnt/smb/50/'.$folder_lan.'/'. $title;
						$folder_s = str_replace('/tmp/mnt/smb/50//','/tmp/mnt/smb/50/',$folder_s);
					}
					else
					{
						$folder_s = '/D/'. $folder_s . "/" .$title;
						$folder_s = str_replace('//', '/', $folder_s);
                    }
					$folder_s = str_replace('(', '', $folder_s);
					$folder_s = str_replace(')', '', $folder_s);
					$folder_s = str_replace(' ', '_', $folder_s);
					foreach($items as $v)
						{
							HD::decode_user_data($v, $media_str, $udata);
							$aria_link = $udata->episode_ref;
							$aria_name = $udata->episode_title;
							if (strlen($aria_link) == 0)
							continue;

						exec("/ltu/bin/aria2c -x2 --file-allocation=none $aria_link --dir=$folder_s > /tmp/run/aria_fs.log &");	
						}
										
					return ActionFactory::show_title_dialog("$title - все файлы добавлены в закачки");
					} 
					else 
							{
					return ActionFactory::show_title_dialog('Для скачивания файлов необходимо установить пакет LTU (Linux Term Utils) для Dune');
							}
                    break;
	
                case 'add_favorite':
                    if (!isset($user_input->selected_media_url))
                        return null;
                    if ($user_input->selected_media_url === 'right_button')
                        HD::decode_user_data($user_input->parent_media_url, $media_str, $user_data);
                    else
                        HD::decode_user_data($user_input->selected_media_url, $media_str, $user_data);
		            
                    if (HD::has_attribute($user_data, 'video_id'))
                    	$video_id = $user_data->video_id;
                    else
                        return null;
					$new_view = isset($plugin_cookies->new_view) ? $plugin_cookies->new_view : 1;
					
                    $fav_items = isset($plugin_cookies->fav_items) ? $plugin_cookies->fav_items : '';

                    if (!ListUtil::is_in_list($fav_items, $video_id))
                    {
		                $plugin_cookies->fav_items = ListUtil::add_item($fav_items,$video_id);
                        $plugin_cookies->$video_id = "";
						
						if ($new_view == 0)
						{         
							$items = $this->parse_season_info($video_id, $plugin_cookies);
							HD::decode_user_data($items, $media_str, $udata);
								
								$img = $udata->video_image_url;
								$titl = $udata->title;
								$s= $titl . "*" . $img;
								if (strlen($s) == 0)
									continue;
									
								$i = $plugin_cookies->$video_id;
								$vid_fav = "fav_links&". $video_id;
								$plugin_cookies->$vid_fav = ListUtil::add_item($i, $s);
							
                        }
						else
						{         
							$items = $this->parse_season($video_id, $plugin_cookies);

							foreach($items as $v)
							{
								HD::decode_user_data($v, $media_str, $udata);
								
								$s = $udata->episode_ref;

								if (strlen($s) == 0)
									continue;
									
								$i = $plugin_cookies->$video_id;
								
								$plugin_cookies->$video_id = ListUtil::add_item($i, $s);
							}
                        }
                    if ($user_input->selected_media_url === 'right_button')
                        {
                            return ActionFactory::invalidate_folders(array($user_input->parent_media_url), $add_fav_complite);
                        }
                        else
                        {
                            return ActionFactory::show_title_dialog('Видео добавлено в избранное');
                        }
                    }
                    else
                    {
                        return ActionFactory::show_title_dialog('Видео уже в избранном');
                    }
                    break;
                case 'remove_fav':
                    if (!isset($user_input->selected_media_url))
                        return null;
                    if ($this->fav_build_in_progess)
                        return ActionFactory::show_title_dialog('Подождите окончания построения списка!');
                        
                    if ($user_input->selected_media_url === 'right_button')
                        HD::decode_user_data($user_input->parent_media_url, $media_str, $user_data);
                    else
                        HD::decode_user_data($user_input->selected_media_url, $media_str, $user_data);

                    if (HD::has_attribute($user_data, 'video_id'))
                        $video_id = $user_data->video_id;
                    else if (HD::has_attribute($user_data, 'fav'))
                        $video_id = $user_data->fav;
                    else
                        return null;

                    $fav_items = isset($plugin_cookies->fav_items) ? ListUtil::del_item($plugin_cookies->fav_items, $video_id) : '';
                    $plugin_cookies->fav_items = $fav_items;
                    if (isset($plugin_cookies->$video_id))
                    {
                        unset($plugin_cookies->$video_id);
                    }
					$s = "fav_links&". $video_id;
                    if (isset($plugin_cookies->$s))
                    {
                        unset($plugin_cookies->$s);
                    }
                    //{ Viewed items
                    $s = $video_id . ".viewed";
                    if (isset($plugin_cookies->$s))
                    {
                        unset($plugin_cookies->$s);
                    }
                    //} Viewed items

                    $folder = FolderCache::get('main_menu:fav');
                    $folder->set_expired();
                    if ($user_input->selected_media_url === 'right_button')
                    {
                        return ActionFactory::invalidate_folders(array($user_input->parent_media_url), $del_fav_complite);
                    }
                    else
                    {
                        return ActionFactory::invalidate_folders(array($user_input->parent_media_url), null);
                    }
                    break;
                case 'move_fav_top':
                    if (!isset($user_input->selected_media_url))
                        return null;
                    if ($this->fav_build_in_progess)
                        return ActionFactory::show_title_dialog('Подождите окончания построения списка!');
                    HD::decode_user_data($user_input->selected_media_url, $media_str, $user_data);
                    $video_id = $user_data->video_id;
                    $fav_items = isset($plugin_cookies->fav_items) ? $plugin_cookies->fav_items : '';
                    $plugin_cookies->fav_items = ListUtil::add_item($fav_items,$video_id);
                    return ActionFactory::invalidate_folders(array($user_input->parent_media_url), null);
                    break;

                case 'just_play':
                    return array(
                            GuiAction::handler_string_id => PLUGIN_VOD_PLAY_ACTION_ID,
                            GuiAction::data =>
                                array(
                                    'media_url' => $user_input->selected_media_url
                                )
                    );
                    break;
                case 'play_item':
                    if (!isset($user_input->selected_media_url))
                        return null;
         
                    return ActionFactory::invalidate_folders(array($user_input->selected_media_url, $user_input->parent_media_url), $just_play);

                    break;
		        case 'view_series':
                    if (!isset($user_input->selected_media_url))
                        return null;
                    
                    HD::decode_user_data($user_input->selected_media_url, $media_str, $user_data);

		            $n_series = 0;
		            if (HD::has_attribute($user_data, 'n_series'))
			            $n_series = $user_data->n_series;
	
		            $ndx_series = -1;
		            if (HD::has_attribute($user_data, 'ndx_series'))
			            $ndx_series = $user_data->ndx_series;

		            if ($n_series < 2 || $ndx_series >= 0)
		            {
	                    return array(
        	                GuiAction::handler_string_id => PLUGIN_VOD_PLAY_ACTION_ID,
                	        GuiAction::data =>
                        	array(
	                            'media_url' => $user_input->selected_media_url
        	                )
			            );
		            }
		            else
		            {
	                    return array(
        	                GuiAction::handler_string_id => PLUGIN_OPEN_FOLDER_ACTION_ID,
                	        GuiAction::data =>
                        	array(
	                            'media_url' => $user_input->selected_media_url
        	                )
			            );
                   }
		           break;
				case 'do_new_play':
                    if (!isset($user_input->selected_media_url))
                        return null;
                    
                    HD::decode_user_data($user_input->selected_media_url, $media_str, $user_data);

                        return array(
                            GuiAction::handler_string_id => PLUGIN_OPEN_FOLDER_ACTION_ID,
                            GuiAction::data =>
                            array(
                                'media_url' => HD::encode_user_data(
                                                                array(
                                                                    'season_ref' => $user_data->video_id,
                                                                    'n_series' => $user_data->n_series,
                                                                )
                                                           )
                            )
                        );

                   break;
				case 'do_other_works_play':
                    if (!isset($user_input->selected_media_url))
                        return null;
                    $new_value = $user_input->{$control_id};
                    HD::decode_user_data($user_input->selected_media_url, $media_str, $user_data);
                        return array(
                            GuiAction::handler_string_id => PLUGIN_OPEN_FOLDER_ACTION_ID,
                            GuiAction::data =>
                            array(
                                'media_url' => HD::encode_user_data(
                                                                array(
                                                                    'subcategory_ref' => $new_value,
                                                                )
                                                           )
                            )
                        );

                   break;
				case 'ex_start':
                    $doc = file_get_contents('/tmp/run/installed_plugins.xml');
					if ($doc === false)
					{
						$msg = "Installed plugins file not exist.";
						hd_print($msg);
						throw new DuneException($msg, 1,
						ActionFactory::show_error(false, T::t('title_internal_error')));
						return;
					}
					$xml = simplexml_load_string($doc);
					if ($xml === false)
					{
						$msg = "Cannot parse installed-plugins file.";
						hd_print($msg);
						hd_print("Document: $doc");
						throw new DuneException($msg, 2,
							ActionFactory::show_error(false, T::t('title_internal_error')));
						return;
					}
					if (preg_match('/Ex.ua/', $doc))
                    {
                    $url = 'plugin_launcher://Ex.ua';
					return ActionFactory::launch_media_url($url);
					}
					else
					{
					$defs = $this->do_get_install_defs($plugin_cookies);
                    return ActionFactory::show_dialog(
                            (is_interface_language_russian_or_similar() ? 'Плагин EX.UA не установлен.' : 'Search'),
                            $defs);
					}
                    break;
                case 'remove_search':
                    if (!isset($user_input->selected_media_url))
                        return null;
                    HD::decode_user_data($user_input->selected_media_url, $media_str, $user_data);
                    $ser_feed = urldecode($user_data->ser_feed);
                    //hd_print("removing search: $ser_feed");
                    $search_items = isset($plugin_cookies->search_items) ? ListUtil::del_item($plugin_cookies->search_items, $ser_feed) : '';
                    $plugin_cookies->search_items = $search_items;
                    //hd_print("-- search list: $search_items");
                    return ActionFactory::update_regular_folder(
                                    $this->get_regular_folder_items('main_menu:search', 0, $plugin_cookies), true);
                    break;
                case 'move_search_top':
                    if (!isset($user_input->selected_media_url))
                        return null;
                    HD::decode_user_data($user_input->selected_media_url, $media_str, $user_data);
                    $ser_feed = urldecode($user_data->ser_feed);
                    $search_items = isset($plugin_cookies->search_items) ? $plugin_cookies->search_items : '';
                    $plugin_cookies->search_items = ListUtil::add_item($search_items,$ser_feed);
                    return ActionFactory::update_regular_folder(
                                    $this->get_regular_folder_items('main_menu:search', 0, $plugin_cookies), true);
                    break;
                case 'mark_viewed':
                    //'url_series'
                    if (isset($user_input->selected_media_url))
                    {
                        HD::decode_user_data($user_input->selected_media_url, $media_str, $user_data);
                        if (HD::has_attribute($user_data, 'video_id'))
                            $video_id = $user_data->video_id;
                        else if (HD::has_attribute($user_data, 'season_ref'))
                            $video_id = $user_data->season_ref;
                        
                            if (isset($plugin_cookies->$video_id))
                            {
                                if (HD::has_attribute($user_data, 'url_series'))
                                    $url = $user_data->url_series;
                                else if (HD::has_attribute($user_data, 'episode_ref'))
                                    $url = $user_data->episode_ref;
                                if (!ListUtil::is_in_list($plugin_cookies->$video_id, $url))
                                {
                                    $viewed = $plugin_cookies->$video_id;
                                    $plugin_cookies->$video_id = ListUtil::add_item($viewed,$url);
                                }
                            }
							//{ Viewed items
                            
                            $s = $video_id . ".viewed";
                            if (!isset($plugin_cookies->$s))
                                $plugin_cookies->$s = "";
                            if (HD::has_attribute($user_data, 'url_series'))
                                $url = $user_data->url_series;
                            else if (HD::has_attribute($user_data, 'episode_ref'))
                                $url = $user_data->episode_ref;
                            $key = $video_id . '.' . $url;
							if (isset($plugin_cookies->$key))
								{
									unset($plugin_cookies->$key);
								}
							else
							{
								if (!ListUtil::is_in_list($plugin_cookies->$s, $url))
								{
									$i = $plugin_cookies->$s;       
									$plugin_cookies->$s = ListUtil::add_item($i, $url);
								}
								else
								{
									$i = $plugin_cookies->$s; 
									$plugin_cookies->$s = ListUtil::del_item($i, $url);							
									
									$key = $video_id . '.' . $url;
									if (isset($plugin_cookies->$key))
									{
										unset($plugin_cookies->$key);
									}
								}
							}
                            //} Viewed items

                        
                    }
                    return ActionFactory::update_regular_folder(
                                    $this->get_regular_folder_items($user_input->parent_media_url, 0, $plugin_cookies), true);
                    break;
            }
        }
        // ActionFactory::open_folder($user_input->selected_media_url);
        return array
            (
            GuiAction::handler_string_id => PLUGIN_OPEN_FOLDER_ACTION_ID,
            GuiAction::data => array(
                'media_url' => $user_input->selected_media_url
                    )
             );
    }
    ///////////////////////////////////////////////////////////////////////
	public function do_get_pin_control_defs(&$plugin_cookies)
    {
		$defs = array();
        $pin1 = '';
        $pin2 = '';
        $this->add_text_field($defs,
            'pin1', 'Старый код закрытых разделов:',
            $pin1, 1, 1, 0, 1, 500, 0, false);
        $this->add_text_field($defs,
            'pin2', 'Новый код закрытых разделов:',
            $pin2, 1, 1, 0, 1, 500, 0, false);
        ControlFactory::add_label($defs, '', '');
		$pin_apply = UserInputHandlerRegistry::create_action($this, 'pin_apply');
        ControlFactory::add_custom_close_dialog_and_apply_buffon($defs,'pin_apply', 'ОК', 300, $pin_apply);
        ControlFactory::add_close_dialog_button($defs,
                'Отмена', 300);
        return $defs;
    }
	
	public function do_get_parent_control_defs()
    {	
		$defs = array();
        $pin_control = '';
        $this->add_text_field($defs,
            'pin_control', 'Код закрытых разделов:',
            $pin_control, 1, 1, 0, 1, 500, 0, true);
        
        return $defs;
    }

    protected function add_text_field(&$defs, $name, $title, $initial_value, $numeric, $password, $has_osk, $always_active, $width, $need_confirm = false, $need_apply = false) {
        ControlFactory::add_text_field($defs, $this, null,
                        $name, $title, $initial_value,
                        $numeric, $password, $has_osk, $always_active, $width,
                        $need_confirm, $need_apply);
    }

    ///////////////////////////////////////////////////////////////////////
    protected function add_combobox(&$defs, $name, $title, $initial_value, $value_caption_pairs, $width, $need_confirm = false, $need_apply = false)
    {
        ControlFactory::add_combobox($defs, $this, null,
                        $name, $title, $initial_value, $value_caption_pairs, $width,
                        $need_confirm, $need_apply);
    }

    protected function add_button(&$defs,$name, $title, $caption, $width)
    {
        ControlFactory::add_button($defs, $this, null,
            $name, $title, $caption, $width);
    }

    ///////////////////////////////////////////////////////////////////////
    public function do_get_control_defs(&$plugin_cookies)
    {
        $defs = array();

        $main_icon = isset($plugin_cookies->main_icon) ? $plugin_cookies->main_icon : 'yes';
        $use_osk = isset($plugin_cookies->use_osk) ? $plugin_cookies->use_osk : 'yes';
        $skip_details = isset($plugin_cookies->skip_details) ? $plugin_cookies->skip_details : 'no';
        $buf_time = isset($plugin_cookies->buf_time) ? $plugin_cookies->buf_time : 0;
		$user_folder = isset($plugin_cookies->user_folder) ? $plugin_cookies->user_folder : '';
		$play_timer = isset($plugin_cookies->play_timer) ? $plugin_cookies->play_timer : 5000;
		$new_view = isset($plugin_cookies->new_view) ? $plugin_cookies->new_view : 1;
		$backgr_view = isset($plugin_cookies->backgr_view) ? $plugin_cookies->backgr_view : 0;
		$show_coment = isset($plugin_cookies->show_coment) ? $plugin_cookies->show_coment : 0;
		
		$play_timer_ops = $this->get_play_timer_array(true);
		$nview = $this->get_newview_array(true);
		$bgview = $this->get_backgroundview_array(true);
		$shw_coment = $this->get_showcoment_array(true);
		
		$ver = file_get_contents(dirname(__FILE__).'/VERSION');
		$ver = str_replace("﻿version=", "", $ver);
		$ver = str_replace('date=', "[", $ver);
		$ver = str_replace(" 00:00", "]", $ver);
		$ver = str_replace("\n", " ", $ver);
		
        $show_ops = array();
        if (is_interface_language_russian_or_similar ()) {
            $show_ops['yes'] = 'Да';
            $show_ops['no'] = 'Нет';
        } else {
            $show_ops['yes'] = 'Yes';
            $show_ops['no'] = 'No';
        }
        
        $this->add_button
        (
            $defs,
            'info_release',
            "Версия: $ver",
            'Информация об изменениях',
            0
        );
        
	    $this->add_combobox($defs,
        	       'use_osk',
                'Экранная клавиатура в поиске:',
                $use_osk, $show_ops, 0, true
            );
		
		$this->add_combobox($defs,
                'skip_details',
                'Пропускать описание:',
                $skip_details, $show_ops, 0, true
        );
		
        $this->add_text_field($defs,
                    'user_folder',
                    'Путь к папке для закачки',
                    $user_folder, 0, 0, 0, 1, 400, 0, true
            );

        $show_buf_time_ops = array();
        
        $show_buf_time_ops[0] = '0.0 с.,Оригинальные файлы';
		$show_buf_time_ops[500] = '0.5 с. МР4';
        $show_buf_time_ops[1000] = '1 с. МР4';
        $show_buf_time_ops[2000] = '2 с. МР4';
        $show_buf_time_ops[3000] = '3 с. МР4';
        $show_buf_time_ops[5000] = '5 с. МР4';
        $show_buf_time_ops[10000] = '10 с. МР4';


        $this->add_combobox
        (
            $defs,
            'buf_time',
            'Буферизация/Видео в MP4:',
            $buf_time, $show_buf_time_ops, 0, true
        );
		
		$this->add_combobox
        (
            $defs,
            'play_timer',
            'Частота запоминания позиции:',
            $play_timer, $play_timer_ops, 0, true
        );
		
		$this->add_combobox
        (
            $defs,
            'new_view',
            'Отмечать новые эпизоды [NEW]:',
            $new_view, $nview, 0, true
        );
		
		$this->add_combobox
        (
            $defs,
            'backgr_view',
            'Использовать бекграунд с сайта:',
            $backgr_view, $bgview, 0, true
        );
		
		$this->add_combobox
        (
            $defs,
            'show_coment',
            'Показывать коментарии:',
            $show_coment, $shw_coment, 0, true
        );
		
        $skin_ops = array();

        $skin_ops['default'] = 'По умолчанию';
        $skin_ops['juniperus'] = 'От Juniperus (Пиратская тема)';

        $skin = isset($plugin_cookies->skin) ? $plugin_cookies->skin : 'default';
        
		// $this->add_button
        // (
            // $defs,
            // 'aria_stop',
            // 'Остановить Aria2c (очистить очередь закачек):',
            // 'Stop - Aria2c',
            // 0
        // );
		
		$this->add_button($defs,
            'pin_dialog',
            'Код для закрытых разделов:',
            'Изменить...',
            400
        );

		$this->add_combobox
        (
            $defs,
            'skin',
            'Оформление (требуется перезагрузка):',
            $skin, $skin_ops, 0, true
        );

        $this->add_button
        (
            $defs,
            'restart',
            '',
            'Перезагрузить плеер',
            0
        );

       return $defs;
    }
	public function do_get_setup_sorting_control_defs(&$plugin_cookies)
    {
        $defs = array();
		$video_quality = isset($plugin_cookies->video_quality) ? $plugin_cookies->video_quality : 1;
		$video_poisk = isset($plugin_cookies->video_poisk) ? $plugin_cookies->video_poisk : 1;
		$video_sorting = isset($plugin_cookies->video_sorting) ? $plugin_cookies->video_sorting : 1;
		$video_production = isset($plugin_cookies->video_production) ? $plugin_cookies->video_production : 1;
		$video_language = isset($plugin_cookies->video_language) ? $plugin_cookies->video_language : 1;
		$video_resolution = isset($plugin_cookies->video_resolution) ? $plugin_cookies->video_resolution : 1;
        
		$vquality = $this->get_quality_array(true);
		$vpoisk = $this->get_poisk_array(true);
		$vsorting = $this->get_sorting_array(true);
		$vproduction = $this->get_production_array(true);
		
        $vlanguage = $this->get_language_array(true);
		$vresolution = $this->get_resolution_array(true);

        $this->add_combobox($defs,
                'video_sorting',
                'Cортировать:',
                $video_sorting, $vsorting, 700, true
        );
		$this->add_combobox($defs,
                'video_poisk',
                'Поиск по разделу:',
                $video_poisk, $vpoisk, 700, true
        );
        $this->add_combobox($defs,
                'video_quality',
                'Видео настроение/ Аудио вид:',
                $video_quality, $vquality, 700, true
        );
		
		$this->add_combobox($defs,
                'video_production',
                'Категории Видео:',
                $video_production, $vproduction, 700, true
        );
		$this->add_combobox($defs,
                'video_language',
                'Язык:',
                $video_language, $vlanguage, 700, true
        );
		$this->add_combobox($defs,
                'video_resolution',
                'Качество:',
                $video_resolution, $vresolution, 700, true
        );
		ControlFactory::add_close_dialog_button($defs, 'Ок', 300);
       return $defs;
    }
	public function do_get_setup_login_control_defs(&$plugin_cookies)
    {
        $defs = array();
		
		$user_login = isset($plugin_cookies->user_login) ? $plugin_cookies->user_login : '';
		$user_pass = isset($plugin_cookies->user_pass) ? $plugin_cookies->user_pass : '';
		$user_login = degleef($user_login); 
		$user_pass = degleef($user_pass);
		$this->add_text_field($defs,
                    'user_login',
                    'Логин:',
                    $user_login, 0, 0, 0, 1, 400, 0, true
            );
			
		$this->add_text_field($defs,
                    'user_pass',
                    'Пароль:',
                    $user_pass, 0, 1, 0, 1, 400, 0, true
            );
		ControlFactory::add_label($defs, "", 'Для вызова клавиатуры нажмите синюю "D" на пульте');
		ControlFactory::add_label($defs, "", 'После ввода символов подтверждайте ввод Enter-ом');
		ControlFactory::add_label($defs, "", 'После ввода логина и пароля необходимо перегрузить');
		ControlFactory::add_label($defs, "", 'плагин для доступа к избранному.');
		$do_new_login_action = UserInputHandlerRegistry::create_action($this, 'do_login_action');

        ControlFactory::add_custom_close_dialog_and_apply_buffon($defs,'apply_subscription',
                (is_interface_language_russian_or_similar() ? 'Перегрузить плагин' : 'Reboot plugin'),
                430, $do_new_login_action);
        
      ControlFactory::add_close_dialog_button($defs,
                'Отмена', 430);
       return $defs;
    }
	public function do_get_series_sorting_control_defs($video_id, &$plugin_cookies)
    {
        $defs = array();
		$ser_qual = "series_quality&". $video_id;
				if (isset($plugin_cookies->$ser_qual))
				{
					$series_quality = $plugin_cookies->$ser_qual;
				}
				else
				{
					$series_quality = 'none';
				}
		$ser_season = "series_season&". $video_id;
				if (isset($plugin_cookies->$ser_season))
				{
					$series_season = $plugin_cookies->$ser_season;
				}
				else
				{
					$series_season = 'none';
				}
		$ser_trans = "series_translation&". $video_id;
				if (isset($plugin_cookies->$ser_trans))
				{
					$series_translation = $plugin_cookies->$ser_trans;
				}
				else
				{
					$series_translation = 'none';
				}

		$items = $this->parse_season_info($video_id, $plugin_cookies);
		HD::decode_user_data($items, $media_str, $udata);
		$quality = $udata->quality;
		$translation = $udata->translation;
		$season = $udata->season;
		
		$videos = explode("filter: '", $quality);
		unset($videos[0]);
		$videos = array_values($videos);
		$squality = array();
		$squality["none"] = 'Без фильтра';
		foreach($videos as $video)
				{
					$item = strstr($video, "'", true);
					$squality[$item] = $item;
				}
		$in_doc = '';
		if (!preg_match("/stop/i",$season))
		{
		$in_doc = HD::http_get_document($season);
		}
		$videos = explode("{parent_id: ", $in_doc);
		unset($videos[0]);
		$videos = array_values($videos);
		$sseason = array();
		$sseason["none"] = 'Без фильтра';
		foreach($videos as $video)
				{
					$item = strstr($video, "}", true);
					$tmp = explode("<b>", $video);
					if (count($tmp) > 1)
					$name = strstr($tmp[1], '</b>', true);
					$item = $item . "&&" . $name;
					$sseason[$item] = $name;
				}
		$videos = explode("translate: '", $translation);
		unset($videos[0]);
		$videos = array_values($videos);
		$stranslation = array();
		$stranslation["none"] = 'Без фильтра';
		foreach($videos as $video)
				{
					$item = strstr($video, "'", true);
					
					$tmp = explode("'>", $video);
					if (count($tmp) > 1)
					$name = strstr($tmp[1], '</a>', true);
					
					$stranslation[$item] = $name;
				}

        $this->add_combobox($defs,
                'series_quality',
                'Качество видео:',
                $series_quality, $squality, 850, true
        );
		if (preg_match("/\/cartoonserials\//i",$video_id))
		{
			$this->add_combobox($defs,
					'series_translation',
					'Перевод:',
					$series_translation, $stranslation, 850, true
			);
			$this->add_combobox($defs,
					'series_season',
					'Сезон:',
					$series_season, $sseason, 850, true
			);
		}
		if (preg_match("/\/serials\//i",$video_id))
		{
			$this->add_combobox($defs,
					'series_translation',
					'Перевод:',
					$series_translation, $stranslation, 850, true
			);
			$this->add_combobox($defs,
					'series_season',
					'Сезон:',
					$series_season, $sseason, 850, true
			);
		}
		
       $do_new_play_action = UserInputHandlerRegistry::create_action($this, 'do_new_play');

        ControlFactory::add_custom_close_dialog_and_apply_buffon($defs,'apply_subscription',
                (is_interface_language_russian_or_similar() ? 'Просмотр' : 'Play'),
                300, $do_new_play_action);
        
      ControlFactory::add_close_dialog_button($defs,
                'Отмена', 300);

        return $defs;
    }
	public function do_get_other_works_control_defs($video_id, &$plugin_cookies)
    {
        $defs = array();

		$items = $this->parse_season_info($video_id, $plugin_cookies);
		HD::decode_user_data($items, $media_str, $udata);
		$oth_works = $udata->oth_works;
		$oth_works = str_replace('"', "'", $oth_works);
		$oth_works = str_replace("<a href='#'", "", $oth_works);
		$oth_works = str_replace("<span itemprop='name'>", "<span>", $oth_works);
		$do_other_works_play ='';
		$videos = explode("<a href='", $oth_works);
		unset($videos[0]);
		$videos = array_values($videos);
		$othworks = array();
		foreach($videos as $video)
				{
					$item = strstr($video, "'", true);
					$item = $item . '?sort=rating';
					$tmp = explode('<span>', $video);
					if (count($tmp) > 1)
					$link_capt = strip_tags(strstr($tmp[1], '</a>', true));
					if (preg_match("/\/director\//i",$item))
					{
					$link_capt = $link_capt . ' (режиссёр)';
					}
					if (preg_match("/\/cast\//i",$item))
					{
					$link_capt = $link_capt . ' (актер)';
					}
					$othworks[$item] = $link_capt;
				}

        $this->add_combobox($defs,
                'do_other_works_play',
                'Другие работы режисера/актера:',
                $do_other_works_play, $othworks, 850, true
        );
				
       $do_new_other_works_action = UserInputHandlerRegistry::create_action($this, 'do_other_works_play');

        ControlFactory::add_custom_close_dialog_and_apply_buffon($defs,'apply_subscription',
                (is_interface_language_russian_or_similar() ? 'Показать' : 'Show'),
                300, $do_new_other_works_action);

        return $defs;
    }
    ///////////////////////////////////////////////////////////////////////
    public function do_get_search_defs(&$plugin_cookies)
    {
        $defs = array();
        $search_text = isset($plugin_cookies->search) ? $plugin_cookies->search : '';
        $use_osk = isset($plugin_cookies->use_osk) ? $plugin_cookies->use_osk : 'yes';
        $this->add_text_field($defs,
                'search', "",
                $search_text, 0, 0,
                ($use_osk === 'yes' ? 1 : 0),
                true, 1300, 0, true
        );
        if ($use_osk === 'yes')
            ControlFactory::add_vgap($defs, 500);
        else
            ControlFactory::add_vgap($defs, 50);

        $do_new_search_action = UserInputHandlerRegistry::create_action($this, 'do_new_search');

        ControlFactory::add_custom_close_dialog_and_apply_buffon($defs,'apply_subscription',
                (is_interface_language_russian_or_similar() ? 'Искать' : 'Search'),
                300, $do_new_search_action);
        ControlFactory::add_close_dialog_button($defs,
                'Отмена', 300);

        return $defs;
    }
	
	public function do_get_go_page_defs(&$plugin_cookies)
    {
        $defs = array();
		$text = '';
        $this->add_text_field($defs,
                'do_new_go_page', "Номер страницы: ",
                $text, 1, 0,
                0,
                true, 150, false, false
        );

        $do_new_go_page_action = UserInputHandlerRegistry::create_action($this, 'do_new_go_page');
        ControlFactory::add_custom_close_dialog_and_apply_buffon($defs,'apply_subscription',
                (is_interface_language_russian_or_similar() ? 'Перейти' : 'Go page'),
                300, $do_new_go_page_action);
        ControlFactory::add_close_dialog_button($defs,
                'Отмена', 300);

        return $defs;
    }
	public function do_get_install_defs(&$plugin_cookies)
    {
        $do_new_install_action = UserInputHandlerRegistry::create_action($this, 'do_new_install');
        ControlFactory::add_custom_close_dialog_and_apply_buffon($defs,'apply_subscription',
                (is_interface_language_russian_or_similar() ? 'Установить EX.UA' : 'Install ex.ua'),
                500, $do_new_install_action);
        ControlFactory::add_close_dialog_button($defs,
                'Отмена', 500);
        return $defs;
    }
    ///////////////////////////////////////////////////////////////////////
    public function get_handler_id()
    {
        return self::ID;
    }

    ///////////////////////////////////////////////////////////////////////
    public function get_next_folder_view($media_url, &$plugin_cookies)
    {
        //hd_print('get_next_folder_view');
        $view_type = isset($plugin_cookies->view_type) ? $plugin_cookies->view_type : 2;
        if ($view_type == 3)
            $view_type = 1;
        else
            ++$view_type;
            
        $plugin_cookies->view_type = $view_type;
        
        return $this->get_folder_view($media_url, $plugin_cookies); 
    }

    ///////////////////////////////////////////////////////////////////////

    public function get_folder_view($media_url, &$plugin_cookies)
    {
	    //hd_print('--->>> get_folder_view --->>>' . $media_url);

	    if ($media_url === 'main_menu')
	    {
            return  array
                    (
                        PluginFolderView::view_kind                 => PLUGIN_FOLDER_VIEW_REGULAR,
                        PluginFolderView::multiple_views_supported  => false,
                        PluginFolderView::data                      => array
                        (
                            PluginRegularFolderView::async_icon_loading => false,
                            PluginRegularFolderView::initial_range =>
                                $this->get_main_menu_items($media_url, $plugin_cookies),
                            PluginRegularFolderView::view_params => self::get_main_menu_folder_view($plugin_cookies),
                            PluginRegularFolderView::base_view_item_params => self::get_main_vip($plugin_cookies),
                            PluginRegularFolderView::not_loaded_view_item_params => array(),
                            PluginRegularFolderView::actions => $this->get_main_menu_action_map($media_url, $plugin_cookies)
                        )
                    );
	    }
        else if 
	    (
		    $media_url === 'main_menu:videos' ||
			$media_url === 'main_menu:genres' ||
			$media_url === 'main_menu:audio' ||
			$media_url === 'main_menu:brbfav' ||
		    $media_url === 'main_menu:search'
	    )
        {
		    switch ($media_url) 
		    {
                    case 'main_menu:search':
                        $actions = $this->get_search_action_map($media_url, $plugin_cookies);
                        break;
                    case 'main_menu:videos':
                        $actions = $this->get_video_action_map($media_url, $plugin_cookies);
                        break;
					case 'main_menu:genres':
                        $actions = $this->get_video_action_map($media_url, $plugin_cookies);
                        break;
					case 'main_menu:audio':
                        $actions = $this->get_video_action_map($media_url, $plugin_cookies);
                        break;
					case 'main_menu:brbfav':
                        $actions = $this->get_video_action_map($media_url, $plugin_cookies);
                        break;
		    }

            return  array
                    (
                        PluginFolderView::view_kind                 => PLUGIN_FOLDER_VIEW_REGULAR,
                        PluginFolderView::multiple_views_supported  => false,
                        PluginFolderView::data                      => array
                        (
                            PluginRegularFolderView::async_icon_loading => false,
                            PluginRegularFolderView::initial_range =>
                                $this->get_main_menu_items($media_url, $plugin_cookies),
                            PluginRegularFolderView::view_params => array
                            (
                                ViewParams::num_cols => 1,
                                ViewParams::num_rows => 12
                            ),
                            PluginRegularFolderView::base_view_item_params => self::$catalog_vip,
                            PluginRegularFolderView::not_loaded_view_item_params => array(),
                            PluginRegularFolderView::actions => $actions
                        )
                    );
	    }
	    elseif ($media_url === 'setup')
	    {
                $defs = $this->do_get_control_defs($plugin_cookies);
                $folder_view = array
                    (
                	    PluginControlsFolderView::defs => $defs,
                	    PluginControlsFolderView::initial_sel_ndx => -1,
            	    );
                return array
                    (
                	    PluginFolderView::multiple_views_supported => false,
	                    PluginFolderView::archive => null,
        	            PluginFolderView::view_kind => PLUGIN_FOLDER_VIEW_CONTROLS,
                	    PluginFolderView::data => $folder_view,
            	    );
        }
		elseif ($media_url === 'setup_sorting')
	    {
                $defs = $this->do_get_setup_sorting_control_defs($plugin_cookies);
                $folder_view = array
                    (
                	    PluginControlsFolderView::defs => $defs,
                	    PluginControlsFolderView::initial_sel_ndx => -1,
            	    );
                return array
                    (
                	    PluginFolderView::multiple_views_supported => false,
	                    PluginFolderView::archive => null,
        	            PluginFolderView::view_kind => PLUGIN_FOLDER_VIEW_CONTROLS,
                	    PluginFolderView::data => $folder_view,
            	    );
        }

         $view_type = isset($plugin_cookies->view_type) ? $plugin_cookies->view_type : 2;
         HD::decode_user_data($media_url, $media_str, $user_data);

         if
         (

            HD::has_attribute($user_data, 'category_ref')

         )
         {
             if (HD::has_attribute($user_data, 'ser_feed'))
             {
                $actions = $this->get_item_action_map($media_url, $plugin_cookies);
                $not_loaded_item = self::$catalog_vip;
             }
             else if ($media_url === 'main_menu:fav')
             {
                $actions = $this->get_fav_action_map($media_url, $plugin_cookies);
                $not_loaded_item = self::$catalog_vip;
             }
             else
             {
                $actions = $this->get_video_action_map($media_url, $plugin_cookies);
                $not_loaded_item = array();
             }

            return  array
                    (
                        PluginFolderView::view_kind                 => PLUGIN_FOLDER_VIEW_REGULAR,
                        PluginFolderView::multiple_views_supported  => false,
                        PluginFolderView::data                      => array
                        (
                            PluginRegularFolderView::async_icon_loading => false,
                            PluginRegularFolderView::initial_range =>
                                $this->get_regular_folder_items($media_url, 0, $plugin_cookies),
                            PluginRegularFolderView::view_params => array
                            (
                                ViewParams::num_cols => 1,
                                ViewParams::num_rows => 12
                            ),
                            PluginRegularFolderView::base_view_item_params => self::$catalog_vip,
                            PluginRegularFolderView::not_loaded_view_item_params => $not_loaded_item,
                            PluginRegularFolderView::actions => $actions
                        )
                    );
         }
         else if 
         (
            $media_url === 'main_menu:fav' ||
            HD::has_attribute($user_data, 'subcategory_ref') ||
            HD::has_attribute($user_data, 'ser_feed')
         )
         {

             if ($media_url === 'main_menu:fav')
             {
                $actions = $this->get_fav_action_map($media_url, $plugin_cookies);
             }
             else
             {
                $actions = $this->get_item_action_map($media_url, $plugin_cookies);		
             }

            return array(
                PluginFolderView::view_kind                 => PLUGIN_FOLDER_VIEW_REGULAR,
                PluginFolderView::multiple_views_supported  => true,
                PluginFolderView::data                      => array(
                    PluginRegularFolderView::async_icon_loading => true,
                    PluginRegularFolderView::initial_range =>
                        $this->get_loading_items($plugin_cookies),

                    PluginRegularFolderView::view_params => array(
                        ViewParams::num_cols => ($view_type == 1 ? 5 : 1),
                        ViewParams::num_rows => ($view_type == 1 ? 2 : ($view_type == 2 ? 3 : ($view_type == 4 ? 1 : 10))),
                        ViewParams::paint_details => ($view_type == 4 ? false : true),
                        ViewParams::zoom_detailed_icon => true,
                        ViewParams::paint_item_info_in_details => true,
                        ViewParams::item_detailed_info_font_size => FONT_SIZE_NORMAL,// FONT_SIZE_SMALL,
                        
                        ViewParams::paint_sandwich => (($view_type == 1 || $view_type == 4) ? true : false),
                        ViewParams::sandwich_base => 'gui_skin://special_icons/sandwich_base.aai',
                        ViewParams::sandwich_mask => 'cut_icon://{name=sandwich_mask}',
                        ViewParams::sandwich_cover => 'cut_icon://{name=sandwich_cover}',
                        ViewParams::sandwich_width => 190,
                        ViewParams::sandwich_height => 290,
                        ViewParams::sandwich_icon_upscale_enabled => true,
                        ViewParams::sandwich_icon_keep_aspect_ratio => true,
                    ),

                    PluginRegularFolderView::base_view_item_params => array(
                        ViewItemParams::item_paint_icon => true,
                        ViewItemParams::icon_sel_scale_factor =>1.2,
                        ViewItemParams::icon_path => 'gui_skin://small_icons/movie.aai',
                        ViewItemParams::item_layout => HALIGN_LEFT,
                        ViewItemParams::icon_valign => VALIGN_CENTER,
                        ViewItemParams::icon_dx => 10,
                        ViewItemParams::icon_dy => -5,
                        ViewItemParams::icon_width => ($view_type == 1 ? 190 : ($view_type == 2 ? 150 : ($view_type == 4 ? 250 : 50))),
                        ViewItemParams::icon_height => ($view_type == 1 ? 290 : ($view_type == 2 ? 200 : ($view_type == 4 ? 350 : 50))),
                        ViewItemParams::icon_sel_margin_top => 0,
                        ViewItemParams::item_paint_caption => ($view_type == 1 ? false : true),
                        ViewItemParams::item_caption_width => ($view_type == 2 ? 950 : 1100)
                    ),

                    PluginRegularFolderView::not_loaded_view_item_params => array(
                        ViewItemParams::item_paint_icon => true,
                        ViewItemParams::icon_width => ($view_type == 1 ? 190 : ($view_type == 2 ? 150 : 50)),
                        ViewItemParams::icon_height => ($view_type == 1 ? 290 : ($view_type == 2 ? 200 : 50)),
                        ViewItemParams::icon_path => ($view_type != 3 ? 'gui_skin://large_icons/movie.aai' : 'gui_skin://small_icons/movie.aai')
                    ),

                    PluginRegularFolderView::actions => $actions
                )
            );
         }
         else if
         (
            HD::has_attribute($user_data, 'season_ref') &&
            !HD::has_attribute($user_data, 'info')
         )
         {
                if (isset($plugin_cookies->fav_items))
                $isfav = ListUtil::is_in_list($plugin_cookies->fav_items, $user_data->season_ref);

            
            if ($isfav)
            {
                    $add_action = UserInputHandlerRegistry::create_action(
                                $this, 'mark_viewed');
                    $add_action['caption'] = (
                                is_interface_language_russian_or_similar() ? 'Просмотрено/Не просмотрено' : "Mark as viewed/not viewed"
                            );
					$aria_inf = UserInputHandlerRegistry::create_action(
								$this, 'aria_inf');
					$aria_inf['caption'] = (
								is_interface_language_russian_or_similar() ? 'Информация о закачке (aria)' : "aria"
							);	
					$aria_download = UserInputHandlerRegistry::create_action(
								$this, 'aria_download');
					$aria_download['caption'] = (
								is_interface_language_russian_or_similar() ? 'Скачать файл (aria)' : "aria_download"
							);
					$aria_download_all = UserInputHandlerRegistry::create_action(
											$this, 'aria_download_all');
					$aria_download_all['caption'] = (
								is_interface_language_russian_or_similar() ? 'Скачать все файлы (aria)' : "aria_download_all"
										);
					$aria_stop = UserInputHandlerRegistry::create_action(
											$this, 'aria_stop');
					$aria_stop['caption'] = (
								is_interface_language_russian_or_similar() ? 'Остановить Aria2c (очистить очередь закачек)' : "aria_stop"
										);
					$menu_items[] = array(
						GuiMenuItemDef::caption => $aria_download['caption'],
						GuiMenuItemDef::action => $aria_download
					);
					$menu_items[] = array(
						GuiMenuItemDef::caption => $aria_download_all['caption'],
						GuiMenuItemDef::action => $aria_download_all
					);
					$menu_items[] = array(
						GuiMenuItemDef::caption => $aria_stop['caption'],
						GuiMenuItemDef::action => $aria_stop
					);
					
					$popup_download_action = ActionFactory::show_popup_menu($menu_items);
                $actions = array
                    (
                        GUI_EVENT_KEY_ENTER => $this->get_play_action($media_url, $plugin_cookies),
                        GUI_EVENT_KEY_PLAY => $this->get_play_action($media_url, $plugin_cookies),
                        GUI_EVENT_KEY_B_GREEN => $add_action,
						GUI_EVENT_KEY_D_BLUE => $aria_inf,
						//GUI_EVENT_KEY_C_YELLOW => $aria_download,
						GUI_EVENT_KEY_POPUP_MENU => $popup_download_action
                    );
            }
            else
            {
					    $add_action = UserInputHandlerRegistry::create_action(
                                $this, 'mark_viewed');
						$add_action['caption'] = (
                                is_interface_language_russian_or_similar() ? 'Просмотрено/Не просмотрено' : "Mark as viewed/not viewed"
                            );
						$aria_inf = UserInputHandlerRegistry::create_action(
									$this, 'aria_inf');
						$aria_inf['caption'] = (
									is_interface_language_russian_or_similar() ? 'Информация о закачке (aria)' : "aria"
								);	
						$aria_download = UserInputHandlerRegistry::create_action(
									$this, 'aria_download');
						$aria_download['caption'] = (
									is_interface_language_russian_or_similar() ? 'Скачать файл (aria)' : "aria_download"
								);
						$aria_download_all = UserInputHandlerRegistry::create_action(
												$this, 'aria_download_all');
						$aria_download_all['caption'] = (
									is_interface_language_russian_or_similar() ? 'Скачать все файлы (aria)' : "aria_download_all"
											);
						$aria_stop = UserInputHandlerRegistry::create_action(
												$this, 'aria_stop');
						$aria_stop['caption'] = (
									is_interface_language_russian_or_similar() ? 'Остановить Aria2c (очистить очередь закачек)' : "aria_stop"
											);
						$menu_items[] = array(
							GuiMenuItemDef::caption => $aria_download['caption'],
							GuiMenuItemDef::action => $aria_download
						);
						$menu_items[] = array(
							GuiMenuItemDef::caption => $aria_download_all['caption'],
							GuiMenuItemDef::action => $aria_download_all
						);
						$menu_items[] = array(
							GuiMenuItemDef::caption => $aria_stop['caption'],
							GuiMenuItemDef::action => $aria_stop
						);
						
						$popup_download_action = ActionFactory::show_popup_menu($menu_items);
					$actions = array
						(
							GUI_EVENT_KEY_ENTER => $this->get_play_action($media_url, $plugin_cookies),
							GUI_EVENT_KEY_PLAY => $this->get_play_action($media_url, $plugin_cookies),
							GUI_EVENT_KEY_B_GREEN => $add_action,
							GUI_EVENT_KEY_D_BLUE => $aria_inf,
							//GUI_EVENT_KEY_C_YELLOW => $aria_download,
							GUI_EVENT_KEY_POPUP_MENU => $popup_download_action
						);
            }

            return array
            (
                PluginFolderView::view_kind                 => PLUGIN_FOLDER_VIEW_REGULAR,
                PluginFolderView::multiple_views_supported  => false,
                PluginFolderView::data                      => array(
                    PluginRegularFolderView::async_icon_loading => true,
                    PluginRegularFolderView::initial_range =>
                        $this->get_regular_folder_items($media_url, 0, $plugin_cookies),

                    PluginRegularFolderView::view_params => array(
                        ViewParams::num_cols => 1,
                        ViewParams::num_rows => 10,
                        ViewParams::paint_details => false,
                    ),

                    PluginRegularFolderView::base_view_item_params => array(
                        ViewItemParams::item_paint_icon => true,
                        ViewItemParams::icon_path => 'gui_skin://small_icons/movie.aai',
                    ViewItemParams::item_layout => HALIGN_LEFT,
                       ViewItemParams::icon_valign => VALIGN_CENTER,
                    ViewItemParams::icon_dx => 20,
                    ViewItemParams::icon_dy => -5,
                    ViewItemParams::item_caption_font_size => FONT_SIZE_NORMAL,
                    ViewItemParams::item_caption_width => 1550
                    ),

                    PluginRegularFolderView::not_loaded_view_item_params => array(),

                    PluginRegularFolderView::actions => $actions
                )
            );
         }
         
         if (HD::has_attribute($user_data, 'info'))
         {
            $c = $this->parse_season_info($user_data->video_id, &$plugin_cookies);
            HD::decode_user_data($c, $media_str, $user_data);

            $movie = array(
                PluginMovie::name => $user_data->title,
                PluginMovie::name_original => $user_data->name_original,
                PluginMovie::description => $user_data->description,
                PluginMovie::poster_url => $user_data->video_image_url,
                PluginMovie::length_min => $user_data->duration_min,
                PluginMovie::year => $user_data->year,
                PluginMovie::directors_str => $user_data->director,
                PluginMovie::scenarios_str => '',
                PluginMovie::actors_str => $user_data->actors,
                PluginMovie::genres_str => $user_data->genre,
                PluginMovie::rate_imdb => $user_data->rate_imdb,
                PluginMovie::rate_kinopoisk => '',
                PluginMovie::rate_mpaa => '',
                PluginMovie::country => $user_data->country,
                PluginMovie::budget => '',
				PluginMovie::details =>  $user_data->coments,
				PluginMovie::rate_details   => $user_data->rate_details,
                );
                
            $right_button_action = UserInputHandlerRegistry::create_action(
                        $this, 'additional');
            $right_button_action['caption'] = "Действия";


            $right_button_caption = $right_button_action['caption'];
            
            $movie_folder_view = array
            (
                PluginMovieFolderView::movie => $movie,
                PluginMovieFolderView::has_right_button => true,
                PluginMovieFolderView::right_button_caption => $right_button_caption,
                PluginMovieFolderView::right_button_action => $right_button_action,
                PluginMovieFolderView::has_multiple_series => ($user_data->n_series > 1),
                PluginMovieFolderView::series_media_url => HD::encode_user_data(
                                                                array(
                                                                    'season_ref' => $user_data->video_id,
                                                                    'season_title' => $user_data->title,
                                                                    'n_series' => $user_data->n_series,
                                                                )
                                                           ),												  
													   
			 PluginMovieFolderView::params => array
            (			               
				PluginFolderViewParams::paint_path_box =>true,
                PluginFolderViewParams::paint_content_box_background => true,
               	PluginFolderViewParams::background_url => $user_data->video_image_url_b
            )
			
            );

             return array
                (
                    PluginFolderView::multiple_views_supported => false,
                    PluginFolderView::archive => null,
                    PluginFolderView::view_kind => PLUGIN_FOLDER_VIEW_MOVIE,
                    PluginFolderView::data => $movie_folder_view
                );
         }
         else if (HD::has_attribute($user_data, 'video_id'))
         {
	        $isfav = ListUtil::is_in_list($plugin_cookies->fav_items, $user_data->video_id);
            
            if ($isfav)
            {
                    $add_action = UserInputHandlerRegistry::create_action(
                                $this, 'mark_viewed');
                    $add_action['caption'] = (
                                is_interface_language_russian_or_similar() ? 'Просмотрено/Не просмотрено' : "Mark as viewed/not viewed"
                            );
					$aria_inf = UserInputHandlerRegistry::create_action(
								$this, 'aria_inf');
					$aria_inf['caption'] = (
								is_interface_language_russian_or_similar() ? 'Информация о закачке (aria)' : "aria"
							);	
					$aria_download = UserInputHandlerRegistry::create_action(
								$this, 'aria_download');
					$aria_download['caption'] = (
								is_interface_language_russian_or_similar() ? 'Скачать файл (aria)' : "aria_download"
							);
					$aria_download_all = UserInputHandlerRegistry::create_action(
											$this, 'aria_download_all');
					$aria_download_all['caption'] = (
								is_interface_language_russian_or_similar() ? 'Скачать все файлы (aria)' : "aria_download_all"
										);
					$aria_stop = UserInputHandlerRegistry::create_action(
											$this, 'aria_stop');
					$aria_stop['caption'] = (
								is_interface_language_russian_or_similar() ? 'Остановить Aria2c (очистить очередь закачек)' : "aria_stop"
										);
					$menu_items[] = array(
						GuiMenuItemDef::caption => $aria_download['caption'],
						GuiMenuItemDef::action => $aria_download
					);
					$menu_items[] = array(
						GuiMenuItemDef::caption => $aria_download_all['caption'],
						GuiMenuItemDef::action => $aria_download_all
					);
					$menu_items[] = array(
						GuiMenuItemDef::caption => $aria_stop['caption'],
						GuiMenuItemDef::action => $aria_stop
					);
					
					$popup_download_action = ActionFactory::show_popup_menu($menu_items);
                $actions = array
                    (
                        GUI_EVENT_KEY_ENTER => $this->get_play_action($media_url, $plugin_cookies),
                        GUI_EVENT_KEY_PLAY => $this->get_play_action($media_url, $plugin_cookies),
                        GUI_EVENT_KEY_B_GREEN => $add_action,
						GUI_EVENT_KEY_D_BLUE => $aria_inf,
						//GUI_EVENT_KEY_C_YELLOW => $aria_download,
						GUI_EVENT_KEY_POPUP_MENU => $popup_download_action
                    );
            }
            else
            {
					    $add_action = UserInputHandlerRegistry::create_action(
                                $this, 'mark_viewed');
						$add_action['caption'] = (
                                is_interface_language_russian_or_similar() ? 'Просмотрено/Не просмотрено' : "Mark as viewed/not viewed"
                            );
						$aria_inf = UserInputHandlerRegistry::create_action(
									$this, 'aria_inf');
						$aria_inf['caption'] = (
									is_interface_language_russian_or_similar() ? 'Информация о закачке (aria)' : "aria"
								);	
						$aria_download = UserInputHandlerRegistry::create_action(
									$this, 'aria_download');
						$aria_download['caption'] = (
									is_interface_language_russian_or_similar() ? 'Скачать файл (aria)' : "aria_download"
								);
						$aria_download_all = UserInputHandlerRegistry::create_action(
												$this, 'aria_download_all');
						$aria_download_all['caption'] = (
									is_interface_language_russian_or_similar() ? 'Скачать все файлы (aria)' : "aria_download_all"
											);
						$aria_stop = UserInputHandlerRegistry::create_action(
												$this, 'aria_stop');
						$aria_stop['caption'] = (
									is_interface_language_russian_or_similar() ? 'Остановить Aria2c (очистить очередь закачек)' : "aria_stop"
											);
						$menu_items[] = array(
							GuiMenuItemDef::caption => $aria_download['caption'],
							GuiMenuItemDef::action => $aria_download
						);
						$menu_items[] = array(
							GuiMenuItemDef::caption => $aria_download_all['caption'],
							GuiMenuItemDef::action => $aria_download_all
						);
						$menu_items[] = array(
							GuiMenuItemDef::caption => $aria_stop['caption'],
							GuiMenuItemDef::action => $aria_stop
						);
						
						$popup_download_action = ActionFactory::show_popup_menu($menu_items);
					$actions = array
						(
							GUI_EVENT_KEY_ENTER => $this->get_play_action($media_url, $plugin_cookies),
							GUI_EVENT_KEY_PLAY => $this->get_play_action($media_url, $plugin_cookies),
							GUI_EVENT_KEY_B_GREEN => $add_action,
							GUI_EVENT_KEY_D_BLUE => $aria_inf,
							//GUI_EVENT_KEY_C_YELLOW => $aria_download,
							GUI_EVENT_KEY_POPUP_MENU => $popup_download_action
						);
            }
            
            return array(
                PluginFolderView::view_kind                 => PLUGIN_FOLDER_VIEW_REGULAR,
                PluginFolderView::multiple_views_supported  => false,
                PluginFolderView::data                      => array(
                    PluginRegularFolderView::async_icon_loading => true,
                    PluginRegularFolderView::initial_range =>
                        $this->get_regular_folder_items($media_url, 0, $plugin_cookies),

                    PluginRegularFolderView::view_params => array(
                        ViewParams::num_cols => 1,
                        ViewParams::num_rows => 10,
                        ViewParams::paint_details => false,
   
                    ),

                    PluginRegularFolderView::base_view_item_params => array(
                        ViewItemParams::item_paint_icon => true,
                        ViewItemParams::icon_path => 'gui_skin://small_icons/movie.aai',
	                ViewItemParams::item_layout => HALIGN_LEFT,
       		        ViewItemParams::icon_valign => VALIGN_CENTER,
        	        ViewItemParams::icon_dx => 20,
        	        ViewItemParams::icon_dy => -5,
        	        ViewItemParams::item_caption_font_size => FONT_SIZE_NORMAL,
        	        ViewItemParams::item_caption_width => 1550
                    ),

                    PluginRegularFolderView::not_loaded_view_item_params => array(),

                    PluginRegularFolderView::actions => $actions
                )
            );
         }

	    if ($media_url === 'main_menu:fav')
            $actions = $this->get_fav_action_map($media_url, $plugin_cookies);
	    else
	        $actions = $this->get_item_action_map($media_url, $plugin_cookies);

        return array(
            PluginFolderView::view_kind                 => PLUGIN_FOLDER_VIEW_REGULAR,
            PluginFolderView::multiple_views_supported  => true,
            PluginFolderView::data                      => array(
                PluginRegularFolderView::async_icon_loading => true,
                PluginRegularFolderView::initial_range =>
                    $this->get_loading_items($plugin_cookies),

                PluginRegularFolderView::view_params => array(
                    ViewParams::num_cols => ($view_type == 1 ? 5 : 1),
                    ViewParams::num_rows => ($view_type == 1 ? 2 : ($view_type == 2 ? 3 : 10)),
                    ViewParams::paint_details => true,
                    ViewParams::zoom_detailed_icon => true,
                    ViewParams::paint_item_info_in_details => true,
//                    ViewParams::detailed_icon_scale_factor => 0.5,
                    ViewParams::item_detailed_info_font_size => FONT_SIZE_NORMAL,// FONT_SIZE_SMALL,
                    
                    ViewParams::paint_sandwich => ($view_type == 1 ? true : false),
                    ViewParams::sandwich_base => 'gui_skin://special_icons/sandwich_base.aai',
                    ViewParams::sandwich_mask => 'cut_icon://{name=sandwich_mask}',
                    ViewParams::sandwich_cover => 'cut_icon://{name=sandwich_cover}',
                    ViewParams::sandwich_width => 190,
                    ViewParams::sandwich_height => 290,
                    ViewParams::sandwich_icon_upscale_enabled => true,
                    ViewParams::sandwich_icon_keep_aspect_ratio => true,

//                    ViewParams::item_detailed_info_rel_y => 560
                ),

                PluginRegularFolderView::base_view_item_params => array(
                    ViewItemParams::item_paint_icon => true,
//                    ViewItemParams::icon_scale_factor =>0.75,
                    ViewItemParams::icon_sel_scale_factor =>1.2,
                    ViewItemParams::icon_path => 'gui_skin://small_icons/movie.aai',
                    ViewItemParams::item_layout => HALIGN_LEFT,
                    ViewItemParams::icon_valign => VALIGN_CENTER,
                    ViewItemParams::icon_dx => 10,
                    ViewItemParams::icon_dy => -5,
                    ViewItemParams::icon_width => ($view_type == 1 ? 190 : ($view_type == 2 ? 150 : 50)),
                    ViewItemParams::icon_height => ($view_type == 1 ? 290 : ($view_type == 2 ? 200 : 50)),
                    ViewItemParams::icon_sel_margin_top => 0,
                    ViewItemParams::item_paint_caption => ($view_type == 1 ? false : true),
                    ViewItemParams::item_caption_width => ($view_type == 2 ? 950 : 1100)
                ),

                PluginRegularFolderView::not_loaded_view_item_params => array(
                    ViewItemParams::item_paint_icon => true,
                    ViewItemParams::icon_width => ($view_type == 1 ? 190 : ($view_type == 2 ? 150 : 50)),
                    ViewItemParams::icon_height => ($view_type == 1 ? 290 : ($view_type == 2 ? 200 : 50)),
                    ViewItemParams::icon_path => ($view_type != 3 ? 'gui_skin://large_icons/movie.aai' : 'gui_skin://small_icons/movie.aai')
                ),

                PluginRegularFolderView::actions => $actions
            )
        );

        throw new Exception('Unknown media url');
    }

    ///////////////////////////////////////////////////////////////////////
    private function parse_season_info($season_ref, &$plugin_cookies)
    {
        //hd_print("--> parse season info: started...");
        $ref = $season_ref;
        $api = HOST_API_URL;
        $backgr_view = isset($plugin_cookies->backgr_view) ? $plugin_cookies->backgr_view : 0;
        $items = array();

        $url = $api . $ref;
        if (file_exists('/tmp/brb.cookies')) 
		{
		$referer = $url;
		$doc = HD::http_get_document_cookie($url, $referer);
		}
		else
		{
        $doc = HD::http_get_document($url);
        }
        $tmp = strstr($doc, '<div class="l-content  ">');
        $info_block = strstr($tmp, '<div class="b-filelist"></div>', true);
        //hd_print("info_block--->>> $info_block");
        if ($backgr_view == 0)
		{
			if (preg_match("/indx=\"1\"/i",$info_block))
			{	
				$tmp = explode('<div class="scrollable">', $doc);
				$video_image_a = str_remove_spec(strstr($tmp[1], ' </div>', true));
				$tmp1 = explode("<img src='", $video_image_a);
				unset($tmp1[0]);
				unset($tmp1[1]);
				$tmp = array_values($tmp1);
				shuffle($tmp);
				$video_image_b = str_remove_spec(strstr($tmp[0], "'", true));  //Vokir
				$video_image_b = str_replace('/5/', '/1/', $video_image_b);  //Vokir
				$video_image_b = str_replace('/11/', '/1/', $video_image_b);  //Vokir
				
				if ($video_image_b == false)
				{
					$video_image_b = 'gui_skin://images/bg.jpg';
				}
			}
			else
			{
					$video_image_b = 'gui_skin://images/bg.jpg';
			}
		}
		else
		{
				$video_image_b = 'gui_skin://images/bg.jpg';
		}
		$block_material = '';
		$tmp = explode("materialId: '", $info_block);
		if (count($tmp) > 1)
        $page_id = str_remove_spec(strstr($tmp[1], "'", true));
		$url_in = $api . $ref . "?ajax&id=$page_id&download=1&view=0&view_embed=0&blocked=1&folder_quality=null&folder_lang=null&folder_translate=null&folder=0";
        $in_doc = HD::http_get_document($url_in);
		if (preg_match("/Некоторые файлы из этого материала временно заблокированы/i",$in_doc))
		{
		$block_material = '
		[Некоторые файлы из этого материала временно заблокированы]';
		}
		// title
		$tmp = explode('Клипы</b>', $info_block);
		if (count($tmp) > 1)
		{
        $video_title = str_remove_spec(strip_tags(strstr($tmp[1], '</div>', true)));
		$video_title = preg_replace('/\s{2,}/', ' ', $video_title);
		}
		else
		{
		$tmp = explode('<div class="b-tab-item__title-inner">', $info_block);
		$video_title = str_remove_spec(strstr($tmp[1], '</span>', true));
		$video_title = preg_replace('/\s{2,}/', ' ', $video_title);
		$video_title = strip_tags($video_title);
		}
		
        //hd_print("video_title--->>> $video_title");
		
        // image
		$video_image = false;
		$tmp = explode('<img src="', $info_block);
		if (count($tmp) > 1)
        $video_image = str_remove_spec(strstr($tmp[1], '"', true));
		if ($video_image == false)
			{
			$tmp = explode('<link rel="image_src" href="', $doc);
			$video_image = str_remove_spec(strstr($tmp[1], '"', true));
			}
		$video_image = str_replace('/2/', '/1/', $video_image);  //Vokir
		$video_image = str_replace('/6/', '/1/', $video_image);  //Vokir
		$video_image = str_replace('/9/', '/1/', $video_image);  //Vokir
		//hd_print("video_image--->>> $video_image");
        
		// description
        $description = '';
		$info_block = str_replace('<div class="b-tab-item__description">', '<p>', $info_block);
        $tmp = explode('<p>', $info_block);
        if (count($tmp) > 1)
        {
            $tmp = strstr($tmp[1], '</p>', true);
			$tmp = str_replace("<br />", "                                                                                        \n", $tmp);
            $description = str_remove_spec($tmp);
			$description = strip_tags($description);
			$description = str_replace('Минимальная необходимая версия: , у вас используется . Рекомендуем обновлять только с официального сайта Adobe', 'Клип', $description);
		}
		
        //hd_print("--->>> $description");
        
        $tmp = "";
        // year
        $year = '';
		
        $tmp = explode("Год:", $info_block);
        if (count($tmp) > 1)
            $year = strstr($tmp[1], '</a>', true);
			$year = strip_tags($year);
        
        //hd_print("--->>> $year");

        // country
        $country = '';
        $tmp = explode("Страна:", $info_block);
        if (count($tmp) > 1)
            $country = str_remove_spec(strstr($tmp[1], '</a>                        </td>', true));
			$country = str_replace('</span></a> ', ",", $country);
			$country = strip_tags($country);
			$country = str_replace("  ", "", $country);
		// delivery
		$deliverys = 0;
		$tmp = explode("Период&nbsp;показа:", $info_block);
        if (count($tmp) > 1)
            $deliverys = str_remove_spec(strstr($tmp[1], '</tr>', true));
			$deliverys = str_replace("<span class='serial-period-from-to'>", " - ", $deliverys);
			$deliverys = strip_tags($deliverys);
			$deliverys = str_replace("  ", "", $deliverys);
		if (!$deliverys == 0){
		$country = $country . ', Период показа: '. $deliverys;
		}
        //hd_print("--->>> $country");

        // genre
        $genre = '';
		$tmp = explode("Жанр:", $info_block);
        if (count($tmp) > 1)
            $genre = str_remove_spec(strstr($tmp[1], '</tr>', true));
			$genre = str_replace('</span></a></span> ', ", ", $genre);
			$genre = strip_tags($genre);
        
        //hd_print("--->>> $genre");
        
        // duration
       
		$duration_min = 0;
        $duration = '';
             
        //hd_print("--->>> $duration");

        // director
        $director = '';
        $tmp = explode("Режиссёр:", $info_block);
        if (count($tmp) > 1)
            $director = str_remove_spec(strstr($tmp[1], '</a>                        </td>', true));
			$director = str_replace('</span></a> ', ", ", $director);
			$director = strip_tags($director);
        //hd_print("--->>> $director");
            
        // actors
        $actors = '';
        $tmp = explode("В ролях:", $info_block);
        if (count($tmp) > 1)
            $actors = str_remove_spec(strstr($tmp[1], 'еще</span></a>', true));
			$actors = str_replace("</a><a href='#'", "<a href=", $actors);
			$actors = str_replace("</a>", ",", $actors);
			$actors = strip_tags($actors);
        //hd_print("--->>> $actors");
		if (preg_match("/_in-favourite-wrap\" >/i",$info_block))
		{
		$brbfav_in = "[В избранном] ";
		}
		else
		{
		$brbfav_in = "";
		}
		if (preg_match("/_in-future-wrap\" >/i",$info_block))
		{
		$brbftr_in = "[На будущее] ";
		}
		else
		{
		$brbftr_in = "";
		}
		$episode_filtr ='';
		if (!preg_match("/\/audio\//i",$url))
				{
				$episode_filtr_1 = "";
				$episode_filtr_2 = "";
				$episode_filtr_3 = "";
				$ser_qual = "series_quality&". $ref;
				if (isset($plugin_cookies->$ser_qual))
				{
					$series_quality = $plugin_cookies->$ser_qual;
				}
				else
				{
					$series_quality = 'none';
				}
				
				$ser_trans = "series_translation&". $ref;
				if (isset($plugin_cookies->$ser_trans))
				{
					$series_translation = $plugin_cookies->$ser_trans;
				}
				else
				{
					$series_translation = 'none';
				}
				$ser_season = "series_season&". $ref;
				if (isset($plugin_cookies->$ser_season))
				{
					$series_season = $plugin_cookies->$ser_season;
				}
				else
				{
					$series_season = 'none';
				}
				if (!preg_match("/none/i",$series_translation))
					{
					$episode_filtr_2 = " " . $series_translation . ",";
					}	
				if (!preg_match("/none/i",$series_quality))
					{
					$episode_filtr_1 = " " . $series_quality. ", ";
					}	
				if (!preg_match("/none/i",$series_season))
					{
					$tmp = explode('&&', $series_season);
					$episode_filtr_3 = " " . $tmp[1]. ", ";
					}	
				$episode_filtr = ' ::Фильтры:'.$episode_filtr_3 . $episode_filtr_1 . $episode_filtr_2;
				}
		//name_original 
		$name_original = '';
        $tmp = explode('itemprop="alternativeHeadline">', $info_block);
        if (count($tmp) > 1)
            $name_original = str_remove_spec(strstr($tmp[1], '</div>', true));
			$name_original = strip_tags($name_original);
		$name_original = $brbfav_in . $brbftr_in . $name_original . $block_material . $episode_filtr;
        //hd_print("name_original--->>> $name_original");
		//rate_imdb
		$rate_imdb = '';
        //hd_print("rate_imdb--->>> $rate_imdb");

        // number of series
        $n_series = 2;
                
        //hd_print("--->>> $n_series");
        
		// quality
        $quality = '';
        $tmp = explode('<div class="b-dropdown m-popup m-quality" style="display: none;">', $info_block);
        if (count($tmp) > 1)
            $quality = str_remove_spec(strstr($tmp[1], '</ul>', true));
        //hd_print("--->>> $quality");
		
		// translation
		$translation = '';
        $tmp = explode('<div class="b-dropdown m-popup m-sound" style="display:none;">', $info_block);
        if (count($tmp) > 1)
            $translation = str_remove_spec(strstr($tmp[1], '</ul>', true));
		
		// season
		$season = 'stop';
		$tmp = explode("materialId: '", $info_block);
		if (count($tmp) > 1)
        $page_id = str_remove_spec(strstr($tmp[1], "'", true));
		$url_in = $api . $ref . "?ajax&id=$page_id&download=1&view=0&view_embed=0&blocked=1&folder_quality=null&folder_lang=null&folder_translate=null&folder=0";
        
		if (preg_match("/сезон/i",$in_doc))
		{
		$season = $url_in;
		}
		// myrating
		$myrating = "Не выставлена.";
        $tmp = explode('<span class="b-tab-item__vote-rating-user-value">', $doc);
        if (count($tmp) > 1)
            $myrating = str_remove_spec(strstr($tmp[1], '<', true));
			$myrating = strip_tags($myrating);
		if ($myrating == '')
		{
		$myrating = "Не выставлена.";
		}
		//oth_works
		$oth_works = '';
        $tmp = explode('<div class="item-info">', $info_block);
        if (count($tmp) > 1)
            $oth_works = str_remove_spec(strstr($tmp[1], '</table>', true));
		//user_ratings
		$user_ratings = '';
        $tmp = explode('Оценки пользователей:', $info_block);
        if (count($tmp) > 1)
            $user_ratings = str_remove_spec(strstr($tmp[1], '<a href= "', true));
			$user_ratings = str_replace("<div class='b-tab-item__vote-value m-tab-item__vote-value_type_yes'>", "Нравится : ", $user_ratings);
			$user_ratings = str_replace("<div class='b-tab-item__vote-value m-tab-item__vote-value_type_no'>", " Не нравится: ", $user_ratings);
			$user_ratings = str_replace("Ваша оценка", "", $user_ratings);
			$user_ratings = str_replace("+1", "", $user_ratings);
			$user_ratings = str_replace("-1", "", $user_ratings);
			$user_ratings = str_replace("Отмена", "", $user_ratings);
			$user_ratings = strip_tags($user_ratings);
		
		//coments
		$show_coment = isset($plugin_cookies->show_coment) ? $plugin_cookies->show_coment : 0;
		$doc = str_replace('b-user_woman" rel="nofollow">', "", $doc);
		$doc = str_replace('b-user_man" rel="nofollow">', "", $doc);
		$videos = explode('man" rel="nofollow">', $doc);
		unset($videos[0]);
		$videos = array_slice($videos, 0, 40);
		$videos = array_values($videos);
		$coments = array();
		$coments["Оценка:"] = $myrating;
		$coments["Оценки:"] = $user_ratings;
		$coments["======"] = '=================================';
		
		if ($show_coment == 1)
		{
		$ndx = 1;
		foreach($videos as $video)
				{
					$name = strstr($video, "</span>", true);
					$name = strip_tags($name);
					$name = str_replace("  ", "", $name);
					$tmp = explode('itemprop="description">', $video);
					if (count($tmp) > 1)
					$item = strstr($tmp[1], '</div>', true);
					$item = str_replace("  ", "", $item);
					$item = strip_tags($item);
					$item = $name .': '. $item;
					$item = wordwrap($item, 85, "\n");
					$coments[$ndx] = $item;
					++$ndx;
				}
		}
        
		// similar
		$similar = '';
        $tmp = explode('<div class="b-similar__title m-themed">', $doc);
        if (count($tmp) > 1)
            $similar = str_remove_spec(strstr($tmp[1], '<div class="clear">', true));
		
		// status
		$status = "";
        $tmp = explode('Статус:', $info_block);
        if (count($tmp) > 1)
            $status = str_remove_spec(strstr($tmp[1], '</tr>', true));
			$status = strip_tags($status);
			$status = str_replace("сезон", "s", $status);
			$status = str_replace("серия", "e", $status);
			$status = str_replace("финал", "F", $status);
			$status = str_replace(" ", "", $status);
			$status = str_replace("\n", "", $status);
		
		// reviews_count
		$reviews_count = 0;
        $tmp = explode('<p class="b-item-material-comments__count">', $doc);
        if (count($tmp) > 1)
            $reviews_count = str_remove_spec(strstr($tmp[1], '  отзыв', true));
		$reviews_count = strip_tags($reviews_count);
		// reviews_count
		$similar_count = "нет";
        if (!$similar == ""){
		$similar_count = "есть";
		}
		
		$rate_details = array();
		$rate_details["Статус:"] = $status;
		$rate_details["Отзывов:"] = $reviews_count;
		$rate_details["Похожие:"] = $similar_count;
		
		return  HD::encode_user_data
                (
                    array
                    (
                        'video_id' => $season_ref,
                        'n_series' => $n_series,
                        'page_ref' => $season_ref,
                        'video_image_url' => $video_image,
                        'title' => $video_title,
                        'genre' => str_remove_spec($genre),
                        'country' => str_remove_spec($country),
                        'year' => str_remove_spec($year),
                        'duration' => str_remove_spec($duration),
                        'duration_min' => $duration_min,
                        'director' => str_remove_spec($director),
                        'actors' => str_remove_spec($actors),
                        'description' => str_remove_spec($description),
						'name_original' => $name_original,
                        'video_image_url_b' => $video_image_b,
                        'rate_imdb' => $rate_imdb,
						'quality' => $quality,
						'translation' => $translation,
						'coments' => $coments,
						'rate_details' => $rate_details,
						'similar' => $similar,
						'oth_works' => $oth_works,
						'myrating' => $myrating,
						'brbfav_in' => $brbfav_in,
						'brbftr_in' => $brbftr_in,
						'season' => $season,
                    )
                );
    }
    ///////////////////////////////////////////////////////////////////////

    private function parse_season($season_ref, &$plugin_cookies)
    {
        //hd_print("--> parse season: started...");
        
		$buf_time = isset($plugin_cookies->buf_time) ? $plugin_cookies->buf_time : 0;
		//hd_print("buf_time--->>> $buf_time");
		$ref = $season_ref;
        $api = HOST_API_URL;
        
        $items = array();

        $url = $api . $ref;
        
        $doc = HD::http_get_document($url);
        
        $tmp = strstr($doc, '<div class="l-content  ">');
        $info_block = strstr($tmp, '<div class="b-filelist"></div>', true);
		//$info_block = str_replace("url: 'http://", '<img src="http://', $info_block);//Vokir
		$info_block = str_replace(".jpg'", '.jpg"', $info_block);//Vokir
		//hd_print("info_block--->>> $info_block");
		
        // title
		$tmp = explode('Клипы</b>', $info_block);
		if (count($tmp) > 1)
		{
        $video_title = str_remove_spec(strip_tags(strstr($tmp[1], '</div>', true)));
		$video_title = preg_replace('/\s{2,}/', ' ', $video_title);
		}
		else
		{
		$tmp = explode('<div class="b-tab-item__title-inner">', $info_block);
		$video_title = str_remove_spec(strstr($tmp[1], '</span>', true));
		$video_title = preg_replace('/\s{2,}/', ' ', $video_title);
		$video_title = strip_tags($video_title);
		}
        //hd_print("video_title--->>> $video_title");
        
        // image
		$video_image = false;
		$tmp = explode('<img src="', $info_block);
		if (count($tmp) > 1)
        $video_image = str_remove_spec(strstr($tmp[1], '"', true));
		if ($video_image == false)
			{
			$tmp = explode('<link rel="image_src" href="', $doc);
			$video_image = str_remove_spec(strstr($tmp[1], '"', true));
			}
		$video_image = str_replace('/2/', '/1/', $video_image);  //Vokir
		$video_image = str_replace('/6/', '/1/', $video_image);  //Vokir
		$video_image = str_replace('/9/', '/1/', $video_image);  //Vokir
		//hd_print("video_image--->>> $video_image");
        
		// description
        $description = '';
		$info_block = str_replace('<div class="b-tab-item__description">', '<p>', $info_block);
        $tmp = explode('<p>', $info_block);
        if (count($tmp) > 1)
        {
            $tmp = strstr($tmp[1], '</p>', true);
			$tmp = str_replace("<br />", "                                                                                        \n", $tmp);
            $description = str_remove_spec($tmp);
			$description = strip_tags($description);
			$description = str_replace('Минимальная необходимая версия: , у вас используется . Рекомендуем обновлять только с официального сайта Adobe', 'Клип', $description);
		}

        //hd_print("--->>> $description");

		$series_tr = '';
		$series_ql ='';
		$episode_filtr ='';
		$series_sn = '';
		if (!preg_match("/\/audio\//i",$url))
				{
				$episode_filtr_1 = "";
				$episode_filtr_2 = "";
				$episode_filtr_3 = "";
				$ser_qual = "series_quality&". $ref;
				if (isset($plugin_cookies->$ser_qual))
				{
					$series_quality = $plugin_cookies->$ser_qual;
				}
				else
				{
					$series_quality = 'none';
				}
				
				$ser_trans = "series_translation&". $ref;
				if (isset($plugin_cookies->$ser_trans))
				{
					$series_translation = $plugin_cookies->$ser_trans;
				}
				else
				{
					$series_translation = 'none';
				}
				$ser_season = "series_season&". $ref;
				if (isset($plugin_cookies->$ser_season))
				{
					$series_season = $plugin_cookies->$ser_season;
				}
				else
				{
					$series_season = 'none';
				}
				if (!preg_match("/none/i",$series_translation))
					{
					$series_tr = '&folder_translate=' . urlencode($series_translation);
					$episode_filtr_2 = " Озвучка: " . $series_translation;
					}	
				if (!preg_match("/none/i",$series_quality))
					{
					$series_ql = '&folder_quality=' . urlencode($series_quality);
					$episode_filtr_1 = " Показывать: " . $series_quality;
					}	
				if (!preg_match("/none/i",$series_season))
					{
					$tmp = explode('&&', $series_season);
					$series_sn = $tmp[0];
					$episode_filtr_3 = " Сезон: " . $tmp[1];
					}	
				$episode_filtr = $episode_filtr_3 . $episode_filtr_1 . $episode_filtr_2;
				}
		
		
		$is_new = false;
        
		$tmp = explode("materialId: '", $info_block);
		if (count($tmp) > 1)
        $page_id = str_remove_spec(strstr($tmp[1], "'", true));
		$tmp = explode('id="vote_up" rel="', $info_block);
		$page_id2 = str_remove_spec(strstr($tmp[1], '"', true));
		$url_in = $api . $ref . "?ajax&id=$page_id&download=1&view=0&view_embed=0&blocked=1&folder_quality=null&folder_lang=null&folder_translate=null&folder=0";
        $in_doc = HD::http_get_document($url_in);
		if (preg_match("/Некоторые файлы из этого материала временно заблокированы/i",$in_doc) && file_exists('/tmp/brb.cookies'))
		{
		// if (file_exists('/tmp/brb.cookies')) 
		// {
		$link = 'http://brb.to/materials/edit/i'.$page_id.'?win=files';
		$doc = HD::http_get_document_cookie($link, $url);
		
		$item_id = "";
		$tmp = explode("{item_id: '", $doc);
		if (count($tmp) > 1)
		$item_id = str_remove_spec(strstr($tmp[1], "'", true));
		
		$user_id = "";
		$tmp = explode("user_id': '", $doc);
		if (count($tmp) > 1)
		$user_id = str_remove_spec(strstr($tmp[1], "'", true));
		$file_page = "http://brb.to/jsfilemanager.aspx?f=files_list&item_id=$item_id&id=false&u=$user_id";
		$docs = HD::http_get_document_cookie($file_page, $link);
		$j_doc = json_decode($docs);
		$videos = array_values($j_doc);
		$episode_title = "";   
        $n = 0;
        $ndx_series = 0;
		foreach($videos as $j_doc)
			{
			hd_silence_warnings();
			$episode_ref = $j_doc->{'link'};
			$episode_ref = $api . $episode_ref;
			$episode_title = $j_doc->{'filename'};
			hd_restore_warnings();
			if (!$episode_title == false)
			{
            $caption = '';
			$new_view = isset($plugin_cookies->new_view) ? $plugin_cookies->new_view : 1;
			if (isset($plugin_cookies->$season_ref))
			if (!$new_view == 0)
			{
			if (!ListUtil::is_in_list($plugin_cookies->$season_ref, $episode_ref))
			$caption = '[NEW]';
			}
			//{ Viewed items
			$is_viewed = false;
            $is_time_viewed = false;
			if (!$is_new)
				{
					$vs = $season_ref . '.' . $episode_ref;
					if (isset($plugin_cookies->$vs))
                    {
                    $t = explode('|', $plugin_cookies->$vs);
                    if (intval($t[0]) != -1)
                    $caption = '[ПРОСМ ' . HD::format_duration(intval($t[0]) * 1) . '/' . HD::format_duration(intval($t[1]) * 1) . '] ';
                    else
                    $caption = '[ПРОСМ] ';
                    }
                    else
                    {
				    $vs = $season_ref . ".viewed";
					if (isset($plugin_cookies->$vs))
					{
					if (ListUtil::is_in_list($plugin_cookies->$vs, $episode_ref))
                    $caption = '[ПРОСМ] ';
					}
                    }
				}
				//} Viewed items

				array_push
				(
					$items,
					HD::encode_user_data
					(
						array
							(
							//{ Viewed items
							'episode_title' => str_remove_spec($caption . $episode_title),
							//} Viewed items
				     		'episode_ref' => str_remove_spec($episode_ref),
							'season_title' => $video_title,
							'season_ref' => str_remove_spec($season_ref),
							'ndx_series' => $ndx_series,
							'poster_url' => str_remove_spec($video_image),
							'description' => str_remove_spec($description),
							)
					)
				);
				if ($ndx_series == 9999) 
				{
				return $items;
				}
				++$this->m_items_count;
				++$ndx_series;
				}		
				}
		//}
		}
		else
		{
		$str = HD::http_get_document( $url . '?ajax&id='.$page_id2. $series_ql . $series_tr . '&folder='. $series_sn);	
			//hd_print("str--->>>: $str");
			
		$videos = explode('" rel="{parent_id: ', $str);
        unset($videos[0]);
        $videos = array_values($videos);
						            
		$episode_title = "";
        //$translation_ql = "";   
        $n = 0;
        $ndx_series = 0;
        foreach($videos as $video)
          
            {
				$translation_ql = '';
				$tmps = explode("'}\">", $video);
				if (count($tmps) > 1)
				$translation_ql = str_remove_spec(strstr($tmps[1], '</span>', true));
				$translation_ql = str_replace("Список файлов</a>", "", $translation_ql);
				$translation_ql = str_replace("&darr;", ":", $translation_ql);
				$translation_ql = str_replace("\n", "", $translation_ql);
				$translation_ql = preg_replace('|\s+|', ' ', $translation_ql);
				$translation_ql = ' ['.strip_tags($translation_ql).']';
				$translation_ql = str_replace("[]", "", $translation_ql);

				$fileid = strstr($video, '}', true);
				$fileid = str_replace("'", "", $fileid);
                $folder_ref = $url .'?ajax&id='.$page_id2. $series_ql . $series_tr .'&folder='.$fileid;
                //hd_print("folder_ref--->>>: $folder_ref");
				$tmp = HD::http_get_document($folder_ref);	
								
				$links = explode('<li class="b-file-new', $tmp);
				unset($links[0]);
				if ($links == false)
				{
										
					$links_seasons = explode('" rel="{parent_id: ', $tmp);
					unset($links_seasons[0]);
					$links_seasons = array_values($links_seasons);		
					foreach($links_seasons as $links_season)
          
					{
						$translation_ql = '';
						$tmps = explode("'}\">", $links_season);
						if (count($tmps) > 1)
						$translation_ql = str_remove_spec(strstr($tmps[1], '</span>', true));
						$translation_ql = str_replace("Список файлов</a>", "", $translation_ql);
						$translation_ql = str_replace("&darr;", ":", $translation_ql);
						$translation_ql = str_replace("\n", "", $translation_ql);
						$translation_ql = preg_replace('|\s+|', ' ', $translation_ql);
						$translation_ql = ' ['.strip_tags($translation_ql).']';
						$translation_ql = str_replace("[]", "", $translation_ql);

						$fileid = strstr($links_season, '}', true);
						$fileid = str_replace("'", "", $fileid);
						$folder_ref = $url .'?ajax&id='.$page_id2. $series_ql . $series_tr .'&folder='.$fileid;
						//hd_print("folder_ref--->>>: $folder_ref");
						$tmp = HD::http_get_document($folder_ref);	
								
						$links = explode('<li class="b-file-new', $tmp);
						unset($links[0]);
						if ($links == false)
						{
							$links_seasons = explode('" rel="{parent_id: ', $tmp);
							unset($links_seasons[0]);
							$links_seasons = array_values($links_seasons);		
							foreach($links_seasons as $links_season)
				  
							{
								
								$fileid = strstr($links_season, '}', true);
								$fileid = str_replace("'", "", $fileid);
								$folder_ref = $url .'?ajax&id='.$page_id2. $series_ql . $series_tr .'&folder='.$fileid;
								//hd_print("folder_ref--->>>: $folder_ref");
								$tmp = HD::http_get_document($folder_ref);	
										
								$links = explode('<li class="b-file-new', $tmp);
								unset($links[0]);
								$links = array_values($links);
								
								foreach($links as $link)
								{
									$link = str_replace("'", '"', $link);
									//hd_print("link--->>>:  $link");
									$tmp = explode('filename-text" >', $link);
									$episode_titl = str_remove_spec(strstr($tmp[1], '<', true));
									
									$tmp = explode('material-size">', $link);
									$episode_size = strstr($tmp[1], "<", true);
									
									if ($buf_time == 0)
									{
										$tmp = explode('" href="', $link);
										$episode_ref = 'http://brb.to'.(strstr($tmp[1], '"', true));
										$episode_title = $episode_titl . ' - ' . $episode_size .  $translation_ql . $episode_filtr;
									}
									else
									{
										$tmp = explode('/get/play/', $link);
										if (count($tmp) > 1)
										{
											$episode_link = strstr($tmp[1], '"', true);
											$tmp = explode('.', $episode_link);
											$episode_ex = $tmp[1];
											$episode_ref = 'http://brb.to/get/play/'. $episode_link;
											$episode_titl = substr($episode_titl, 0, strrpos($episode_titl, '.' ));
											$episode_title = $episode_titl . "." . $episode_ex . $translation_ql . $episode_filtr;
										}
										else
										{
											$tmp = explode('" href="', $link);
											$episode_ref = 'http://brb.to'.(strstr($tmp[1], '"', true));
											$episode_title = $episode_titl . ' - ' . $episode_size .  $translation_ql . $episode_filtr;
										}
										
									}
									// hd_print("--->>> episode_ref: $episode_ref"); 
									// hd_print("--->>> episode_title: $episode_title");
									
									$caption = '';
									$new_view = isset($plugin_cookies->new_view) ? $plugin_cookies->new_view : 1;
									if (isset($plugin_cookies->$season_ref))
											if (!$new_view == 0)
											{
											if (!ListUtil::is_in_list($plugin_cookies->$season_ref, $episode_ref))
												$caption = '[NEW]';
											}
													//{ Viewed items
													$is_viewed = false;
													$is_time_viewed = false;
													if (!$is_new)
													{
														$vs = $season_ref . '.' . $episode_ref;
														if (isset($plugin_cookies->$vs))
														{
															$t = explode('|', $plugin_cookies->$vs);
															if (intval($t[0]) != -1)
																$caption = '[ПРОСМ ' . HD::format_duration(intval($t[0]) * 1) . '/' . HD::format_duration(intval($t[1]) * 1) . '] ';
															else
																$caption = '[ПРОСМ] ';
														}
														else
														{
															$vs = $season_ref . ".viewed";
															if (isset($plugin_cookies->$vs))
															{
																if (ListUtil::is_in_list($plugin_cookies->$vs, $episode_ref))
																	$caption = '[ПРОСМ] ';
															}
														}
													}
													//} Viewed items

									array_push
									(
										$items,
										HD::encode_user_data
										(
											array
											(
												//{ Viewed items
												'episode_title' => str_remove_spec($caption . $episode_title),
												//} Viewed items
												'episode_ref' => str_remove_spec($episode_ref),
												'season_title' => $video_title,
												'season_ref' => str_remove_spec($season_ref),
												'ndx_series' => $ndx_series,
												'poster_url' => str_remove_spec($video_image),
												'description' => str_remove_spec($description),
											)
										)
									);
									if ($ndx_series == 999) 
									{
									 return $items;
									}
									++$this->m_items_count;
									++$ndx_series;
									
								}
							}
						}
						else
						{
							$links = array_values($links);
							
							foreach($links as $link)
							{
								
								$link = str_replace("'", '"', $link);
								//hd_print("link--->>>:  $link");
								$tmp = explode('filename-text" >', $link);
								$episode_titl = str_remove_spec(strstr($tmp[1], '<', true));
								
								$tmp = explode('material-size">', $link);
								$episode_size = strstr($tmp[1], "<", true);
								
								if ($buf_time == 0)
								{
									$tmp = explode('" href="', $link);
									$episode_ref = 'http://brb.to'.(strstr($tmp[1], '"', true));
									$episode_title = $episode_titl . ' - ' . $episode_size .  $translation_ql . $episode_filtr;
								}
								else
								{
									$tmp = explode('/get/play/', $link);
									if (count($tmp) > 1)
									{
										$episode_link = strstr($tmp[1], '"', true);
										$tmp = explode('.', $episode_link);
										$episode_ex = $tmp[1];
										$episode_ref = 'http://brb.to/get/play/'. $episode_link;
										$episode_titl = substr($episode_titl, 0, strrpos($episode_titl, '.' ));
										$episode_title = $episode_titl . "." . $episode_ex . $translation_ql . $episode_filtr;
									}
									else
									{
										$tmp = explode('" href="', $link);
										$episode_ref = 'http://brb.to'.(strstr($tmp[1], '"', true));
										$episode_title = $episode_titl . ' - ' . $episode_size .  $translation_ql . $episode_filtr;
									}
									
								}
								// hd_print("--->>> episode_ref: $episode_ref"); 
								// hd_print("--->>> episode_title: $episode_title");
								
								$caption = '';
								$new_view = isset($plugin_cookies->new_view) ? $plugin_cookies->new_view : 1;
								if (isset($plugin_cookies->$season_ref))
										if (!$new_view == 0)
										{
										if (!ListUtil::is_in_list($plugin_cookies->$season_ref, $episode_ref))
											$caption = '[NEW]';
										}
												//{ Viewed items
												$is_viewed = false;
												$is_time_viewed = false;
												if (!$is_new)
												{
													$vs = $season_ref . '.' . $episode_ref;
													if (isset($plugin_cookies->$vs))
													{
														$t = explode('|', $plugin_cookies->$vs);
														if (intval($t[0]) != -1)
															$caption = '[ПРОСМ ' . HD::format_duration(intval($t[0]) * 1) . '/' . HD::format_duration(intval($t[1]) * 1) . '] ';
														else
															$caption = '[ПРОСМ] ';
													}
													else
													{
														$vs = $season_ref . ".viewed";
														if (isset($plugin_cookies->$vs))
														{
															if (ListUtil::is_in_list($plugin_cookies->$vs, $episode_ref))
																$caption = '[ПРОСМ] ';
														}
													}
												}
												//} Viewed items

								array_push
								(
									$items,
									HD::encode_user_data
									(
										array
										(
											//{ Viewed items
											'episode_title' => str_remove_spec($caption . $episode_title),
											//} Viewed items
											'episode_ref' => str_remove_spec($episode_ref),
											'season_title' => $video_title,
											'season_ref' => str_remove_spec($season_ref),
											'ndx_series' => $ndx_series,
											'poster_url' => str_remove_spec($video_image),
											'description' => str_remove_spec($description),
										)
									)
								);
								if ($ndx_series == 999) 
								{
								 return $items;
								}
								++$this->m_items_count;
								++$ndx_series;
								
							}
						}
					}		
						
				}
				else
				{	
						$links = array_values($links);
						
						foreach($links as $link)
						{
							$link = str_replace("'", '"', $link);
							//hd_print("link--->>>:  $link");
							$tmp = explode('filename-text" >', $link);
							$episode_titl = str_remove_spec(strstr($tmp[1], '<', true));
							
							$tmp = explode('material-size">', $link);
							$episode_size = strstr($tmp[1], "<", true);
							
							if ($buf_time == 0)
							{
								$tmp = explode('" href="', $link);
								$episode_ref = 'http://brb.to'.(strstr($tmp[1], '"', true));
								$episode_title = $episode_titl . ' - ' . $episode_size .  $translation_ql . $episode_filtr;
							}
							else
							{
								$tmp = explode('/get/play/', $link);
								if (count($tmp) > 1)
								{
									$episode_link = strstr($tmp[1], '"', true);
									$tmp = explode('.', $episode_link);
									$episode_ex = $tmp[1];
									$episode_ref = 'http://brb.to/get/play/'. $episode_link;
									$episode_titl = substr($episode_titl, 0, strrpos($episode_titl, '.' ));
									$episode_title = $episode_titl . "." . $episode_ex . $translation_ql . $episode_filtr;
								}
								else
								{
									$tmp = explode('" href="', $link);
									$episode_ref = 'http://brb.to'.(strstr($tmp[1], '"', true));
									$episode_filtr = str_replace('&folder_quality=', '* Показывать: ', $series_quality);
									$episode_title = $episode_titl . ' - ' . $episode_size . $translation_ql . $episode_filtr;
								}
								
							}
							//hd_print("--->>> episode_ref: $episode_ref"); 
							//hd_print("--->>> episode_title: $episode_title");
							$new_view = isset($plugin_cookies->new_view) ? $plugin_cookies->new_view : 1;
                            $caption = '';
                            if (isset($plugin_cookies->$season_ref))
  									if (!$new_view == 0)
									{
									if (!ListUtil::is_in_list($plugin_cookies->$season_ref, $episode_ref))
										$caption = '[NEW]';
									}
                                            //{ Viewed items
                                            $is_viewed = false;
                                            $is_time_viewed = false;
                                            if (!$is_new)
                                            {
                                                $vs = $season_ref . '.' . $episode_ref;
                                                if (isset($plugin_cookies->$vs))
                                                {
                                                    $t = explode('|', $plugin_cookies->$vs);
                                                    if (intval($t[0]) != -1)
                                                        $caption = '[ПРОСМ ' . HD::format_duration(intval($t[0]) * 1) . '/' . HD::format_duration(intval($t[1]) * 1) . '] ';
                                                    else
                                                        $caption = '[ПРОСМ] ';
                                                }
                                                else
                                                {
                                                    $vs = $season_ref . ".viewed";
                                                    if (isset($plugin_cookies->$vs))
                                                    {
                                                        if (ListUtil::is_in_list($plugin_cookies->$vs, $episode_ref))
                                                            $caption = '[ПРОСМ] ';
                                                    }
                                                }
                                            }
                                            //} Viewed items

							array_push
							(
								$items,
								HD::encode_user_data
								(
									array
									(
										//{ Viewed items
										'episode_title' => str_remove_spec($caption . $episode_title),
										//} Viewed items
										'episode_ref' => str_remove_spec($episode_ref),
										'season_title' => $video_title,
										'season_ref' => str_remove_spec($season_ref),
										'ndx_series' => $ndx_series,
										'poster_url' => str_remove_spec($video_image),
										'description' => str_remove_spec($description),
									)
								)
							);
							if ($ndx_series == 999) 
							{
							 return $items;
							}
							++$this->m_items_count;
							++$ndx_series;
						}	
				}		
            }
		}
        return $items;
    }
    ///////////////////////////////////////////////////////////////////////

    public function get_regular_folder_items($media_url, $from_ndx, &$plugin_cookies)
    {
	    //hd_print("--> get_regular_folder_items(): starting." . $media_url);
        if 
	    (
		    $media_url === 'main_menu' ||
		    $media_url === 'main_menu:videos' ||
			$media_url === 'main_menu:genres' ||
		    $media_url === 'main_menu:audio' ||
		    $media_url === 'main_menu:search' ||
		    $media_url === 'main_menu:fav' && isset($plugin_cookies->dynamic_fav_off)

	    )
        {
            return $this->get_main_menu_items($media_url, $plugin_cookies);
        }

        if (intval($from_ndx) > 0)
            --$from_ndx; // replace aux "loading" item
            
	    $videos = false;
        $folder_items = array();
        $total_num = 0;

        HD::decode_user_data($media_url, $media_str, $user_data);

        if (HD::has_attribute($user_data, 'season_ref'))
        {
            if (HD::has_attribute($user_data, 'season_ref'))
                $season_ref = $user_data->season_ref;
            else if (HD::has_attribute($user_data, 'fav'))
                $season_ref = $user_data->fav;
            $videos = $this->parse_season($season_ref, $plugin_cookies);
            $total_num = 0;
                
           if ($videos === false)
            {
		        //hd_print("--> get_regular_folder_items(): no more available.");
                $caption = (is_interface_language_russian_or_similar() ? 'Больше ничего не найдено' : 'No more items found');
                $m_url = 'dummy';
                if (HD::has_attribute($user_data, 'fav'))
                {
                    $caption = (is_interface_language_russian_or_similar() ? 'Hе найдено' : 'Not found');
                    $m_url = $media_url;
                }
		        $more_items_available = 0;
                
                if ($media_url === 'main_menu:fav')
                {
                    $folder->set_expired();
                }

                $view_type = isset($plugin_cookies->view_type) ? $plugin_cookies->view_type : 2;
		        array_push
		        (
                    $folder_items,
                    array
			        (
                        PluginRegularFolderItem::media_url => $m_url,
	                    PluginRegularFolderItem::caption => $caption,
                	    PluginRegularFolderItem::view_item_params => array
                        	    (
                                    ViewItemParams::icon_width => ($view_type == 1 ? 190 : ($view_type == 2 ? 150 : 50)),
                                    ViewItemParams::icon_height => ($view_type == 1 ? 290 : ($view_type == 2 ? 200 : 50)),
                                    ViewItemParams::icon_path => ($view_type != 3 ? 'gui_skin://large_icons/movie.aai' : 'gui_skin://small_icons/movie.aai'),
                		            ViewItemParams::item_detailed_icon_path => 'missing://'
                        		)
			        )
		        );
                ++$total_num;
                
                $this->fav_build_in_progess = false;
            }
            else
            {
                //hd_print("--> get_regular_folder_items(): items count: " . count($videos));
                $more_items_available = 0;

                foreach ($videos as $v)
                {
                    HD::decode_user_data($v, $media_str, $user_data);

                    array_push
                    (
                        $folder_items, 
                        array
                        (
                            PluginRegularFolderItem::media_url => $v,
                            PluginRegularFolderItem::caption => $user_data->episode_title,
                            PluginRegularFolderItem::view_item_params => array(
                                        ViewItemParams::item_paint_icon => true,
                                        ViewItemParams::icon_path => 'gui_skin://small_icons/movie.aai',
                                        ViewItemParams::item_layout => HALIGN_LEFT,
                                        ViewItemParams::icon_valign => VALIGN_CENTER,
                                        ViewItemParams::icon_dx => 20,
                                        ViewItemParams::icon_dy => -5,
                                        ViewItemParams::item_caption_font_size => FONT_SIZE_NORMAL,
                                        ViewItemParams::item_caption_width => 1550
                            ),
                        )
                     );
            
                    ++$total_num;
                }
            }
            return array(
                PluginRegularFolderRange::total => $total_num,
                PluginRegularFolderRange::count => count($folder_items),
                PluginRegularFolderRange::more_items_available => $more_items_available,
                PluginRegularFolderRange::from_ndx => $from_ndx,
                PluginRegularFolderRange::items => $folder_items
            );
        }
                else if (HD::has_attribute($user_data, 'fav'))
        {
            $v = $this->get_next_fav($user_data->fav, $plugin_cookies);
            
            HD::decode_user_data($v[0], $media_str, $user_data);

            $view_type = isset($plugin_cookies->view_type) ? $plugin_cookies->view_type : 2;
            $video_id = '1';
            $n_series = 1;
            $ndx_series = 0;
            $is_has_new = false;
            $title = "";

            if (HD::has_attribute($user_data, 'video_id'))
                $video_id = $user_data->video_id;

            if (HD::has_attribute($user_data, 'is_has_new'))
                $is_has_new = $user_data->is_has_new;
                
            if (HD::has_attribute($user_data, 'season_title'))
                $title = $user_data->season_title;
                
                    if ($view_type == 1)
                    {
                            array_push(
                                $folder_items, 
                                array(
                                    PluginRegularFolderItem::media_url =>
                                        HD::encode_user_data(
                                            array(
                                                'video_id' => $video_id,
                                                'n_series' => $n_series,
                                                'season_title' => $title
                                            )
                                    ),
                                    PluginRegularFolderItem::caption => ($is_has_new ? '[NEW] ' . $title : $title),
                                    PluginRegularFolderItem::view_item_params => array(
                                        ViewItemParams::icon_path => $this->get_preview_url($user_data),
                                        ViewItemParams::item_paint_caption => false,
                                        ViewItemParams::item_caption_wrap_enabled => TRUE,
            //                            ViewItemParams::item_detailed_info => $info
                                    )
                                ));
                    }
                    else if ($view_type == 2)
                    {
                            array_push(
                                $folder_items, 
                                array(
                                    PluginRegularFolderItem::media_url =>
                                        HD::encode_user_data(
                                            array(
                                                'video_id' => $video_id,
                                                'n_series' => $n_series,
                                            )
                                    ),
                                    PluginRegularFolderItem::caption => ($is_has_new ? '[NEW] ' . $title : $title),
                                    PluginRegularFolderItem::view_item_params => array(
                                        ViewItemParams::icon_path => $this->get_preview_url($user_data),
//                                        ViewItemParams::item_detailed_info => $info
                                    )
                                ));
                    }
                    else
                    {
                        array_push(
                            $folder_items, 
                            array(
                                PluginRegularFolderItem::media_url =>
                                    HD::encode_user_data(
                                        array(
                                            'video_id' => $video_id,
                                            'n_series' => $n_series
                                        )
                                ),
                                PluginRegularFolderItem::caption => ($is_has_new ? '[NEW] ' . $title : $title),
                                PluginRegularFolderItem::view_item_params => array(
                                    ViewItemParams::item_detailed_icon_path => $this->get_preview_url($user_data),
  //                                  ViewItemParams::item_detailed_info => $info
                                )
                            ));
                    }

             $more_items_available = 0;
             $total_num = 1;
             
            return array(
                PluginRegularFolderRange::total => $total_num,
                PluginRegularFolderRange::count => count($folder_items),
                PluginRegularFolderRange::more_items_available => $more_items_available,
                PluginRegularFolderRange::from_ndx => $from_ndx,
                PluginRegularFolderRange::items => $folder_items
            );
        }

        $folder = FolderCache::get($media_url);

        if (HD::has_attribute($user_data, 'category_ref'))
        {
            $videos = $folder->get_elements($from_ndx, $plugin_cookies);
            $total_num = 0;
                
            if ($videos === false)
            {
		     //   hd_print("--> get_regular_folder_items(): no more available.");
                $caption = (is_interface_language_russian_or_similar() ? 'Больше ничего не найдено' : 'No more items found');
                $m_url = 'dummy';
                if (HD::has_attribute($user_data, 'fav'))
                {
                    $caption = (is_interface_language_russian_or_similar() ? 'Hе найдено' : 'Not found');
                    $m_url = $media_url;
                }
		        $more_items_available = 0;
                
                if ($media_url === 'main_menu:fav')
                {
                    $folder->set_expired();
                }

                $view_type = isset($plugin_cookies->view_type) ? $plugin_cookies->view_type : 2;
		        array_push
		        (
                    $folder_items,
                    array
			        (
                        PluginRegularFolderItem::media_url => $m_url,
	                    PluginRegularFolderItem::caption => $caption,
                	    PluginRegularFolderItem::view_item_params => array
                        	    (
                                    ViewItemParams::icon_width => ($view_type == 1 ? 190 : ($view_type == 2 ? 150 : 50)),
                                    ViewItemParams::icon_height => ($view_type == 1 ? 290 : ($view_type == 2 ? 200 : 50)),
                                    ViewItemParams::icon_path => ($view_type != 3 ? 'gui_skin://large_icons/movie.aai' : 'gui_skin://small_icons/movie.aai'),
                		            ViewItemParams::item_detailed_icon_path => 'missing://'
                        		)
			        )
		        );
                ++$total_num;
                
                $this->fav_build_in_progess = false;
            }
            else
            {
              //  hd_print("--> get_regular_folder_items(): items count: " . count($videos));
                $more_items_available = 1;

                foreach ($videos as $v)
                {
                    HD::decode_user_data($v, $media_str, $user_data);

                    array_push
                    (
                        $folder_items, 
                        array
                        (
                            PluginRegularFolderItem::media_url => $v,
                            PluginRegularFolderItem::caption => $user_data->subcategory_title,
                            PluginRegularFolderItem::view_item_params => self::$catalog_vip
                        )
                     );
            
                    ++$total_num;
                }
            }
            return array(
                PluginRegularFolderRange::total => $total_num,
                PluginRegularFolderRange::count => count($folder_items),
                PluginRegularFolderRange::more_items_available => $more_items_available,
                PluginRegularFolderRange::from_ndx => $from_ndx,
                PluginRegularFolderRange::items => $folder_items
            );
        }
        else if
        (
            HD::has_attribute($user_data, 'subcategory_ref') ||
            HD::has_attribute($user_data, 'ser_feed') ||
            $media_url === 'main_menu:fav'
        )
        {
       	    $videos = $folder->get_elements($from_ndx, $plugin_cookies);
            $total_num = $folder->get_num_elements();
	            
           if ($videos === false)
            {
		    //    hd_print("--> get_regular_folder_items(): no more available.");
                $caption = (is_interface_language_russian_or_similar() ? 'Больше ничего не найдено' : 'No more items found');
                $m_url = 'dummy';
                if (HD::has_attribute($user_data, 'fav'))
                {
                    $caption = (is_interface_language_russian_or_similar() ? 'Hе найдено' : 'Not found');
                    $m_url = $media_url;
                }
		        $more_items_available = 0;
                
                if ($media_url === 'main_menu:fav')
                {
                    $folder->set_expired();
                }

                $view_type = isset($plugin_cookies->view_type) ? $plugin_cookies->view_type : 2;
		        array_push
		        (
                    $folder_items,
                    array
			        (
                        PluginRegularFolderItem::media_url => $m_url,
	                    PluginRegularFolderItem::caption => $caption,
                	    PluginRegularFolderItem::view_item_params => array
                        	    (
                                    ViewItemParams::icon_width => ($view_type == 1 ? 190 : ($view_type == 2 ? 150 : 50)),
                                    ViewItemParams::icon_height => ($view_type == 1 ? 290 : ($view_type == 2 ? 200 : 50)),
                                    ViewItemParams::icon_path => ($view_type != 3 ? 'gui_skin://large_icons/movie.aai' : 'gui_skin://small_icons/movie.aai'),
                		            ViewItemParams::item_detailed_icon_path => 'missing://'
                        		)
			        )
		        );
                ++$total_num;
                
                $this->fav_build_in_progess = false;
            }
            else
            {
	          //  hd_print("--> get_regular_folder_items(): subcategory_ref: items count: " . count($videos));
                if ($media_url === 'main_menu:fav')
                    $this->fav_build_in_progess = true;
                else
                    $this->fav_build_in_progess = false;

                $more_items_available = 1;

                foreach ($videos as $v)
                {
	                HD::decode_user_data($v, $media_str, $user_data);
		    
		            $genre = ' ';
		            if (HD::has_attribute($user_data, 'genre'))
			            $genre = $user_data->genre;

                    $duration = ' ';
		            if (HD::has_attribute($user_data, 'duration'))
			            $duration = $user_data->duration;

                    if (is_interface_language_russian_or_similar())
                    {
		                $genre_caption = 'Жанр';
                        $duration_caption = 'Продолжительность';
                    }
                    else
                    {
		                $genre_caption = 'Genre';
                        $duration_caption = 'Duration';
                    }

                    $info = '111';
                    $title = ' ';
                    // Workaround for a bug in Shell.
                    if (HD::has_attribute($user_data, 'season_title'))
                        $title = str_replace('^', '', $user_data->season_title);
                        $titls = wordwrap($title, 35, "||");
						$season_page = $user_data->season_page;
                    $view_type = isset($plugin_cookies->view_type) ? $plugin_cookies->view_type : 2;
					$info = $titls . ' || ' .  $season_page . ' || ';
                   
		            $video_id = '1';
		            $n_series = 1;
		            $ndx_series = 0;
                    $is_has_new = false;

		            if (HD::has_attribute($user_data, 'video_id'))
	                    $video_id = $user_data->video_id;

 		            if (HD::has_attribute($user_data, 'n_series'))
	                    $n_series = $user_data->n_series;

                    if (HD::has_attribute($user_data, 'is_has_new'))
                        $is_has_new = $user_data->is_has_new;

                    if ($view_type == 1)
                    {
                            array_push(
                                $folder_items, 
                                array(
                                    PluginRegularFolderItem::media_url =>
                                        HD::encode_user_data(
                                            array(
				                	            'video_id' => $video_id,
                                                'n_series' => $n_series,
                                                'season_title' => $title
				                            )
			                        ),
                                    PluginRegularFolderItem::caption => ($is_has_new ? '[NEW] ' . $title : $title),
                                    PluginRegularFolderItem::view_item_params => array(
                                        ViewItemParams::icon_path => $this->get_preview_url($user_data),
                                        ViewItemParams::item_paint_caption => false,
                                        ViewItemParams::item_caption_wrap_enabled => TRUE,
                                        ViewItemParams::item_detailed_info => $info
                                    )
                                ));
                    }
                    else if ($view_type == 2)
                    {
                            array_push(
                                $folder_items, 
                                array(
                                    PluginRegularFolderItem::media_url =>
                                        HD::encode_user_data(
                                            array(
                                                'video_id' => $video_id,
                                                'n_series' => $n_series,
                                            )
                                    ),
                                    PluginRegularFolderItem::caption => ($is_has_new ? '[NEW] ' . $title : $title),
                                    PluginRegularFolderItem::view_item_params => array(
                                        ViewItemParams::icon_path => $this->get_preview_url($user_data),
                                        ViewItemParams::item_detailed_info => $info
                                    )
                                ));
                    }
                    else
                    {
                        array_push(
                            $folder_items, 
                            array(
                                PluginRegularFolderItem::media_url =>
                                    HD::encode_user_data(
                                        array(
                                            'video_id' => $video_id,
                                            'n_series' => $n_series
                                        )
                                ),
                                PluginRegularFolderItem::caption => ($is_has_new ? '[NEW] ' . $title : $title),
                                PluginRegularFolderItem::view_item_params => array(
                                    ViewItemParams::item_detailed_icon_path => $this->get_preview_url($user_data),
                                    ViewItemParams::item_detailed_info => $info
                                )
                            ));
                    }
                }

                array_push(
                    $folder_items, 
                    array(
                        PluginRegularFolderItem::media_url => 'dummy',
                        PluginRegularFolderItem::caption =>
                            (is_interface_language_russian_or_similar() ? 'Загрузка...' : 'Loading...'),
                        PluginRegularFolderItem::view_item_params => array(
                            ViewItemParams::icon_width => ($view_type == 1 ? 63 : ($view_type == 2 ? 150 : 50)),
                            ViewItemParams::icon_height => (1 ? 63 : ($view_type == 2 ? 150 : 50)),
                            ViewItemParams::icon_path => ($view_type != 3 ? 'gui_skin://large_icons/movie.aai' : 'gui_skin://small_icons/movie.aai'),
                            ViewItemParams::item_detailed_icon_path => 'missing://'
                        )
                    ));

                ++$total_num;
            }
        }
		else if
        (
			HD::has_attribute($user_data, 'vid_similar')
        )
        {
			$vid_similar = $user_data->vid_similar;
			
			$items = array();
			$links = explode("<a href='", $vid_similar);
			unset($links[0]);
			$links = array_values($links);
			foreach($links as $link)
			{	
				$season_ref = strstr($link, "'", true);
							
				$tmp = explode("url('", $link);
				if (count($tmp) > 1)
				$season_image = strstr($tmp[1], "'", true);
				$season_image = str_replace('/6/', '/1/', $season_image);
				$season_image = str_replace('/9/', '/1/', $season_image);
				
				$tmp = explode("<span class='m-poster-new__full_title'>", $link);
				if (count($tmp) < 2)
					continue;
				$season_title = str_remove_spec(strstr($tmp[1], '</span>', true));
				$season_title = str_replace('<p>', ' ', $season_title);
				$season_title = strip_tags($season_title);				
				array_push
				(
					$items,
					HD::encode_user_data
					(
						array
						(
							'video_id' => $season_ref,
							'season_title' => $season_title,
							'season_ref' => $season_ref,
							'season_image' => $season_image,
							'is_has_new' => false
						)
					)
				);
				++$this->m_items_count;
			}
			
			$videos = $items;
            $total_num = 5;
	            
           if ($videos === false)
            {
		  
                $caption = (is_interface_language_russian_or_similar() ? 'Больше ничего не найдено' : 'No more items found');
                $m_url = 'dummy';
                if (HD::has_attribute($user_data, 'fav'))
                {
                    $caption = (is_interface_language_russian_or_similar() ? 'Hе найдено' : 'Not found');
                    $m_url = $media_url;
                }
		        $more_items_available = 0;
                
                if ($media_url === 'main_menu:fav')
                {
                    $folder->set_expired();
                }

                $view_type = isset($plugin_cookies->view_type) ? $plugin_cookies->view_type : 2;
		        array_push
		        (
                    $folder_items,
                    array
			        (
                        PluginRegularFolderItem::media_url => $m_url,
	                    PluginRegularFolderItem::caption => $caption,
                	    PluginRegularFolderItem::view_item_params => array
                        	    (
                                    ViewItemParams::icon_width => ($view_type == 1 ? 190 : ($view_type == 2 ? 150 : 50)),
                                    ViewItemParams::icon_height => ($view_type == 1 ? 290 : ($view_type == 2 ? 200 : 50)),
                                    ViewItemParams::icon_path => ($view_type != 3 ? 'gui_skin://large_icons/movie.aai' : 'gui_skin://small_icons/movie.aai'),
                		            ViewItemParams::item_detailed_icon_path => 'missing://'
                        		)
			        )
		        );
                ++$total_num;
                
                $this->fav_build_in_progess = false;
            }
            else
            {
	          //  hd_print("--> get_regular_folder_items(): subcategory_ref: items count: " . count($videos));
                if ($media_url === 'main_menu:fav')
                    $this->fav_build_in_progess = true;
                else
                    $this->fav_build_in_progess = false;

                $more_items_available = 0;

                foreach ($videos as $v)
                {
	                HD::decode_user_data($v, $media_str, $user_data);
		    
		            $genre = ' ';
		            if (HD::has_attribute($user_data, 'genre'))
			            $genre = $user_data->genre;

                    $duration = ' ';
		            if (HD::has_attribute($user_data, 'duration'))
			            $duration = $user_data->duration;

                    if (is_interface_language_russian_or_similar())
                    {
		                $genre_caption = 'Жанр';
                        $duration_caption = 'Продолжительность';
                    }
                    else
                    {
		                $genre_caption = 'Genre';
                        $duration_caption = 'Duration';
                    }

                    $info = '';
                    $title = ' ';
                    // Workaround for a bug in Shell.
                    if (HD::has_attribute($user_data, 'season_title'))
                        $title = str_replace('^', '', $user_data->season_title);
                                
                    $view_type = isset($plugin_cookies->view_type) ? $plugin_cookies->view_type : 2;

                   
		            $video_id = '1';
		            $n_series = 1;
		            $ndx_series = 0;
                    $is_has_new = false;

		            if (HD::has_attribute($user_data, 'video_id'))
	                    $video_id = $user_data->video_id;

 		            if (HD::has_attribute($user_data, 'n_series'))
	                    $n_series = $user_data->n_series;

                    if (HD::has_attribute($user_data, 'is_has_new'))
                        $is_has_new = $user_data->is_has_new;

                    if ($view_type == 1)
                    {
                            array_push(
                                $folder_items, 
                                array(
                                    PluginRegularFolderItem::media_url =>
                                        HD::encode_user_data(
                                            array(
				                	            'video_id' => $video_id,
                                                'n_series' => $n_series,
                                                'season_title' => $title
				                            )
			                        ),
                                    PluginRegularFolderItem::caption => ($is_has_new ? '[NEW] ' . $title : $title),
                                    PluginRegularFolderItem::view_item_params => array(
                                        ViewItemParams::icon_path => $this->get_preview_url($user_data),
                                        ViewItemParams::item_paint_caption => false,
                                        ViewItemParams::item_caption_wrap_enabled => TRUE,
            //                            ViewItemParams::item_detailed_info => $info
                                    )
                                ));
                    }
                    else if ($view_type == 2)
                    {
                            array_push(
                                $folder_items, 
                                array(
                                    PluginRegularFolderItem::media_url =>
                                        HD::encode_user_data(
                                            array(
                                                'video_id' => $video_id,
                                                'n_series' => $n_series,
                                            )
                                    ),
                                    PluginRegularFolderItem::caption => ($is_has_new ? '[NEW] ' . $title : $title),
                                    PluginRegularFolderItem::view_item_params => array(
                                        ViewItemParams::icon_path => $this->get_preview_url($user_data),
                                        ViewItemParams::item_detailed_info => $info
                                    )
                                ));
                    }
                    else
                    {
                        array_push(
                            $folder_items, 
                            array(
                                PluginRegularFolderItem::media_url =>
                                    HD::encode_user_data(
                                        array(
                                            'video_id' => $video_id,
                                            'n_series' => $n_series
                                        )
                                ),
                                PluginRegularFolderItem::caption => ($is_has_new ? '[NEW] ' . $title : $title),
                                PluginRegularFolderItem::view_item_params => array(
                                    ViewItemParams::item_detailed_icon_path => $this->get_preview_url($user_data),
                                    ViewItemParams::item_detailed_info => $info
                                )
                            ));
                    }
                }

                array_push(
                    $folder_items, 
                    array(
                        PluginRegularFolderItem::media_url => 'dummy',
                        PluginRegularFolderItem::caption =>
                            (is_interface_language_russian_or_similar() ? 'Загрузка...' : 'Loading...'),
                        PluginRegularFolderItem::view_item_params => array(
                            ViewItemParams::icon_width => ($view_type == 1 ? 63 : ($view_type == 2 ? 150 : 50)),
                            ViewItemParams::icon_height => (1 ? 63 : ($view_type == 2 ? 150 : 50)),
                            ViewItemParams::icon_path => ($view_type != 3 ? 'gui_skin://large_icons/movie.aai' : 'gui_skin://small_icons/movie.aai'),
                            ViewItemParams::item_detailed_icon_path => 'missing://'
                        )
                    ));

                ++$total_num;
            }
        }
        else if (HD::has_attribute($user_data, 'video_id'))
        {
	        $more_items_available = 0;
            $c = VideoCache::get($user_data->video_id);
            HD::decode_user_data($c, $media_str, $user_data);
	        $vod_info = $this->get_vod_info($media_url, &$plugin_cookies);
	    
	        $series = $vod_info[PluginVodInfo::series];

	        $i = 0;
	        foreach($series as $s)
	        {
		        $info = '';
                $caption = '';
                $isnew = false;
                $video_id = $user_data->video_id;
                if (isset($plugin_cookies->$video_id))
                {
				$new_view = isset($plugin_cookies->new_view) ? $plugin_cookies->new_view : 1;
				if (!$new_view == 0)
						{
						   if (!ListUtil::is_in_list($plugin_cookies->$video_id, $s[PluginVodSeriesInfo::playback_url]))
								$caption = '[NEW]';
						}
                }
				//{ Viewed items
                $is_viewed = false;
                if (!$isnew)
                {
                    $vs = $video_id . "." . $s[PluginVodSeriesInfo::playback_url];
                    if (isset($plugin_cookies->$vs))
                    {
                        $t = explode('|', $plugin_cookies->$vs);
                        if (intval($t[0]) != -1)
                            $caption = '[ПРОСМ ' . HD::format_duration(intval($t[0]) * 1) . '/' . HD::format_duration(intval($t[1]) * 1) . '] ';
                        else
                            $caption = '[ПРОСМ] ';
                    }
                    $vs = $video_id . ".viewed";
                    if (isset($plugin_cookies->$vs))
                    {
                        if (ListUtil::is_in_list($plugin_cookies->$vs, $s[PluginVodSeriesInfo::playback_url]))
                            $caption = '[ПРОСМ] ';
                    }
                }
                //} Viewed items

                array_push
                (
                    $folder_items, 
                    array
                    (
                        PluginRegularFolderItem::media_url =>
                            HD::encode_user_data
                            (
                                array
                                (
					                'video_id' => $user_data->video_id,
					                'ndx_series' => $i,
                                    'url_series' => $s[PluginVodSeriesInfo::playback_url]
				                )
  			                ),
                            //{ Viewed items
                            PluginRegularFolderItem::caption => $caption . $s[PluginVodSeriesInfo::name],
                            //} Viewed items
                            PluginRegularFolderItem::view_item_params => array(
                                    ViewItemParams::icon_path => 'gui_skin://small_icons/movie.aai',
                                    ViewItemParams::item_detailed_info => $info
                            )
                    )
		        );
		        ++$i;
	        }

	        $total_num = $i;
        }
    

        return array(
            PluginRegularFolderRange::total => $total_num,
            PluginRegularFolderRange::count => count($folder_items),
            PluginRegularFolderRange::more_items_available => $more_items_available,
            PluginRegularFolderRange::from_ndx => $from_ndx,
            PluginRegularFolderRange::items => $folder_items
        );
    }

    ///////////////////////////////////////////////////////////////////////
    private function get_episode_url($episode_ref)
    {
		hd_print("  episode_ref => $episode_ref");
		if (preg_match("/flv/i",$episode_ref))
        {
			$url = 'http://ts://127.0.0.1:81/cgi-bin/flv.sh?'. $episode_ref;
		}
		elseif (preg_match("/.mp4/i",$episode_ref))
        {
			$url = str_replace("http://", "http://mp4://", $episode_ref);
		}
		else
		{
			$url = $episode_ref;
		}
      return($url);
    }
    
    protected function get_movie_actions($initial_series_ndx, &$plugin_cookies)
    {
        UserInputHandlerRegistry::get_instance()->register_handler($this);

        $actions[GUI_EVENT_PLAYBACK_STOP] =
            UserInputHandlerRegistry::create_action($this, 'postroll_watched');
            
        $play_timer = isset($plugin_cookies->play_timer) ? $plugin_cookies->play_timer : 5000;
        if ($play_timer > 0)
        {
            $actions[GUI_EVENT_TIMER] =
                UserInputHandlerRegistry::create_action($this, 'content_timer');
        }
        
        return $actions;        
    }

    protected function get_movie_timer($initial_series_ndx, &$plugin_cookies)
    {
        $play_timer = isset($plugin_cookies->play_timer) ? $plugin_cookies->play_timer : 5000;
        if ($play_timer == 0)
            return false;
        
        return array(GuiTimerDef::delay_ms => $play_timer);        
    }

    public function get_vod_info($media_url, &$plugin_cookies)
    {
        //hd_print("--> get_vod_info: started...");
 
        if ($media_url === 'dummy')
            return array();

        HD::decode_user_data($media_url, $media_str, $user_data);
        foreach ($user_data as $key => $value)
           // hd_print("  $key => $value");

        $season_ref = "";
        if (!HD::has_attribute($user_data, 'season_ref'))
        {
            if (!HD::has_attribute($user_data, 'video_id'))
                throw new Exception('No season_ref or video_id');
            
            $season_ref = $user_data->video_id;
        }
        else
        {
            $season_ref = $user_data->season_ref;
        }

	    $ndx_series = 0;
        if (HD::has_attribute($user_data, 'ndx_series'))
		    $ndx_series = $user_data->ndx_series;
            
        $title = '';
        if (HD::has_attribute($user_data, 'season_title'))
            $title = $user_data->season_title;

        $descr = '';
        $icon_path = '';
            
	    $series = array();
	    
        $series_urls = $this->parse_season($season_ref, $plugin_cookies);
        
        $video_quality = isset($plugin_cookies->video_quality) ? $plugin_cookies->video_quality : 1;

        if (isset($this->series_list))
            unset($this->series_list);
        if (isset($this->playback_info))
            unset($this->playback_info);

        $this->series_idx = -1;
        $this->series_list = array();
        $i = 0;
	    foreach ($series_urls as $s)
	    {
	        HD::decode_user_data($s, $media_str, $user_data);
            if ($i === 0)
            {
                if (HD::has_attribute($user_data, 'season_title'))
                    $title = $user_data->season_title;

                if (HD::has_attribute($user_data, 'description'))
                    $descr = $user_data->description;
                if (HD::has_attribute($user_data, 'poster_url'))
                    $icon_path = $user_data->poster_url;
            }
            
            $url = $user_data->episode_ref . "&season_ref=$season_ref";
		    //hd_print("url-+-+->>>: $url");
		    if (strlen($user_data->episode_ref) > 0)
		    {
                array_push($this->series_list, $user_data->episode_ref);
			    array_push
			    (
				    $series,
		            array
        		    (
                		PluginVodSeriesInfo::name => $user_data->episode_title,
	                	PluginVodSeriesInfo::playback_url => $url,
		                PluginVodSeriesInfo::playback_url_is_stream_url => false
        		    )
			    );
                ++$i;
		    }
	    }
	
        $buf_time = isset($plugin_cookies->buf_time) ? $plugin_cookies->buf_time : 0;
        //hd_print('--->>> Buffer time: ' . $buf_time);
        mb_internal_encoding("UTF-8");
        
        $main_key = $season_ref . '.' . $this->series_list[$ndx_series];

        //hd_print('--->>> main_key: '.$main_key);
        
        $initial_position_ms = 0;
        if (isset($plugin_cookies->$main_key))
        {
            $t = explode('|', $plugin_cookies->$main_key);
            $pos = intval($t[0]);
            if ($pos < 0)
                $pos = 0;
            if ($pos + 5 >= intval($t[1]))
                $pos = 0;
            $initial_position_ms = $pos * 1000;
        }
        $timer = $this->get_movie_timer($ndx_series, $plugin_cookies);
        $this->series_idx = $ndx_series;
        if (!$timer)
        {
            return  array
                    (
                        PluginVodInfo::id => $season_ref,
                        PluginVodInfo::name => $title,
                        PluginVodInfo::description => $descr,
                        PluginVodInfo::poster_url => $icon_path,
                        PluginVodInfo::initial_series_ndx => $ndx_series,
                        PluginVodInfo::buffering_ms => $buf_time,
                        PluginVodInfo::series => $series,
                        PluginVodInfo::advert_mode => false,
                        PluginVodInfo::actions => $this->get_movie_actions($ndx_series, $plugin_cookies),
                        PluginVodInfo::initial_position_ms => $initial_position_ms,
                    );
        }
        else
        {
            return  array
                    (
                        PluginVodInfo::id => $season_ref,
                        PluginVodInfo::name => $title,
                        PluginVodInfo::description => $descr,
                        PluginVodInfo::poster_url => $icon_path,
                        PluginVodInfo::initial_series_ndx => $ndx_series,
                        PluginVodInfo::buffering_ms => $buf_time,
                        PluginVodInfo::series => $series,
                        PluginVodInfo::advert_mode => false,
                        PluginVodInfo::actions => $this->get_movie_actions($ndx_series, $plugin_cookies),
                        PluginVodInfo::initial_position_ms => $initial_position_ms,
                        PluginVodInfo::timer => $timer,
                    );
        }
    }
 /////////////////////////////////////////////////////////////////////////////   
	
	public function get_vod_stream_url($media_url, &$plugin_cookies)
    {
        $tmp = explode('&season_ref=', $media_url);
        $media_url = $tmp[0];
        $season_ref = $tmp[1];
		return $this->get_episode_url($media_url);				
    }

    ///////////////////////////////////////////////////////////////////////

    private function get_loading_items()
    {
        $view_type = isset($plugin_cookies->view_type) ? $plugin_cookies->view_type : 2;
        $items = array(
            array
            (
                PluginRegularFolderItem::caption =>
                    (is_interface_language_russian_or_similar() ? 'Загрузка...' : 'Loading...'),
                PluginRegularFolderItem::view_item_params => array(
                    ViewItemParams::item_paint_icon => true,
                    ViewItemParams::item_layout => HALIGN_LEFT,
                    ViewItemParams::icon_valign => VALIGN_CENTER,
                    ViewItemParams::icon_dx => 10,
                    ViewItemParams::icon_width => ($view_type == 1 ? 63 : ($view_type == 2 ? 150 : 50)),
                    ViewItemParams::icon_height => (1 ? 63 : ($view_type == 2 ? 150 : 50)),
                    ViewItemParams::icon_path => ($view_type != 3 ? 'gui_skin://large_icons/movie.aai' : 'gui_skin://small_icons/movie.aai'),
                    ViewItemParams::item_caption_width => 950,
                    ViewItemParams::item_detailed_icon_path => 'missing://'
                ),
                PluginRegularFolderItem::media_url => 'dummy'
            )
        );

        return array(
            PluginRegularFolderRange::total => count($items),
            PluginRegularFolderRange::count => count($items),
            PluginRegularFolderRange::more_items_available => 1,
            PluginRegularFolderRange::from_ndx => 0,
            PluginRegularFolderRange::items => $items
        );
    }

    ///////////////////////////////////////////////////////////////////////
    private function FSUA_main_video($media_url, &$plugin_cookies)
    {
        set_time_limit(0);
        $api = HOST_API_URL;
        $api = $api . "/video/";
		
		$video_sorting = isset($plugin_cookies->video_sorting) ? $plugin_cookies->video_sorting : 1;
		//hd_print('video_sorting--->' . $video_sorting);
        
		if ($video_sorting == "0")
			{
			$sort_path ='/?sort=rating';
			}
		elseif ($video_sorting == "1")
			{
			 $sort_path ='/?sort=new';
			}
		elseif ($video_sorting == "2")
			{
			$sort_path ='/?sort=year';
			}
        //hd_print("sort_path--->>> : $sort_path");
		
		$video_quality = isset($plugin_cookies->video_quality) ? $plugin_cookies->video_quality : 1;
		//hd_print('video_quality--->' . $video_quality);
        
		if ($video_quality == "0")
			{
			$mood_path ='_comedy';
			}
		elseif ($video_quality == "1")
			{
			 $mood_path ='';
			}
		elseif ($video_quality == "2")
			{
			$mood_path ='_horror';
			}
		elseif ($video_quality == "3")
			{
			$mood_path ='_action';
			}
		elseif ($video_quality == "4")
			{
			$mood_path ='_soul';
			}
        //hd_print("mood_path--->>> : $mood_path");
		
		$video_resolution = isset($plugin_cookies->video_resolution) ? $plugin_cookies->video_resolution : 1;
		//hd_print('video_quality--->' . $video_production);
        
		if ($video_resolution == "0")
			{
			$resolution_path ='_hight';
			}
		elseif ($video_resolution == "1")
			{
			 $resolution_path ='';
			}
		elseif ($video_resolution == "2")
			{
			$resolution_path ='_good';
			}
		elseif ($video_resolution == "3")
			{
			$resolution_path ='_middle';
			}
        //hd_print("resolution_path--->>> : $resolution_path");
		
		$video_language = isset($plugin_cookies->video_language) ? $plugin_cookies->video_language : 1;
		//hd_print('video_quality--->' . $video_production);
        
		if ($video_language == "0")
			{
			$language_path ='_russion';
			}
		elseif ($video_language == "1")
			{
			 $language_path ='';
			}
		elseif ($video_language == "2")
			{
			$language_path ='_ukrainian';
			}
		
        //hd_print("language_path--->>> : $language_path");
		
		$main_menu = file_get_contents(dirname(__FILE__).'/data/fsua_video_1.xml');
        $videos = explode('<a href="/video/', $main_menu);
        // exclude before first <li
        unset($videos[0]);
        $videos = array_values($videos);
        $items = array();
    
        //hd_print('Before parsing: Categories count: ' . count($videos) + 1);

        // iterate through categories
        foreach($videos as $video)
        {
            // get category view reference

            $category_ref = strstr($video, '/"', true);
            $category_ref = str_replace($api, '', $category_ref);
			//hd_print("--->>> category_ref: $category_ref");
		
		if ($category_ref == "clips")
			{
			$mood_path = str_replace("_comedy", "_pop", $mood_path);
			$mood_path = str_replace("_horror", "_hard", $mood_path);
			$mood_path = str_replace("_action", "_dance", $mood_path);
			$language_path = str_replace("_russion", "", $language_path);
			$language_path = str_replace("_ukrainian", "", $language_path);
			}
		if (preg_match("/\/fl/i",$category_ref))
			{
			$category_ref = '/video/'.$category_ref . $mood_path . $language_path . $resolution_path . $sort_path .'&view=list';                     
			}
		else
			{
			$category_ref = '/video/'.$category_ref . '/fl' . $mood_path . $language_path . $resolution_path . $sort_path .'&view=list';
			}
        $category_ref = str_replace("=,", "=", $category_ref);
		$category_ref = str_replace("//", "/", $category_ref);
		$category_ref = str_replace("/fl/", "/", $category_ref);
			
		if (preg_match("/video\/ldr/i",$category_ref))
			{
			$category_ref = str_replace("/fl", "", $category_ref);
			$category_ref = str_replace("_comedy", "", $category_ref);
			$category_ref = str_replace("_horror", "", $category_ref);
			$category_ref = str_replace("_action", "", $category_ref);
			$category_ref = str_replace("_soul", "", $category_ref);
			$category_ref = str_replace("_hight", "", $category_ref);
			$category_ref = str_replace("_good", "", $category_ref);
			$category_ref = str_replace("_middle", "", $category_ref);
			$category_ref = str_replace("_russion", "", $category_ref);
			$category_ref = str_replace("_ukrainian", "", $category_ref);
			}
			
			//hd_print("--->>> category_ref: $category_ref");

            // get category title 
            $tmp = explode('" >', $video);
            $category_name = strstr($tmp[1], '</a>', true);
			//hd_print("--->>> category_name: $category_name");
        
            // push category in to categories list
            array_push
            (
                $items,
                array
                (
                    PluginRegularFolderItem::caption => $category_name,
                    PluginRegularFolderItem::view_item_params => self::$catalog_vip,
                    PluginRegularFolderItem::media_url =>
                            HD::encode_user_data
                            (
                                array
                                (
                                    'subcategory_ref' => $category_ref,
                                )
                            )
                )
            );
        }

        //hd_print('After parsing: Categories count: ' . count($items));
        
        return $items;
    }
	private function FSUA_main_brbfav($media_url, &$plugin_cookies)
    {
        set_time_limit(0);
        $api = HOST_API_URL;
		
        //hd_print("language_path--->>> : $language_path");
		
		$main_menu = file_get_contents(dirname(__FILE__).'/data/fsua_fav.xml');
        $videos = explode('<a href="', $main_menu);
        // exclude before first <li
        unset($videos[0]);
        $videos = array_values($videos);
        $items = array();
    
        //hd_print('Before parsing: Categories count: ' . count($videos) + 1);

        // iterate through categories
        foreach($videos as $video)
        {
            // get category view reference
            $category_ref = strstr($video, '"', true);
			//hd_print("--->>> category_ref: $category_ref");
			
            // get category title 
            $tmp = explode('" >', $video);
            $category_name = strstr($tmp[1], '</a>', true);
			//hd_print("--->>> category_name: $category_name");
        
            // push category in to categories list
            array_push
            (
                $items,
                array
                (
                    PluginRegularFolderItem::caption => $category_name,
                    PluginRegularFolderItem::view_item_params => self::$catalog_vip,
                    PluginRegularFolderItem::media_url =>
                            HD::encode_user_data
                            (
                                array
                                (
                                    'subcategory_ref' => $category_ref,
                                )
                            )
                )
            );
        }

        //hd_print('After parsing: Categories count: ' . count($items));
        
        return $items;
    }
	 ///////////////////////////////////////////////////////////////////////
	private function FSUA_main_genre($media_url, &$plugin_cookies)
    {
        set_time_limit(0);
        $api = HOST_API_URL;
        $api = $api . "/video/";
		
		$video_sorting = isset($plugin_cookies->video_sorting) ? $plugin_cookies->video_sorting : 1;
		//hd_print('video_sorting--->' . $video_sorting);
        
		if ($video_sorting == "0")
			{
			$sort_path ='/?sort=rating';
			}
		elseif ($video_sorting == "1")
			{
			 $sort_path ='/?sort=new';
			}
		elseif ($video_sorting == "2")
			{
			$sort_path ='/?sort=year';
			}
        //hd_print("sort_path--->>> : $sort_path");
		
				
		$video_production = isset($plugin_cookies->video_production) ? $plugin_cookies->video_production : 1;
		//hd_print('video_quality--->' . $video_production);

		$main_menu = file_get_contents(dirname(__FILE__).'/data/fsua_video_2.xml');
        $videos = explode('<a href="/video/', $main_menu);
        // exclude before first <li
		unset($videos[0]);
		if ($video_production == "1")
			{
			unset($videos[1]);
			unset($videos[2]);
			unset($videos[55]);
			unset($videos[56]);
			}
		elseif ($video_production == "2")
			{
			unset($videos[1]);
			unset($videos[2]);
			unset($videos[55]);
			unset($videos[56]);
			}
        
        $videos = array_values($videos);

        $items = array();
    
        //hd_print('Before parsing: Categories count: ' . count($videos) + 1);

        // iterate through categories
        foreach($videos as $video)
        {
            // get category view reference

            $category_ref = strstr($video, '/"', true);
            $category_ref = str_replace($api, '', $category_ref);
			$category_ref = '/video/'.$category_ref . $sort_path .'&view=list';                     
			//hd_print("--->>> category_ref: $category_ref");

            // get category title 
            $tmp = explode('" >', $video);
            $category_name = strstr($tmp[1], '</a>', true);
            //hd_print("--->>> category_name: $category_name");
        
            // push category in to categories list
            array_push
            (
                $items,
                array
                (
                    PluginRegularFolderItem::caption => $category_name,
                    PluginRegularFolderItem::view_item_params => self::$catalog_vip,
                    PluginRegularFolderItem::media_url =>
                            HD::encode_user_data
                            (
                                array
                                (
                                    'subcategory_ref' => $category_ref,
                                )
                            )
                )
            );
        }

        //hd_print('After parsing: Categories count: ' . count($items));
        
        return $items;
    }
	 /////////////////////////////////////////////////////////////////////// 
    private function FSUA_main_audio($media_url, &$plugin_cookies)
    {
        set_time_limit(0);
        $api = HOST_API_URL;
        
        $api = $api . "/audio/";
		
		$video_sorting = isset($plugin_cookies->video_sorting) ? $plugin_cookies->video_sorting : 1;
		//hd_print('video_sorting--->' . $video_sorting);
        
		if ($video_sorting == "0")
			{
			$sort_path ='/?sort=rating';
			}
		elseif ($video_sorting == "1")
			{
			 $sort_path ='/?sort=new';
			}
		elseif ($video_sorting == "2")
			{
			$sort_path ='/?sort=year';
			}
        //hd_print("sort_path--->>> : $sort_path");
		
		$video_quality = isset($plugin_cookies->video_quality) ? $plugin_cookies->video_quality : 1;
		//hd_print('video_quality--->' . $video_quality);
        
		if ($video_quality == "0")
			{
			$mood_path ='_pop';
			}
		elseif ($video_quality == "1")
			{
			 $mood_path ='';
			}
		elseif ($video_quality == "2")
			{
			$mood_path ='_hard';
			}
		elseif ($video_quality == "3")
			{
			$mood_path ='_dance';
			}
		elseif ($video_quality == "4")
			{
			$mood_path ='_soul';
			}
		//hd_print("mood_path--->>> : $mood_path");
		
		// download main video page
        $doc = HD::http_get_document($api);
    
        // get list of categories
        
		$main_menu_block2 = file_get_contents(dirname(__FILE__).'/data/fsua_audio.xml');
		$main_menu_block = $main_menu_block2;
        //hd_print("--->>> main_menu_block: $main_menu_block");
        
        $videos = explode('<a href="/audio/', $main_menu_block);
        // exclude before first <li
        unset($videos[0]);
        $videos = array_values($videos);

        $items = array();
    

        //hd_print('Before parsing: Categories count: ' . count($videos) + 1);

        // iterate through categories
        foreach($videos as $video)
        {
            // get category view reference

            $category_ref = strstr($video, '/"', true);
            $category_ref = str_replace($api, '', $category_ref);
			//hd_print("--->>> category_ref: $category_ref");
						
			$category_ref = '/audio/'.$category_ref . '/fl' . $mood_path . $sort_path .'&view=list';
            $category_ref = str_replace("=,", "=", $category_ref);
			$category_ref = str_replace("//", "/", $category_ref);
			$category_ref = str_replace("/fl/", "/", $category_ref);
			
			if (preg_match("/audio\/ldr/i",$category_ref))
				{
					$category_ref = str_replace("/fl", "", $category_ref);
					$category_ref = str_replace("_pop", "", $category_ref);
					$category_ref = str_replace("_hard", "", $category_ref);
					$category_ref = str_replace("_dance", "", $category_ref);
					$category_ref = str_replace("_soul", "", $category_ref);
										
				}
			if (preg_match("/music_genre&search/i",$category_ref))
				{
					$category_ref = str_replace("/fl", "", $category_ref);
					$category_ref = str_replace("_pop", "", $category_ref);
					$category_ref = str_replace("_hard", "", $category_ref);
					$category_ref = str_replace("_dance", "", $category_ref);
					$category_ref = str_replace("_soul", "", $category_ref);
										
				}
			//hd_print("--->>> category_ref: $category_ref");
			//$category_ref = str_replace('++/?', '&', $category_ref);
            // get category title 
            $tmp = explode('" >', $video);
            $category_name = strstr($tmp[1], '</a>', true);
            //hd_print("--->>> category_name: $category_name");
        
            // push category in to categories list
            array_push
            (
                $items,
                array
                (
                    PluginRegularFolderItem::caption => $category_name,
                    PluginRegularFolderItem::view_item_params => self::$catalog_vip,
                    PluginRegularFolderItem::media_url =>
                            HD::encode_user_data
                            (
                                array
                                (
                                    'subcategory_ref' => $category_ref,
                                )
                            )
                )
            );
        }

        //hd_print('After parsing: Categories count: ' . count($items));
        
        return $items;
    }
    ///////////////////////////////////////////////////////////////////////

    private function get_main_menu_items($media_url, &$plugin_cookies)
    {
        HD::decode_user_data($media_url,$url,$user_data);
        if (update_interface_language())
            FolderCache::clear();

        $skin = isset($plugin_cookies->skin) ? $plugin_cookies->skin : 'default';
	    if ($media_url === 'main_menu')
	    {
		$logon=0;
		$user_login = isset($plugin_cookies->user_login) ? $plugin_cookies->user_login : '';
		$user_pass = isset($plugin_cookies->user_pass) ? $plugin_cookies->user_pass : '';
		$login = degleef($user_login); 
		$pass = degleef($user_pass);
		$login = urlencode ($login);
		if (!$user_login == '')
		{
		if (!file_exists('/tmp/brb.cookies')) 
		{
		$url = 'http://brb.to/login.aspx';
		$referer = 'http://brb.to/';
		$post_data = 'remember=on&login=' .$login. '&passwd=' .$pass;
		$doc = HD::http_get_document_login($url, $post_data, $referer);
		if (preg_match("/logout.aspx/i",$doc))
		{
		$logon=1;
		}
		}
		else
		{
		$logon=1;
		}
		}
		if ($logon==0)	
		{
		$items = array
		    (
			    array
			    (
				    PluginRegularFolderItem::caption => 'Видео',
				    PluginRegularFolderItem::view_item_params => array(
				            ViewItemParams::icon_path => self::get_skin_path($plugin_cookies) . 'icons/video.png',
				            ViewItemParams::item_layout => HALIGN_CENTER,
				            ViewItemParams::icon_valign => VALIGN_CENTER,
					        ViewItemParams::item_paint_caption_within_icon => false,
                            ViewItemParams::item_paint_caption => ($skin == 'juniperus' ? true : false),
				            ViewItemParams::item_caption_font_size => FONT_SIZE_NORMAL,
				    ),
				    PluginRegularFolderItem::media_url => 'main_menu:videos'
                ),
				array
			    (
				    PluginRegularFolderItem::caption => 'По жанрам и годам',
				    PluginRegularFolderItem::view_item_params => array(
				            ViewItemParams::icon_path => self::get_skin_path($plugin_cookies) . 'icons/video.png',
				            ViewItemParams::item_layout => HALIGN_CENTER,
				            ViewItemParams::icon_valign => VALIGN_CENTER,
					        ViewItemParams::item_paint_caption_within_icon => false,
                            ViewItemParams::item_paint_caption => ($skin == 'juniperus' ? true : false),
				            ViewItemParams::item_caption_font_size => FONT_SIZE_NORMAL,
				    ),
				    PluginRegularFolderItem::media_url => 'main_menu:genres'
                ),
				array
			    (
				    PluginRegularFolderItem::caption => 'Аудио',
				    PluginRegularFolderItem::view_item_params => array(
				            ViewItemParams::icon_path => self::get_skin_path($plugin_cookies) . 'icons/audio.png',
				            ViewItemParams::item_layout => HALIGN_CENTER,
				            ViewItemParams::icon_valign => VALIGN_CENTER,
					        ViewItemParams::item_paint_caption_within_icon => false,
                            ViewItemParams::item_paint_caption => ($skin == 'juniperus' ? true : false),
				            ViewItemParams::item_caption_font_size => FONT_SIZE_NORMAL,
				    ),
				    PluginRegularFolderItem::media_url => 'main_menu:audio'
                ),
			    array
			    (
				    PluginRegularFolderItem::caption => 'Избранное',
				    PluginRegularFolderItem::view_item_params => array(
				            ViewItemParams::icon_path => self::get_skin_path($plugin_cookies) . 'icons/favorites.png',
				            ViewItemParams::item_layout => HALIGN_CENTER,
				            ViewItemParams::icon_valign => VALIGN_CENTER,
					        ViewItemParams::item_paint_caption_within_icon => false,
                            ViewItemParams::item_paint_caption => ($skin == 'juniperus' ? true : false),
				            ViewItemParams::item_caption_font_size => FONT_SIZE_NORMAL,
				    ),
				    PluginRegularFolderItem::media_url => 'main_menu:fav'
                ),
				array
			    (
				    PluginRegularFolderItem::caption => 'Поиск',
				    PluginRegularFolderItem::view_item_params => array(
				            ViewItemParams::icon_path => self::get_skin_path($plugin_cookies) . 'icons/search.png',
				            ViewItemParams::item_layout => HALIGN_CENTER,
				            ViewItemParams::icon_valign => VALIGN_CENTER,
					        ViewItemParams::item_paint_caption_within_icon => false,
                            ViewItemParams::item_paint_caption => ($skin == 'juniperus' ? true : false),
				            ViewItemParams::item_caption_font_size => FONT_SIZE_NORMAL,
				    ),
				    PluginRegularFolderItem::media_url => 'main_menu:search'
                ),
				array
			    (
				    PluginRegularFolderItem::caption => 'Настройки',
				    PluginRegularFolderItem::view_item_params => array(
				            ViewItemParams::icon_path => self::get_skin_path($plugin_cookies) . 'icons/setup.png',
				            ViewItemParams::item_layout => HALIGN_CENTER,
				            ViewItemParams::icon_valign => VALIGN_CENTER,
					        ViewItemParams::item_paint_caption_within_icon => false,
                            ViewItemParams::item_paint_caption => ($skin == 'juniperus' ? true : false),
				            ViewItemParams::item_caption_font_size => FONT_SIZE_NORMAL,
				    ),
				    PluginRegularFolderItem::media_url => 'setup'
                ),
		    );
		}
		else
		{
		$items = array
		    (
			    array
			    (
				    PluginRegularFolderItem::caption => 'Видео',
				    PluginRegularFolderItem::view_item_params => array(
				            ViewItemParams::icon_path => self::get_skin_path($plugin_cookies) . 'icons/video.png',
				            ViewItemParams::item_layout => HALIGN_CENTER,
				            ViewItemParams::icon_valign => VALIGN_CENTER,
					        ViewItemParams::item_paint_caption_within_icon => false,
                            ViewItemParams::item_paint_caption => ($skin == 'juniperus' ? true : false),
				            ViewItemParams::item_caption_font_size => FONT_SIZE_NORMAL,
				    ),
				    PluginRegularFolderItem::media_url => 'main_menu:videos'
                ),
				array
			    (
				    PluginRegularFolderItem::caption => 'По жанрам и годам',
				    PluginRegularFolderItem::view_item_params => array(
				            ViewItemParams::icon_path => self::get_skin_path($plugin_cookies) . 'icons/video.png',
				            ViewItemParams::item_layout => HALIGN_CENTER,
				            ViewItemParams::icon_valign => VALIGN_CENTER,
					        ViewItemParams::item_paint_caption_within_icon => false,
                            ViewItemParams::item_paint_caption => ($skin == 'juniperus' ? true : false),
				            ViewItemParams::item_caption_font_size => FONT_SIZE_NORMAL,
				    ),
				    PluginRegularFolderItem::media_url => 'main_menu:genres'
                ),
				array
			    (
				    PluginRegularFolderItem::caption => 'Аудио',
				    PluginRegularFolderItem::view_item_params => array(
				            ViewItemParams::icon_path => self::get_skin_path($plugin_cookies) . 'icons/audio.png',
				            ViewItemParams::item_layout => HALIGN_CENTER,
				            ViewItemParams::icon_valign => VALIGN_CENTER,
					        ViewItemParams::item_paint_caption_within_icon => false,
                            ViewItemParams::item_paint_caption => ($skin == 'juniperus' ? true : false),
				            ViewItemParams::item_caption_font_size => FONT_SIZE_NORMAL,
				    ),
				    PluginRegularFolderItem::media_url => 'main_menu:audio'
                ),
			    array
			    (
				    PluginRegularFolderItem::caption => 'Избранное',
				    PluginRegularFolderItem::view_item_params => array(
				            ViewItemParams::icon_path => self::get_skin_path($plugin_cookies) . 'icons/favorites.png',
				            ViewItemParams::item_layout => HALIGN_CENTER,
				            ViewItemParams::icon_valign => VALIGN_CENTER,
					        ViewItemParams::item_paint_caption_within_icon => false,
                            ViewItemParams::item_paint_caption => ($skin == 'juniperus' ? true : false),
				            ViewItemParams::item_caption_font_size => FONT_SIZE_NORMAL,
				    ),
				    PluginRegularFolderItem::media_url => 'main_menu:fav'
                ),
				array
			    (
				    PluginRegularFolderItem::caption => 'Поиск',
				    PluginRegularFolderItem::view_item_params => array(
				            ViewItemParams::icon_path => self::get_skin_path($plugin_cookies) . 'icons/search.png',
				            ViewItemParams::item_layout => HALIGN_CENTER,
				            ViewItemParams::icon_valign => VALIGN_CENTER,
					        ViewItemParams::item_paint_caption_within_icon => false,
                            ViewItemParams::item_paint_caption => ($skin == 'juniperus' ? true : false),
				            ViewItemParams::item_caption_font_size => FONT_SIZE_NORMAL,
				    ),
				    PluginRegularFolderItem::media_url => 'main_menu:search'
                ),
				array
			    (
				    PluginRegularFolderItem::caption => 'Избранное brb.to',
				    PluginRegularFolderItem::view_item_params => array(
				            ViewItemParams::icon_path => self::get_skin_path($plugin_cookies) . 'icons/favoritesbrb.png',
				            ViewItemParams::item_layout => HALIGN_CENTER,
				            ViewItemParams::icon_valign => VALIGN_CENTER,
					        ViewItemParams::item_paint_caption_within_icon => false,
                            ViewItemParams::item_paint_caption => ($skin == 'juniperus' ? true : false),
				            ViewItemParams::item_caption_font_size => FONT_SIZE_NORMAL,
				    ),
				    PluginRegularFolderItem::media_url => 'main_menu:brbfav'
                ),
		    );
		}
	    }
        else if ($media_url === 'main_menu:search')
	    {
		    $items = array();
            array_push
            (
                $items,
                array
                (
                    PluginRegularFolderItem::caption => '[Новый запрос]',
                    PluginRegularFolderItem::view_item_params => self::$catalog_vip,
                    PluginRegularFolderItem::media_url =>
                        HD::encode_user_data
                        (
                            array('ser_feed' => '')
                        )
                )
            );

		    $search_items = isset($plugin_cookies->search_items) ? $plugin_cookies->search_items : '';
		    foreach (ListUtil::string_to_list($search_items) as $item)
		    {
			    if ($item !== "")
			    {
				    array_push
				    (
					    $items,
					    array
					    (
						    PluginRegularFolderItem::caption => (is_interface_language_russian_or_similar() ? "поиск: $item" : "search: $item"),
						    PluginRegularFolderItem::view_item_params => self::$catalog_vip,
						    PluginRegularFolderItem::media_url =>
							    HD::encode_user_data
							    (
								    array('ser_feed' => urlencode($item))
							    )
                        )
                    );
                }
            }
        }
        else if ($media_url === 'main_menu:fav')
	    {
		    $items = array();
		    $fav_items = isset($plugin_cookies->fav_items) ? $plugin_cookies->fav_items : '';
		    foreach (ListUtil::string_to_list($fav_items) as $item)
		    {
			    if ($item !== "")
			    {
				    $folderRange = $this->get_regular_folder_items(
							    HD::encode_user_data(array('fav' => $item)),
							    0,
							    $plugin_cookies
					   	    );

				    if (!$folderRange)
					    continue;
				    if ($folderRange[PluginRegularFolderRange::count] < 1)
					    continue;

				    array_push
				    (
					    $items,
					    $folderRange[PluginRegularFolderRange::items][0]
                    );
                }
            }
        }
	    elseif ($media_url === 'main_menu:videos')
	    {
            $items = $this->FSUA_main_video($media_url, $plugin_cookies);
	    }
		elseif ($media_url === 'main_menu:genres')
	    {
            $items = $this->FSUA_main_genre($media_url, $plugin_cookies);
	    }
		 elseif ($media_url === 'main_menu:audio')
	    {
            $items = $this->FSUA_main_audio($media_url, $plugin_cookies);
	    }
		 elseif ($media_url === 'main_menu:brbfav')
	    {
            $items = $this->FSUA_main_brbfav($media_url, $plugin_cookies);
	    }
	    return 	array
	    (
		        PluginRegularFolderRange::total => count($items),
		        PluginRegularFolderRange::count => count($items),
	            PluginRegularFolderRange::more_items_available => false,
		        PluginRegularFolderRange::from_ndx => 0,
		        PluginRegularFolderRange::items => $items
	    );
	}
}

///////////////////////////////////////////////////////////////////////////

DefaultDunePluginFw::$plugin_class_name = 'FSUAPlugin';

///////////////////////////////////////////////////////////////////////////
?>
